/***************************************************************************/
/** @file     addsetilocdlg.cpp
  * @brief    A dialog for adding and editing a %S@h location
  *           (implementation).
  *
  * @b begin      : Sat Oct 30 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * @todo Check for correct %S@h directory when clicking OK, or when the
  *       corresponding edit field loses focus.
  **************************************************************************/
#include <kapp.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kcolorbutton.h>
#include <kiconloader.h>

#include <qcheckbox.h>
#include <qtooltip.h>
#include <qlayout.h>
#include <qslider.h>
#include <qlineedit.h>

#include "addsetilocdlg.h"
#include "locationpage.h"
#include "clientpage.h"
#include "loggingpage.h"

/*------------------------------------------------------------------------ */
AddSetiLocDlg::AddSetiLocDlg(QWidget *parent, const char *name)
             : KDialogBase(IconList, QString::null, Ok|Cancel, Ok, parent, name)
{
QFrame*      frame;
QVBoxLayout* frameLayout;

resize(400, 300);
enableButtonOK(false);

frame = addPage(i18n("Location"),
                i18n("General location settings"),
                DesktopIcon("ksetiwatch", KIcon::SizeMedium));
frameLayout = new QVBoxLayout(frame, 0, 0);
m_locationPage = new LocationPageLayout(frame);
frameLayout->addWidget(m_locationPage);
// signals and slots connections
connect(m_locationPage->DirecButton, SIGNAL(clicked()), 
        this, SLOT(slotGetSetiDirectory()));
connect(m_locationPage->DescrEdit, SIGNAL(textChanged(const QString&)), 
        this, SLOT(slotHandleText(const QString&)));
connect(m_locationPage->DirecEdit, SIGNAL(textChanged(const QString&)), 
        this, SLOT(slotHandleText(const QString&)));
connect(m_locationPage->StopExeButton, SIGNAL(clicked()), 
        this, SLOT(slotGetStopExe()));
connect(m_locationPage->StartExeButton, SIGNAL(clicked()), 
        this, SLOT(slotGetStartExe()));

frame = addPage(i18n("WU Logging"),
                i18n("Logging of completed work units"),
                DesktopIcon("document", KIcon::SizeMedium));
frameLayout = new QVBoxLayout(frame, 0, 0);
m_clientPage = new ClientPageLayout(frame);
frameLayout->addWidget(m_clientPage);

frame = addPage(i18n("SETI@home Client"),
                     i18n("Settings of the SETI@home client"),
                     DesktopIcon("exec", KIcon::SizeMedium));
frameLayout = new QVBoxLayout(frame, 0, 0);
m_loggingPage = new LoggingPageLayout(frame);
frameLayout->addWidget(m_loggingPage);
// signals and slots connections
connect(m_loggingPage->RedirButton, SIGNAL(clicked()), 
        this, SLOT(slotGetLogDirectory()));

QToolTip::add(m_clientPage->StopAfterProcChkBox,
              i18n("Process the work unit and stop after \n"
                   "processing is complete, do not return \n"
                   "the result."));

QToolTip::add(m_clientPage->StopAfterSendChkBox,
              i18n("Finish the current work unit, and \n"
                   "return the result without downloading\n"
                   "a new work unit."));

QToolTip::add(m_clientPage->EmailChkBox,
              i18n("Send email (to login email address) on\n"
                   "errors. This option is not available\n"
                   "for all clients."));

QToolTip::add(m_clientPage->GraphicsChkBox,
              i18n("Generate a data stream for the xsetiathome\n"
                   "graphical interface (see README.xsetiathome).\n"
                   "This option is not available for all clients."));


m_locationPage->DirecButton->setPixmap(SmallIcon("fileopen"));
m_locationPage->StartExeButton->setPixmap(SmallIcon("fileopen"));
m_locationPage->StopExeButton->setPixmap(SmallIcon("fileopen"));
m_locationPage->ColorButton->setColor(Qt::red);
m_loggingPage->RedirButton->setPixmap(SmallIcon("fileopen"));
}

/*------------------------------------------------------------------------ */
AddSetiLocDlg::~AddSetiLocDlg()
{
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotGetSetiDirectory()
{
QString d = KFileDialog::getExistingDirectory();
if(!d.isNull())
  setSetiDirectory(d);
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotGetLogDirectory()
{
QString d = KFileDialog::getExistingDirectory();
if(!d.isNull())
  setLogDirectory(d);
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotGetStartExe()
{
QString d = KFileDialog::getOpenFileName();
if(!d.isNull())
  setStartExeFilePath(d);
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotGetStopExe()
{
QString d = KFileDialog::getOpenFileName();
if(!d.isNull())
  setStopExeFilePath(d);
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotHandleText(const QString&)
{
QString l1 = m_locationPage->DescrEdit->text();
QString l2 = m_locationPage->DirecEdit->text();
if(l1.isEmpty() || l2.isEmpty())
  enableButtonOK(false);
else
  enableButtonOK(true);
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::slotOk()
{
QDir d(m_locationPage->DirecEdit->text());
if(d.exists())
  accept();
else
  KMessageBox::error(this, i18n("Directory not found.\n"
                                "Make sure to enter a valid\n"
                                "SETI@Home directory."));
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::setOptions(const LocSettings& setloc)
{
// location page
m_locationPage->DescrEdit->setText(setloc.location.description);
m_locationPage->ColorButton->setColor(setloc.location.color);
m_locationPage->DirecEdit->setText(setloc.location.directory);
m_locationPage->StartExeChkBox->setChecked(setloc.location.useCustomStartExe);
m_locationPage->StartExeEdit->setText(setloc.location.customStartExe);
m_locationPage->StopExeChkBox->setChecked(setloc.location.useCustomStopExe);
m_locationPage->StopExeEdit->setText(setloc.location.customStopExe);
// client page
m_clientPage->StartSetiChkBox->setChecked(setloc.client.startSeti);
m_clientPage->StopSetiChkBox->setChecked(setloc.client.stopSeti);
m_clientPage->KeepAliveCheckBox->setChecked(setloc.client.keepClientAlive);
m_clientPage->ArgEdit->setText(setloc.client.arguments);
m_clientPage->StopAfterProcChkBox->setChecked(setloc.client.stopAfterProcess);
m_clientPage->StopAfterSendChkBox->setChecked(setloc.client.stopAfterSend);
m_clientPage->EmailChkBox->setChecked(setloc.client.email);
m_clientPage->GraphicsChkBox->setChecked(setloc.client.graphics);
m_clientPage->PrioritySlider->setValue(setloc.client.priority);
m_clientPage->ProxyServerChkBox->setChecked(setloc.client.useProxy);
m_clientPage->ProxyServerEdit->setText(setloc.client.proxyServer);
m_clientPage->ProxyPortEdit->setText(QString::number(setloc.client.proxyPort));
// logging page
m_loggingPage->LogChkBox->setChecked(setloc.logging.log);
m_loggingPage->RedirChkBox->setChecked(setloc.logging.redirectLog);
m_loggingPage->RedirEdit->setText(setloc.logging.logDirectory);
}

/*------------------------------------------------------------------------ */
LocSettings AddSetiLocDlg::options() const
{
LocSettings setloc;

// location
setloc.location.description       = m_locationPage->DescrEdit->text();
setloc.location.color             = m_locationPage->ColorButton->color();
setloc.location.directory         = m_locationPage->DirecEdit->text();
setloc.location.useCustomStartExe = m_locationPage->StartExeChkBox->isChecked();
setloc.location.customStartExe    = m_locationPage->StartExeEdit->text().stripWhiteSpace();
setloc.location.useCustomStopExe  = m_locationPage->StopExeChkBox->isChecked();
setloc.location.customStopExe     = m_locationPage->StopExeEdit->text().stripWhiteSpace();
// client
setloc.client.startSeti         = m_clientPage->StartSetiChkBox->isChecked();
setloc.client.stopSeti          = m_clientPage->StopSetiChkBox->isChecked();
setloc.client.keepClientAlive   = m_clientPage->KeepAliveCheckBox->isChecked();
setloc.client.arguments         = m_clientPage->ArgEdit->text();
setloc.client.stopAfterProcess  = m_clientPage->StopAfterProcChkBox->isChecked();
setloc.client.stopAfterSend     = m_clientPage->StopAfterSendChkBox->isChecked();
setloc.client.email             = m_clientPage->EmailChkBox->isChecked();
setloc.client.graphics          = m_clientPage->GraphicsChkBox->isChecked();
setloc.client.priority          = m_clientPage->PrioritySlider->value();
setloc.client.useProxy          = m_clientPage->ProxyServerChkBox->isChecked();
setloc.client.proxyServer       = m_clientPage->ProxyServerEdit->text();
setloc.client.proxyPort         = m_clientPage->ProxyPortEdit->text().toInt();
// logging
setloc.logging.log               = m_loggingPage->LogChkBox->isChecked();
setloc.logging.redirectLog       = m_loggingPage->RedirChkBox->isChecked();
setloc.logging.logDirectory      = m_loggingPage->RedirEdit->text().stripWhiteSpace();

return(setloc);
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::setSetiDirectory(const QString& t)
{
m_locationPage->DirecEdit->setText(t);
}

/*------------------------------------------------------------------------ */
QString AddSetiLocDlg::setiDirectory() const
{
return(m_locationPage->DirecEdit->text());
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::setLogDirectory(const QString& t)
{
m_loggingPage->RedirEdit->setText(t);
}

/*------------------------------------------------------------------------ */
QString AddSetiLocDlg::logDirectory() const
{
return(m_loggingPage->RedirEdit->text().stripWhiteSpace());
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::setStartExeFilePath(const QString& exe)
{
m_locationPage->StartExeEdit->setText(exe);
}

/*------------------------------------------------------------------------ */
QString AddSetiLocDlg::startExeFilePath() const
{
return(m_locationPage->StartExeEdit->text().stripWhiteSpace());
}

/*------------------------------------------------------------------------ */
void AddSetiLocDlg::setStopExeFilePath(const QString& exe)
{
m_locationPage->StopExeEdit->setText(exe);
}

/*------------------------------------------------------------------------ */
QString AddSetiLocDlg::stopExeFilePath() const
{
return(m_locationPage->StopExeEdit->text().stripWhiteSpace());
}

#include "addsetilocdlg.moc"

