/***************************************************************************/
/** @file     addsetilocdlg.h
  * @brief    A dialog for adding and editing a %S@h location (declaration).
  *
  * @b begin      : Sat Oct 30 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef ADDSETILOCDLG_H
#define ADDSETILOCDLG_H

#include <kdialogbase.h>

#include "setiloc.h"

class LocationPageLayout;
class ClientPageLayout;
class LoggingPageLayout;

/** A dialog for adding and editing a %S@h location.
  *
  * This dialog inherits from KDialogBase. Different pages with options and
  * settings can be selected via an icon list.
  */
class AddSetiLocDlg : public KDialogBase
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  AddSetiLocDlg(QWidget* parent=0, const char* name=0);
  /** The destructor. */
  ~AddSetiLocDlg();
  /** Passes the location settings to the dialog.
    *
    * Sets all widgets in the dialog into the state represented by the
    * parameters given in @a locsets.
    *
    * @param locsets A structure containing the options of a %S@h location.
    */
  void setOptions(const LocSettings& locsets);
  /** Returns the settings made in the dialog in a LocSettings structure. */
  LocSettings options() const;
  /** Sets the %S@h directory.
    *
    * The %S@h directory is the directory where all the sah stae files, the
    * WU file and (optionally) the %SETI@home client are located.
    *
    * @see setiDirectory(), setExeFilePath(), exeFilePath(),
    *      setLogDirectory(), logDirectory()
    *
    * @param t A string pointing to the %S@h directory.
    */
  void setSetiDirectory(const QString& t);
  /** Returns the %S@h directory string from the dialog.
    *
    * @see setSetiDirectory(), setExeFilePath(), exeFilePath(),
    *      setLogDirectory(), logDirectory()
    */
  QString setiDirectory() const;
  /** Sets the log directory.
    *
    * By default, log files are written to the %S@h directory. The user
    * can redirect the writing of the log files to another directory by
    * specifying it in the dialog.
    *
    * @see logDirectory(), setSetiDirectory(), setiDirectory(),
    *      setExeFilePath(), exeFilePath()
    *
    * @param t A string pointing to the log directory.
    */
  void setLogDirectory(const QString& t);
  /** Returns the log directory string from the dialog.
    *
    * @see setLogDirectory(), setSetiDirectory(), setiDirectory(),
    *      setExeFilePath(), exeFilePath()
    */
  QString logDirectory() const;
  /** Sets path and file name of the %S@h start-up executable.
    *
    * The specified executable is responsible for starting the %S@h client.
    * Purpose of this feature is to allow users to execute other things prior
    * or after starting the client.
    *
    * @see setStopExeFilePath(), startExeFilePath(), stopExeFilePath
    *
    * @param exe A string containing a valid path and file name of the %S@h
    *            start-up executable.
    */
  void setStartExeFilePath(const QString& exe);
  /** Returns path and file name of the %S@h start-up executable in one
    * string.
    *
    * @see setStartExeFilePath(), setStopExeFilePath(), stopExeFilePath()
    */
  QString startExeFilePath() const;
  /** Sets path and file name of the %S@h shut-down executable.
    *
    * The specified executable is responsible for stopping the %S@h client.
    * Purpose of this feature is to allow users to execute other things prior
    * or after stopping the client.
    *
    * @see setStartExeFilePath(), startExeFilePath(), stopExeFilePath
    *
    * @param exe A string containing a valid path and file name of the %S@h
    *            shut-down executable.
    */
  void setStopExeFilePath(const QString& exe);
  /** Returns path and file name of the %S@h shut-down executable in one
    * string.
    *
    * @see setStartExeFilePath(), setStopExeFilePath(), startExeFilePath()
    */
  QString stopExeFilePath() const;

protected slots:
  /** Checks the given %S@h directory before exiting the dialog. */
  void slotOk();
  /** Opens a file dialog for the selection of the %S@h directory. */
  void slotGetSetiDirectory();
  /** Opens a file dialog for the selection of the log directory. */
  void slotGetLogDirectory();
  /** Opens a file dialog for the selection of path and file name of the %S@h
    * start-up executable. */
  void slotGetStartExe();
  /** Opens a file dialog for the selection of path and file name of the %S@h
    * shutdown executable. */
  void slotGetStopExe();
  /** Enables or disables the OK button.
    *
    * The OK button of the dialog will only be enabled if both a %S@h
    * directory and a descriptive name for the location have been entered.
    *
    * @param t Not used in this context.
    */
  void slotHandleText(const QString& t);

  protected:
  LocationPageLayout* m_locationPage;
  ClientPageLayout*   m_clientPage;
  LoggingPageLayout*  m_loggingPage;
};

#endif







