/***************************************************************************/
/** @file     analysis.h
  * @brief    The Analysis tab page and helper classes (declaration).
  *
  * @b begin      : Tue Oct 26 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef ANALYSIS_H
#define ANALYSIS_H

#include <qwidget.h>
#include <qlistview.h>
#include <qdict.h>
#include <qpixmap.h>

#include "setidefs.h"
#include "poplistview.h"

class QPushButton;
class QPopupMenu;
class QComboBox;
class QLabel;
class SetiLoc;

/**  A customized QListViewItem for the list view in the Analysis tab.
  */
class AnalysisListViewItem : public ColorListViewItem
{
public:
  /** The constructor.
    *
    * @param parent The parent list view.
    */
  AnalysisListViewItem(QListView* parent) : ColorListViewItem(parent) {}
  /** The destructor. */
  ~AnalysisListViewItem() {}
  /** Reimplemented to allow customized sorting of the items in the list.
    *
    * @param column    The column for which a key is returned.
    * @param ascending Indicates the sorting order (usage not recommended).
    *
    * @return A key that can be used for sorting by column @a column.
    */
  virtual QString key(int column, bool ascending) const;

};

/** The widget for the Analysis tab page.
  */
class Analysis : public QWidget
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  Analysis(QWidget *parent=0, const char *name=0);
  /** The destructor. */
  ~Analysis();
  /** Selects and highlights the listview item given by the id of the
    * location.
    *
    * @param lid The unique id string of the location.
    */
  void setSelectedLocation(const QString& lid);
  /** Returns the currently highlighted (selected) location in the
    * Analysis list.
    *
    * @return A pointer to the SetiLoc object corresponding to the selected
    *         location.
    */
  SetiLoc* selectedLocation();

public slots:
  /** Refreshes all items in the list view.
    *
    * If corresponding list view item for a %S@h location does not exist,
    * it is created on-the-fly.
    */
  void refreshList();
  /** Updates the private list of %S@h locations.
    *
    * Updates the private list of %S@h locations with the location @a loc. The
    * @a type parameter specifies what to do with @a loc.
    *
    * @param loc  The %S@h location to be updated.
    * @param type The type of action to be performed with @a loc. See
    *             Ksetiwatch::ListId for details.
    */
  void slotUpdateList(SetiLoc* loc, int type);

protected: // attributes
  /** The list view shown in the Analysis tab page. */
  popListView* LocListView;
  /** A hidden list view containing all items currently hidden. */
  QListView*   LocListViewHidden;
  QPushButton* RefreshBtn;
  QPushButton* StartAllBtn;
  QPushButton* StopAllBtn;
  QPushButton* HomepageBtn;
  QPushButton* HelpBtn;
  QLabel*      ComboLabel;
  QComboBox*   StatusFilter;
  QPixmap      StatusIcon[5];
  QPopupMenu*  showOptions;
  /** A dictionary that stores all the list view items currently shown in the
    * Analysis tab page. Items are identified by the unique ID of a %S@h
    * location. */
  QDict<AnalysisListViewItem> lvDict;
  /** A dictionary that stores all the list view items currently hidden.
    * Items are identified by the unique ID of a %S@h location. */
  QDict<AnalysisListViewItem> lvDictHidden;
  bool sortingorder;
  /** Identifiers for the menu items in the right-click popup menu. */
  enum soIds {soGaussian, soPulse, soTriplet};
  /** Identifiers for the menu items in the right-click popup menu. */
  enum popupIds {PopupAdd, PopupEdit, PopupDelete, PopupStart,
                 PopupStop, PopupShow};
  /** Identifiers for the columns in the Analysis list view. */
  enum ColumnId {ColLocation, ColStatus, ColChirpRate, ColProgress,
                 ColCpuTime, ColTimeLeft, ColProgressRate, ColMFlops,
                 ColSpike, ColGaussian, ColPulse, ColTriplet};

protected: // methods
  /** Resizes and rearranges widgets in the tab page when the window size
    * changes.
    *
    * A relict from ancient times when there were no layout widgets.
    *
    * @param e An object containing details about the resize event.
    */
  void resizeEvent(QResizeEvent* e);
  /** Reimplemented to tell the list view whether it shall draw grid lines.
    *
    * @param e An object containing details about the paint event.
    */
  void paintEvent(QPaintEvent* e);
  /** Determines the list view item from the name of a %S@h location.
    *
    * @param locName The name of the location.
    *
    * @return A pointer to the AnalysisListViewItem, or 0 if it couldn't be
    *         found.
    */
  AnalysisListViewItem* locationItemByName(const QString& locName);
  /** Determines the list view item from the unique ID of a %S@h location.
    *
    * @param locId The unique id string of the location.
    *
    * @return A pointer to the AnalysisListViewItem, or 0 if it couldn't be
    *         found.
    */
  AnalysisListViewItem* locationItemById(const QString& locId);
  /** Determines the %S@h location object from an item in the Analysis list
    * view.
    *
    * @param it The list view item.
    *
    * @return The corresponding SetiLoc object, or 0 if it couldn't be found.
    */
  SetiLoc* getLocation(AnalysisListViewItem* it);
  /** Returns the currently selected item in the Analysis list view.
    *
    * @return The selected item or zero.
    */
  AnalysisListViewItem* selectedItem();
  /** Refreshes all columns of the list view item of location @a loc.
    *
    * @param loc A pointer to the SetiLoc object.
    */
  void refreshListItem(SetiLoc* loc);
  /** Refreshes the time data of %S@h location @a loc.
    *
    * @param loc A pointer to the SetiLoc object.
    */
  void updateListItemTimeData(SetiLoc* loc);
  /** Refreshes the best spike data of %S@h location @a loc.
    *
    * @param loc A pointer to the SetiLoc object.
    */
  void updateListItemSpikeData(SetiLoc* loc);
  /** Refreshes the best gaussian data of %S@h location @a loc.
    *
    * @param loc A pointer to the SetiLoc object.
    */
  void updateListItemGaussianData(SetiLoc* loc);
  /** Refreshes the best pulse data of %S@h location @a loc.
    *
    * @param loc A pointer to the SetiLoc object.
    */
  void updateListItemPulseData(SetiLoc* loc);
  /** Refreshes the best triplet data of %S@h location @a loc.
    *
    * @param loc A pointer to the SetiLoc object.
    */
  void updateListItemTripletData(SetiLoc* loc);
  /** Refreshes the state icon of %S@h location @a loc.
    *
    * @param loc      A pointer to the SetiLoc object.
    * @param state    The new state of the client of @a loc.
    * @param loadsize If state is @c Loading, this argument reflects the
    *                 progress of the loading process.
    */
  void updateStateIcon(SetiLoc* loc, int state, int loadsize);
  /** Updates the internal spike high score of the %S@h location @a loc.
    *
    * If @a newspike is larger than the highest spike, a high-spike (or
    * record-spike) message box will be shown. Optionally, an audio signal
    * will be played.
    *
    * @param loc      A pointer to the SetiLoc object.
    * @param newspike A SpikeScore structure with the new high spike data.
    */
  void updateMaxScore(SetiLoc* loc, const SpikeScore& newspike);
  /** Updates the internal gaussian high score of the %S@h location @a loc.
    *
    * If @a newgaussian is larger than the highest gaussian, a high-gaussian
    * (or record-gaussian) message box will be shown. Optionally, an audio
    * signal will be played.
    *
    * @param loc         A pointer to the SetiLoc object.
    * @param newgaussian A GaussianScore structure with the new high gaussian
    *                    data.
    */
  void updateMaxScore(SetiLoc* loc, const GaussianScore& newgaussian);
  /** Updates the internal pulse high score of the %S@h location @a loc.
    *
    * If @a newpulse is larger than the highest pulse, a high-pulse (or
    * record-pulse) message box will be shown. Optionally, an audio signal
    * will be played.
    *
    * @param loc      A pointer to the SetiLoc object.
    * @param newspike A PulseScore structure with the new high pulse data.
    */
  void updateMaxScore(SetiLoc* loc, const PulseScore& newpulse);
  /** Updates the internal triplet high score of the %S@h location @a loc.
    *
    * If @a newtriplet is larger than the highest triplet, a high-triplet
    * (or record-triplet) message box will be shown. Optionally, an audio
    * signal will be played.
    *
    * @param loc        A pointer to the SetiLoc object.
    * @param newtriplet A TripletScore structure with the new high triplet
    *                   data.
    */
  void updateMaxScore(SetiLoc* loc, const TripletScore& newtriplet);
  /** Shows or hides the list view item of a %S@h location.
    *
    * Depending on the status of the client of location @a loc and the
    * settings of the status filter, the list view item of location @a loc
    * will either be shown or hidden.
    *
    * @param loc The %S@h location for which the corresponding list view
    *            item is to be updated.
    */
  void updateAnalysisList(SetiLoc* loc);
  /** Centers the status icon in the corresponding column of list view
    * item @a it..
    *
    * This method is required because Qt doesn't center pixmaps in list view
    * columns even if AlignCenter is set.
    *
    * @param it    A pointer to the list view item for which the status icon
    *              shall be centered.
    * @param state Determines which icon is shown.
    */
  void centerStatusIcon(QListViewItem* it, int state);

protected slots: // Protected slots
  /** Refreshes the time-related columns in the Analysis list */
  void slotUpdateAnalysisTimeData();
  /** Refreshes the spike column in the Analysis list and displays
    * a record message if necessary.
    *
    * @param newspike A SpikeScore structure with the new spike data.
    */
  void slotUpdateAnalysisSpikeData(SpikeScore newspike);
  /** Refreshes the gaussian column in the Analysis list and displays
    * a record message if necessary.
    *
    * @param newgaussian A GaussianScore structure with the new gaussian data.
    */
  void slotUpdateAnalysisGaussianData(GaussianScore newgaussian);
  /** Refreshes the pulse column in the Analysis list and displays
    * a record message if necessary.
    *
    * @param newpulse A PulseScore structure with the new pulse data.
    */
  void slotUpdateAnalysisPulseData(PulseScore newpulse);
  /** Refreshes the triplet column in the Analysis list and displays
    * a record message if necessary.
    *
    * @param newtriplet A TripletScore structure with the new triplet data.
    */
  void slotUpdateAnalysisTripletData(TripletScore newtriplet);
  /** Changes the status icon to @a state for the object connected to
    * this slot.
    *
    * @param state    The new state of the client of @a loc.
    * @param loadsize If state is @c Loading, this argument reflects the
    *                 progress of the loading process.
    */
  void slotShowNewState(int state, int loadsize);
  /** Opens the default web browser with the Ksetiwatch home page. */
  void openBrowser();
  /** Changes the StatusFilterValue to @a sel. */
  void modifyStatusFilter(int sel);
  /** Invokes the Help window. */
  void slotShowHelp();
  /** Toggles the sorting order of column @a column. */
  void toggleSorting(int column);
  /** Notifies other widgets when another location is selected in the
    * Analysis list.
    *
    * Determines the SetiLoc object from the list view item @a it and emits
    * a selectionChanged() signal.
    *
    * @param it The newly selected (highlighted) list view item.
    */
  void handleNewSelection(QListViewItem* it);
  /** Asks the user for the configuration of the first %S@h location.
    *
    * If Ksetiwatch is started for the first time without preconfigured
    * %S@h locations, this slot is called after approximately 1.5 seconds.
    */
  void slotAskForLocation();
  /** Stops the clients of all %S@h locations (if possible). */
  void slotStopAll();
  /** Starts the clients of all %S@h locations (if possible). */
  void slotStartAll();
  /** Executes the selected command from the right-click popup menu of the
    * Analysis list.
    *
    * @param id The unique ID number of the selected popup menu item. See
    *           Analysis::popupIds for details.
    */
  void handlePopupCommand(int id);
  /** Prepares the right-click popup menu just before showing it.
    *
    * Checks if the popup menu commands are allowed or make sense in the
    * current situation, and enables or disables the menu items appropriately.
    */
  void checkPopupStatus();
  /** Shows the selected graph from the popup sub-menu.
    *
    * @param id The unique ID number of the selected popup menu item. See
    *           Analysis::soIds for details.
    */
  void handleShowPopupCommand(int id);
  /** Adjusts the position of the icons in the status column such that all
    * icons are centered appropriately.
    *
    * @param col     The column for which the size has changed.
    * @param oldSize The old size of the column.
    * @param newSize The new size of the column.
    */
  void slotAdjustAllStatusIcons(int col, int, int);

signals:
  /** This signal is emitted when the user wants to add, edit, or delete a
    * %S@h location.
    *
    * The primary reason for this signal is to notify other widgets or object
    * (in particular, the Ksetiwatch main window which maintains the list of
    * %S@h locations) of the change. Arguments are a pointer to a (new or
    * already existing) SetiLoc object, and the type of action requested (see
    * Ksetiwatch::ListId).
    */
  void listChangeRequested(SetiLoc*, int);
  /** This signal is emitted whenever the user selects another %S@h location
    * in the Analysis list. Argument is a pointer to the SetiLoc object of
    * the selected location, or 0 if no location is selected in the list.
    */
  void selectionChanged(SetiLoc*);
};

#endif




















