/***************************************************************************/
/** @file     completedwulist.h
  * @brief    The Completed Work Units tab page and helper classes
  *           (declaration).
  *
  * @b begin      : Tue Nov 23 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef COMPLETEDWULIST_H
#define COMPLETEDWULIST_H

#include <qlistview.h>
#include <qpalette.h>
#include <qpixmap.h>

#include "ksetiwatch.h"
#include "poplistview.h"

class QSplitter;
class SetiLoc;
class SkyMap;

/** A customized QListViewItem for the WU list view in the Completed
  * Work Units tab.
  */
class WUListViewItem : public ColorListViewItem
{
public:
  /** The constructor.
    *
    * @param parent The parent list view.
    */
  WUListViewItem(QListView* parent) : ColorListViewItem(parent) {}
  /** The destructor. */
  ~WUListViewItem() {}
  /** Reimplemented to allow customized sorting of the items in the list.
    *
    * @param column    The column for which a key is returned.
    * @param ascending Indicates the sorting order (usage not recommended).
    *
    * @return A key that can be used for sorting by column @a column.
    */
  virtual QString key(int column, bool ascending) const;
};

/** A customized QListViewItem for the WU list view in the Completed
  * Work Units tab.
  */
class SigListViewItem : public ColorListViewItem
{
public:
  /** The constructor.
    *
    * @param parent The parent list view.
    */
  SigListViewItem(QListView* parent) : ColorListViewItem(parent) {}
  /** The destructor. */
  ~SigListViewItem() {}
  /** Reimplemented to allow customized sorting of the items in the list.
    *
    * @param column    The column for which a key is returned.
    * @param ascending Indicates the sorting order (usage not recommended).
    *
    * @return A key that can be used for sorting by column @a column.
    */
  virtual QString key(int column, bool ascending) const;
};

/** The widget for the Completed Work Units tab page.
  */
class CompletedWUList : public QWidget
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level widget.
    * @param name   A text that can be used to identify this widget.
    */
  CompletedWUList(QWidget *parent=0, const char *name=0);
  /** The destructor. */
  ~CompletedWUList();

public slots:
  /** Updates the tree list.
    *
    * Updates the tree list (at the left of the tab page) with %S@h location
    * @a loc. @a type specifies in detail what to do with @a loc (valid
    * IDs are Ksetiwatch::ListAdd, Ksetiwatch::ListEdit, and
    * Ksetiwatch::ListDelete).
    *
    * @param loc  A pointer to the SetiLoc object.
    * @param type The type of action to be performed with @a loc. See
    *             Ksetiwatch::ListId for details.
    */
  void slotUpdateList(SetiLoc* loc, int type);

protected:
  /** Resizes and rearranges widgets in the tab page when the window size changes.
    *
    * A relict from ancient times when there were no layout widgets.
    *
    * @param e An object containing details about the resize event.
    */
  void resizeEvent(QResizeEvent *e);
  /** Reimplemented to tell the list views whether they shall draw grid lines.
    *
    * @param e An object containing details about the paint event.
    */
  void paintEvent(QPaintEvent* e);
  /** Creates a list view.
    *
    * This method adds the columns defined in @a tc to the list view @a list.
    *
    * @param list A pointer to a popListView.
    * @param tc   An array of TableColumn structures defining the columns in
    *             the list view.
    */
  void createList(popListView* list, Ksetiwatch::TableColumn tc[]);
  /** Fills the tree list view with items representing the different %S@h
    * locations.
    */
  void fillTreeList();
  /** Fills a list view with data about the logged WUs of %S@h location
    * @a loc.
    *
    * The list of logged work units is read from the file @c SETILog.csv in
    * the log directory of location @a loc (by default, this is the working
    * directory of the client).
    *
    * @param loc A pointer to a SetiLoc object for which the logged WUs are to
    *            be shown.
    */
  void fillWUList(SetiLoc* loc);
  /** Fills a list view with a summary of logged WUs for all %S@h locations.
    *
    * The list of logged work units is read from the file @c SETILog.csv in
    * the log directory of the respective %S@h locations (by default, this is
    * the working directory of the client).
    */
  void fillTotalList();
  /** Fills a list view with data about the returned signals of %S@h location
    * @a loc.
    *
    * The list of returned signals is read from the file @c SETIResult.log in
    * the log directory of location @a loc (by default, this is the working
    * directory of the client).
    *
    * @param loc A pointer to a SetiLoc object for which the list of returned
    *            signals is to be shown.
    */
  void fillSigList(SetiLoc* loc);
  /** Fills a list view with a summary of returned signals for all %S@h
    * locations.
    *
    * The list of returned signals is read from the file @c SETILog.csv in
    * the log directory of the respective %S@h locations (by default, this is
    * the working directory of the client).
    */
  void fillSigSumList();
  /** Adds the logged WUs of %S@h location @a loc to the skymap window.
    *
    * @param loc A pointer to a SetiLoc object.
    */
  void addLoggedWU(SetiLoc* loc);
  /** Adds the logged WUs of all %S@h locations to the skymap window.
    */
  void addAllLoggedWU();
  /** Refreshes the currently visible list view.
    *
    * @param loc A pointer to the SetiLoc object used to refresh the data.
    */
  void refreshList(SetiLoc* loc);
  /** Checks whether the log directory of %S@h location @a loc is redirected
    * to another %S@h location.
    *
    * @param loc A pointer to a SetiLoc object that is to be checked.
    *
    * @return A pointer to a SetiLoc object to which the log directory of
    *         @a loc is redirected, or 0 if it is not redirected.
    */
  SetiLoc* checkForRedirect(SetiLoc* loc);
  /** Updates the tree list.
    *
    * Updates the tree list (at the left of the tab page) with %S@h location
    * @a loc. @a type specifies in detail what to do with @a loc (valid
    * IDs are Ksetiwatch::ListAdd, Ksetiwatch::ListEdit, and
    * Ksetiwatch::ListDelete).
    *
    * @param loc  A pointer to the SetiLoc object.
    * @param type The type of action to be performed with @a loc. See
    *             Ksetiwatch::ListId for details.
    */
  void updateList(SetiLoc* loc, int type);
  /** Sets the columns of list view item @a it to the given arguments.
    *
    * @param it        A pointer to the list view item.
    * @param wus       The number of work units.
    * @param spikes    The number of spikes returned.
    * @param gaussians The number of gaussians returned.
    * @param pulses    The number of pulses returned.
    * @param triplets  The number of triplets returned.
    * @param top       A WUScore structure containing the data of the top
    *                  signals.
    */
  void fillSigSumColumns(QListViewItem* it, const int wus, const int spikes,
                         const int gaussians, const int pulses,
                         const int triplets, const WUScore& top);
  /** Sets the columns of list view item @a it to the given arguments.
    *
    * @param it     A pointer to the list view item.
    * @param wus    The number of work units.
    * @param nzwus  The number of work units with non-zero processing time.
    * @param totime The total CPU processing time.
    * @param top    A WUScore structure containing the data of the top
    *               signals.
    */
  void fillTotalColumns(QListViewItem* it, const int wus, const int nzwus,
                        const double totime, const WUScore& top);

protected slots:
  /** Processes mouse-button click event on the tree list.
    *
    * @param item The list view item clicked, or 0 if the click event didn't
    *             happen on an item.
    */
  void slotSelect(QListViewItem *item);
  /** Disables menu items depending on which tree list item is chosen. */
  void prepareTreeListPopupMenu();
  /** Processes a command from the right-click popup menu of the tree list.
    *
    * @param id The identifier of the menu item selected.
    */
  void handleTreeListPopupCommand(int id);
  /** Processes a command from the right-click popup menu of the WU list.
    *
    * @param id The identifier of the menu item selected.
    */
  void handleWUListPopupCommand(int id);
  /** Processes a command from the right-click popup menu of the signal list.
    *
    * @param id The identifier of the menu item selected.
    */
  void handleSigListPopupCommand(int id);
  /** Toggles the sorting order of column @a column. */
  void toggleSorting(int column);
  /** Shows the Logged Signals dialog after double-clicking an item
    * in the signal list.
    *
    * @param sit A pointer to the list view item that was double-clicked.
    */
  void handleSigListDoubleClick(QListViewItem* sit);
  /** Refreshes the currently visible list view.
    *
    * @param loc A pointer to the SetiLoc object used to refresh the data.
    */
  void slotRefreshList(SetiLoc* loc);

protected: // Protected attributes
  /** The main widget of this tab. All list views below are
    * children of this widget.
    */
  QSplitter*   splitview;
  /** The tree list at the left. Always visible. */
  popListView* treelist;
  /** Four different list views (WU summary list, WU list, signal summary
      list, signal list). */
  popListView* cwuList[4];
  /** Identifiers for the four different list views. */
  enum ListId {WUSummary, WUList, SignalSummary, SignalList};
  /** Identifiers for the columns in the WU list view. */
  enum WulColumnIds {WulDateLogged, WulWuName, WulCpuTime, WulProgRate,
                     WulSpike, WulGaussian, WulPulse, WulTriplet, WulFrom,
                     WulRecordedOn, WulBaseFreq, WulAngleRange, WulTeraFlops};
  /** Identifiers for the columns in the WU summary list view. */
  enum WusColumnIds {WusLocation, WusWus, WusTotalTime, WusAvgTime,
                     WusAvgProgRate, WusRecSpike, WusRecGaussian, WusRecPulse,
                     WusRecTriplet};
  /** Identifiers for the columns in the signal list view. */
  enum SilColumnIds {SilDateLogged, SilWuName, SilSpikes, SilTopSpike,
                     SilGaussians, SilTopGaussian, SilPulses, SilTopPulse,
                     SilTriplets, SilTopTriplet};
  /** Identifiers for the columns in the signal summary list view. */
  enum SisColumnIds {SisLocation, SisWus, SisTotalSpikes, SisTopSpike,
                     SisTotalGaussians, SisTopGaussian, SisTotalPulses,
                     SisTopPulse, SisTotalTriplets, SisTopTriplet};
  /** Identifiers for the right-click popup menu items. */
  enum popupIds {ShowSkymap, ReloadLocations, ShowLoggedSignals};
  /** Determines which list is currently shown. */
  int currentView;
  bool sortorder;
  QPixmap folder_icon;
  QPixmap dish_icon;

signals:
  /** This signal is emitted whenever a list has been updated. Needed to save
    * potential new record signals.
    *
    * @todo Check for a better implementation.
    */
  void listUpdated();
};

#endif
