/***************************************************************************/
/** @file     csvdatabase.h
  * @brief    A class to access a comma-separated-value (csv) file.
  *           (declaration).
  *
  * @b begin      : Thu Dec 16 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef CSVDATABASE_H
#define CSVDATABASE_H

#include <qfile.h>
#include <qstring.h>
#include <qstrlist.h>

/** A class that allows reading of csv files.
  *
  * This class allows access to the elements of a comma-separated-value
  * (csv) file. It doesn't store the entire array in memory, but reads and
  * processes the elements line-by-line. The first line of the csv file must
  * be a header line which consists of ID strings defining the contents of
  * the columns.
  */
class CSVDataBase : public QFile
{
public:
  /** The constructor
    *
    * Initializes the access to a csv file.
    *
    * @param file The file name. If the pointer is 0, an uninitialized
    *             object is created, and the file name must be set with
    *             setName().
    */
  CSVDataBase(const char *file=0);
  /** The destructor.
    */
  ~CSVDataBase();
  /** Reads an element from the csv file.
    *
    * The element is read from column @a column and line @a line.
    *
    * @param column The column number of the element.
    * @param line   The line number of the element.
    *
    * @return The element at the given position, or an empty string if the
    *         element could not be read.
    */
  QString readItem(int column, int line);
  /** Reads an element from the csv file.
    *
    * @overload
    *
    * The element is read from the column specified by @a identifier and line
    * @a line.
    *
    * @param identifier The identifier string of the column.
    * @param line       The line number.
    *
    * @return The element at the given position, or an empty string if the
    *         element could not be read.
    */
  QString readItem(const char* identifier, int line);
  /** Returns the number of columns in the csv file. */
  int columnCount();

protected:
  /** Reads all elements of a line.
    *
    * Reads all elements of the line @a linestr and stores them in the string
    * list @a strlist.
    *
    * @param linestr A string containing a line of a csv file.
    * @param strlist A pointer to a QStrList where the elements are stored.
    *
    * @return The number of elements in the line.
    */
  int readItems(const char* linestr, QStrList* strlist);
  /** Reads the definition line.
    *
    * The definition (or header) line must always be the first line of a csv
    * file.
    *
    * @return The number of columns in the definition line.
    */
  int readDefinitionLine();
  /** Reads the current line.
    *
    * @a line becomes the current line.
    *
    * @param line The line to be read.
    *
    * @return The number of columns in the line.
    */
  int readCurrentLine(int line);
  /** Reads a line and returns it in a string.
    *
    * @param line The line to be read.
    *
    * @return A string containing the line, or an empty string if the line
    *         number is too high.
    */
  QString readLine(int line);
  /** Reads a line at the current file index and returns it in a string.
    *
    * @return A string containing the line, or an empty string if the file
    *         index is at the end of the file.
    */
  QString readNextLine();
  /** Extracts an element from a string.
    *
    * The element at column @a column is read from the string @a linestr.
    *
    * @param column  The column number of the element.
    * @param linestr A string containing a line of a csv file.
    *
    * @return A string containing the line, or an empty string if the column
    *         number is too high.
    */
  QString readItem(int column, const char *linestr);
  /** Extracts an element from a string.
    *
    * @overload
    *
    * The element is read from the column specified by @a identifier in the
    * current line.
    *
    * @param identifier The identifier string of the column.
    *
    * @return A string containing the line, or an empty string if the column
    *         number is too high.
    */
  QString readItem(const char *identifier);

protected:
  QString definitionLine;
  QStrList defentries;
  QString currentLine;
  QStrList currententries;
  int currentLineNr;
  int currentLineColumns;
  int columns;
};

#endif
