/***************************************************************************/
/** @file     datainfo.cpp
  * @brief    The Data Info tab page and helper classes (implementation).
  *
  * @b begin      : Tue Oct 26 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <math.h>

#include <kapp.h>
#include <klocale.h>
#include <kmessagebox.h>

#include <qheader.h>
#include <qpushbutton.h>
#include <qpainter.h>

#include "skymap.h"
#include "datainfo.h"
#include "setiloc.h"
#include "ksetiwatch.h"

/*------------------------------------------------------------------------ */
QString DataInfoListViewItem::key(int column, bool) const
{
// return value must be static
static QString ret;
QString val;
double tmp;

if(column == 4)
  {
  val = text(column);
  tmp = val.toDouble();
  val.sprintf("%f", tmp);
  ret = val.rightJustify(12, '0');
  }
else
  {
  ret = text(column);
  }

return(ret);
}


/*------------------------------------------------------------------------ */
DataInfo::DataInfo(QWidget* parent, const char* name) : QWidget(parent,name)
{
Ksetiwatch::TableColumn ditc[] =
  {
  {i18n("Location"), 60, AlignLeft},
  {i18n("From"), 170, AlignRight},
  {i18n("Recorded On"), 170, AlignRight},
  {i18n("Source"), 120, AlignRight},
  {i18n("Base Frequency"), 100, AlignLeft},
  {i18n("Angle Range"), 100, AlignRight},
  {i18n("TeraFlops"), 80, AlignRight}
  };

QListView_1= new popListView(this);
QListView_1->setShowSortIndicator(false);
QListView_1->setPopupStyle(popListView::ShowAnywhere);
QListView_1->move(5,1);

for(int i=0; i<(int)(sizeof(ditc)/sizeof(ditc[0])); i++)
  {
  QListView_1->addColumn(ditc[i].text,-1);
  QListView_1->setColumnWidth(i, ditc[i].width);
  QListView_1->setColumnAlignment(i, ditc[i].alignment);
  }

QPushButton_1= new QPushButton(this);
QPushButton_1->move(10,100);
QPushButton_1->setText(i18n("Sky Map"));
QPushButton_1->setMinimumSize(QPushButton_1->sizeHint().width(), 25);
QPushButton_1->resize(QPushButton_1->sizeHint());
connect(QPushButton_1, SIGNAL(clicked()), SLOT(showMap()));

connect(QListView_1->header(), SIGNAL(sectionClicked(int)),
                         this, SLOT(toggleSorting(int)));
sortingorder = true;

lvDict.setAutoDelete(true);

refreshList();
}

/*------------------------------------------------------------------------ */
DataInfo::~DataInfo()
{
}

/*------------------------------------------------------------------------ */
void DataInfo::resizeEvent(QResizeEvent*)
{
QListView_1->resize(this->width()-10,
                    this->height()-QPushButton_1->height()-5);
QPushButton_1->move(5, QListView_1->y()+QListView_1->height()+2);
}

/*------------------------------------------------------------------------ */
void DataInfo::paintEvent(QPaintEvent* e)
{
QListView_1->setGridEnabled(globalopts->DrawGrid);
QWidget::paintEvent(e);
}

/*------------------------------------------------------------------------ */
void DataInfo::refreshList()
{
SetiLoc *loc;

for(loc=Ksetiwatch::locationList().first(); loc != 0;
    loc=Ksetiwatch::locationList().next())
  {
  DataInfoListViewItem* it = lvDict[loc->id()];
  if(it == 0)
    slotUpdateList(loc, Ksetiwatch::ListAdd);
  else
    refreshListItem(loc, it);
  }
}

/*------------------------------------------------------------------------ */
void DataInfo::slotUpdateList(SetiLoc* loc, int type)
{
DataInfoListViewItem* it;

switch(type)
  {
  case Ksetiwatch::ListAdd:
    {
    it = new DataInfoListViewItem(QListView_1);
    if(it)
      {
      lvDict.insert(loc->id(), it);
      connect(loc,  SIGNAL(updatedWorkUnitData()),
              this, SLOT(slotUpdateWorkUnitData()));
      refreshListItem(loc, it);
      }
    break;
    }
  case Ksetiwatch::ListEdit:
    {
    refreshListItem(loc);
    break;
    }
  case Ksetiwatch::ListDelete:
    {
    disconnect(loc,  SIGNAL(updatedWorkUnitData()),
               this, SLOT(slotUpdateWorkUnitData()));
    lvDict.remove(loc->id());
    break;
    }
  }
}

/*------------------------------------------------------------------------ */
void DataInfo::refreshListItem(SetiLoc* loc, DataInfoListViewItem* it)
{
QString val;

if(loc && it == 0)
    it = lvDict[loc->id()];
if(loc && it)
  {
  it->setText(ColLocation, loc->description());
  if(loc->exists(SetiLoc::SC_WorkUnitFile))
    {
    double ra  = loc->startRA();
    double dec = loc->startDec();
    val = loc->convertRAToString(ra) + " RA ";
    if(dec >= 0.0) val += "+ "; else val += "- ";
    val += loc->convertDecToString(fabs(dec)) + " Dec";
    it->setText(ColFrom, val);
    it->setText(ColRecordedOn, loc->timeRecordedString());
    if(loc->receiver() == "ao1420")
      it->setText(ColSource, i18n("Arecibo Telescope "));
    else
      it->setText(ColSource, QString::null);
    it->setText(ColBaseFreq, (QString("  %1 GHz")).arg(loc->baseFrequency(), 0, 'f', 9));
    it->setText(ColAngleRange, QString::number(loc->angleRange()));
    it->setText(ColTeraFlops, val.setNum(loc->teraFlops(loc->angleRange()), 'f', 4));
    }
  else
    {
    it->setText(ColFrom,       QString::null);
    it->setText(ColRecordedOn, QString::null);
    it->setText(ColSource,     QString::null);
    it->setText(ColBaseFreq,   QString::null);
    it->setText(ColAngleRange, QString::null);
    it->setText(ColTeraFlops,  QString::null);
    }
  }
}

/*------------------------------------------------------------------------ */
void DataInfo::showMap()
{
SkyMap* map = SkyMap::showMap();

if(map)
  {
  map->clearMap();
  for(SetiLoc *loc=Ksetiwatch::locationList().first(); loc != 0;
      loc=Ksetiwatch::locationList().next() )
    {
    map->addLocation(loc);
    }
  map->repaint();
  }
}

/*------------------------------------------------------------------------ */
void DataInfo::toggleSorting(int column)
{
sortingorder = !sortingorder;
QListView_1->setSorting(column, sortingorder);
}

/*------------------------------------------------------------------------ */
void DataInfo::slotUpdateWorkUnitData()
{
SetiLoc* loc = (SetiLoc*)(sender());

if(loc)
  refreshListItem(loc);
}

#include "datainfo.moc"

