/***************************************************************************/
/** @file     datainfo.h
  * @brief    The Data Info tab page and helper classes (declaration).
  *
  * @b begin      : Tue Oct 26 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef DATAINFO_H
#define DATAINFO_H

#include <qwidget.h>
#include <qdict.h>

#include "poplistview.h"

class QPushButton;

/**  A customized QListViewItem for the list view in the Data Info tab.
  */
class DataInfoListViewItem : public ColorListViewItem
{
public:
  /** The constructor.
    *
    * @param parent The parent list view.
    */
  DataInfoListViewItem(QListView* parent) : ColorListViewItem(parent) {}
  /** The destructor. */
  ~DataInfoListViewItem() {}
  /** Reimplemented to allow customized sorting of the items in the list.
    *
    * @param column    The column for which a key is returned.
    * @param ascending Indicates the sorting order (usage not recommended).
    *
    * @return A key that can be used for sorting by column @a column.
    */
  virtual QString key(int column, bool ascending) const;
};


/** The widget for the Data Info tab page.
  *
  * This widget contains a list view with information about the work units
  * currently being processed.
  */
class DataInfo : public QWidget
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  DataInfo(QWidget *parent=0, const char *name=0);
  /** The destructor. */
  ~DataInfo();

public slots:
  /** Refreshes all items in the list view.
    *
    * If corresponding list view item for a %S@h location does not exist,
    * it is created on-the-fly.
    */
  void refreshList();
  /** Updates the private dictionary of %S@h locations.
    *
    * Updates the private list of %S@h locations with the location @a loc. The
    * @a type parameter specifies what to do with @a loc.
    *
    * @param loc  The %S@h location to be updated.
    * @param type The type of action to be performed with @a loc. See
    *             Ksetiwatch::ListId for details.
    */
  void slotUpdateList(SetiLoc* loc, int type);

protected: // methods
  /** Resizes and rearranges widgets in the tab page when the window size
    * changes.
    *
    * A relict from ancient times when there were no layout widgets.
    *
    * @param e An object containing details about the resize event.
    */
  void resizeEvent(QResizeEvent *e);
  /** Reimplemented to tell the list view whether it shall draw grid lines.
    *
    * @param e An object containing details about the paint event.
    */
  void paintEvent(QPaintEvent* e);
  /** Refreshes all columns of the list view item of location @a loc.
    *
    * @param loc A pointer to the SetiLoc object.
    * @param it  A pointer to the list view item to be updated. If 0, the item
    *            will be looked up in the private dictionary.
    */
  void refreshListItem(SetiLoc* loc, DataInfoListViewItem* it=0);

protected slots:
  /** Determines the sender of the signal and refreshes the corresponding item
    * in the Data Info list.
    */
  void slotUpdateWorkUnitData();
  /** Opens the skymap window. */
  void showMap();
  /** Toggles the sorting order of column @a column. */
  void toggleSorting(int column);

protected: // attributes
  /** The list view shown in the Data Info tab page. */
  popListView *QListView_1;
  /** A dictionary that stores all the list view items currently shown in the
    * Data Info tab page. Items are identified by the unique ID of a %S@h
    * location. */
  QDict<DataInfoListViewItem> lvDict;
  /** The button that needs to be clicked to open the skymap window. */
  QPushButton *QPushButton_1;
  bool sortingorder;
  /** Identifiers for the columns in the Data Info list view. */
  enum ColumnIds {ColLocation, ColFrom, ColRecordedOn, ColSource, ColBaseFreq,
                  ColAngleRange, ColTeraFlops};
};

#endif
