/***************************************************************************/
/** @file     dockwidget.cpp
  * @brief    The dock widget that appears in the system tray
  *           (implementation).
  *
  * @b begin      : Sun Nov 28 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <kapp.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kiconloader.h>
#include <kglobalsettings.h>

#include <qtooltip.h>
#include <qpainter.h>
#include <qiconset.h>

#include "dockwidget.h"
#include "setiloc.h"

/*------------------------------------------------------------------------ */
DockWidget::DockWidget(QWidget* parent, const char* name)
          : KSystemTray(parent,name), location(0), docked(false)
{
mini_seti         = UserIcon("mini-seti");
mini_setigrey     = UserIcon("mini-setidockgrey");
mini_setifinished = UserIcon("mini-setidockfinished");
mini_setistopped  = UserIcon("mini-setidockstopped");
mini_setiloading  = UserIcon("mini-setidockloading");
picsmall_pixmap   = UserIcon("ksetiwatch");

contextMenu()->changeTitle(contextMenu()->idAt(0), "Ksetiwatch");	
contextMenu()->insertItem(QIconSet(SmallIcon("mini-setirunning"), QIconSet::Small), i18n("Start Client"), StartClient);
contextMenu()->insertItem(QIconSet(SmallIcon("mini-setistopped"), QIconSet::Small), i18n("Stop Client"), StopClient);
connect(contextMenu(), SIGNAL(aboutToShow()), SLOT(preparePopupMenu()));
connect(contextMenu(), SIGNAL(activated(int)), SLOT(handlePopupCommand(int)));

QToolTip::add(this, "Ksetiwatch");

// initialize and start the timer for a blinking icon
showIcon = true;
// blinking interval is 1 sec
startTimer(1000);
}

/*------------------------------------------------------------------------ */
DockWidget::~DockWidget()
{
}

/*------------------------------------------------------------------------ */
void DockWidget::timerEvent(QTimerEvent*)
{
slotBlinkIcon();
}

/*------------------------------------------------------------------------ */
void DockWidget::dock(SetiLoc* loc)
{
if (!docked)
  {
  // copy the pointer to the SetiLoc location
  location = loc;
  docked = true;
  this->show();
  }
}

/*------------------------------------------------------------------------ */
void DockWidget::undock()
{
if (docked)
  {
  docked = false;
  this->hide();
  }
}

/*------------------------------------------------------------------------ */
void DockWidget::paintEvent(QPaintEvent*)
{
paintIcon(0, 0);
}

/*------------------------------------------------------------------------ */
void DockWidget::updateProgress()
{
if(location)
  {
  if(location->clientState() == SetiLoc::Running)
    updateState(location->clientState(), 0);
  }
}

/*------------------------------------------------------------------------ */
void DockWidget::updateState(int st, int ls)
{
// determine the text to be shown
if(location)
  {
  QString info;
  int state;
  if(st == 0)
    state = location->clientState();
  else
    state = st;

  switch(state)
    {
    case SetiLoc::Stopped:
      if(location->progress() >= 0.0)
        info = i18n("stopped");
      else
        info = QString::null;
      break;
    case SetiLoc::Running:
      info = "-" + location->remainingTimeAsString();
      break;
    case SetiLoc::Loading:
      info = i18n("loading: %1%").arg(ls);
      break;
    case SetiLoc::Finished:
      info = i18n("finished");
      break;
    case SetiLoc::Connecting:
      info = i18n("connecting");
      break;
    default:
      info = QString::null;
      break;
    }
  // update tooltip with "remaining time" info
  QString tip("Ksetiwatch - ");
  tip += location->description();
  if(!info.isEmpty()) tip += " (" + info + ")";
  QToolTip::remove(this);
  QToolTip::add(this, tip);
  // make a backup of the loading progress
  loadingState = ls;
  }
else
  {
  QToolTip::remove(this);
  QToolTip::add(this, "Ksetiwatch");
  }
paintIcon(st, ls);
}

/*------------------------------------------------------------------------ */
void DockWidget::paintIcon(int st, int ls)
{
int x(0),y(0);

//clear the widget first
erase();
if(location == 0) // no location selected
  {
  QSize s(picsmall_pixmap.size());
  if(s.width() < 24) x = (24 - s.width())/2;
  if(s.height() < 24) y = (24 - s.height())/2;
  bitBlt(this, x, y, &picsmall_pixmap);
  }
else //location selected, show details
  {
  QPixmap* icon;

  int state;
  if(st == 0)
    state = location->clientState();
  else
    state = st;

  if(showIcon)
    {
    // determine the icon to be shown
    switch(state)
      {
      case SetiLoc::Stopped:
        icon = &mini_setistopped;
        break;
      case SetiLoc::Running:
        icon = &mini_seti;
        break;
      case SetiLoc::Loading:
        icon = &mini_setiloading;
        break;
      case SetiLoc::Finished:
        icon = &mini_setifinished;
        break;
      case SetiLoc::Connecting:
        // in the Connecting state icon toggles between Loading and Finished
        icon = &mini_setiloading;
        break;
      default:
        icon = &mini_setigrey;
        break;
      }
    }
  else
    {
    if(state == SetiLoc::Connecting)
      icon = &mini_setifinished;
    else
      icon = &mini_setigrey;
    }
  // determine the text to be shown
  QString  prog;
  double   progVal;
  switch(state)
    {
    case SetiLoc::Stopped:
      if(location->progress() >= 0.0)
        progVal = location->progress();
      else
        progVal = -1.0;
      break;
    case SetiLoc::Running:
      progVal = location->progress();
      break;
    case SetiLoc::Loading:
      progVal = (double)ls;
      break;
    case SetiLoc::Finished:
      progVal = 100.0;
      break;
    case SetiLoc::Connecting:
      progVal = 0.0;
      break;
    default:
      // don't show any text
      progVal = -1.0;
      break;
    }
  if(progVal >= 0.0)
    prog.sprintf("%.0f%%", progVal);
  else
    prog = QString::null;

  QSize s(icon->size());
  if(s.width() < 24) x = (24 - s.width())/2;
  bitBlt(this, x, 2, icon);
  // draw the progress bar
  QRect pbar(0, 14, 23, 8);
  QPainter p(this);
  p.fillRect(pbar, darkGray);
  p.setPen(QColor(200, 200, 200));
  p.moveTo(0, 21);
  p.lineTo(23, 21);
  p.lineTo(23, 14);
  if(progVal >= 0.0)
    p.fillRect(0, 15, ((int)(progVal*24)/100), 6, QColor(180,100,0));
  p.setPen(white);

  QFont font = KGlobalSettings::fixedFont();
  font.setPointSize(6);
  p.setFont(font);
  p.drawText( 0, 15, 23, 6, AlignCenter, prog);
  }
}

/*------------------------------------------------------------------------ */
void DockWidget::handlePopupCommand(int id)
{
switch(id)
  {
  case StartClient:
    if(location) location->startClient();
    break;
  case StopClient:
    if(location) location->stopClient();
    break;
  }
}

/*------------------------------------------------------------------------ */
void DockWidget::preparePopupMenu()
{
// disable start/stop client first
contextMenu()->setItemEnabled(StartClient, false);
contextMenu()->setItemEnabled(StopClient, false);
if(location)
  {
  switch(location->clientState())
    {
    case SetiLoc::Running:
    case SetiLoc::Loading:
      // enable stop
      contextMenu()->setItemEnabled(StopClient, true);
      break;
    case SetiLoc::Stopped:
      // enable start
      contextMenu()->setItemEnabled(StartClient, true);
      break;
    case SetiLoc::Finished:
      if(location->isClientRunning() == true)
        contextMenu()->setItemEnabled(StopClient, true);
      else
        contextMenu()->setItemEnabled(StartClient, true);
      break;
    }
  }
}

/*------------------------------------------------------------------------ */
void DockWidget::updateDockWidget(SetiLoc* loc)
{
// disconnect any previous connection
if(location != loc && location != 0)
  {
  disconnect(location, SIGNAL(progressIncreased()),
             this,     SLOT(updateProgress()));
  disconnect(location, SIGNAL(stateChanged(int,int)),
             this,     SLOT(updateState(int,int)));
  }
// react on changes of the status
if(location != loc)
  {
  location = loc;
  if(location)
    {
    connect(location, SIGNAL(progressIncreased()),
            this,     SLOT(updateProgress()));
    connect(location, SIGNAL(stateChanged(int,int)),
            this,     SLOT(updateState(int,int)));
    }
  }
updateState(0, 0);
}

/*------------------------------------------------------------------------ */
void DockWidget::slotBlinkIcon()
{
if(location)
  {
  if(location->clientState() != SetiLoc::Running)
    {
    showIcon ^= true;
    paintIcon(0, loadingState);
    }
  else
    {
    if(showIcon == false)
      {
      showIcon = true;
      paintIcon(0, 0);
      }
    }
  }
}

#include "dockwidget.moc"

