/***************************************************************************/
/** @file     dockwidget.h
  * @brief    The dock widget that appears in the system tray (declaration).
  *
  * @b begin      : Sun Nov 28 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef DOCKWIDGET_H
#define DOCKWIDGET_H

#include <ksystemtray.h>

class QWidget;
class QPixmap;
class SetiLoc;

/** The dock widget for the system tray.
  *
  * If `Minimize to Panel' is enabled, this widget will be shown in the
  * system tray area of the Kicker panel. It either displays the standard
  * Ksetiwatch icon, or, if a location is selected in the Analysis tab, an
  * icon showing the progress and state of said location. In the latter
  * case, the client can be started or stopped via the right-click context
  * menu.
  */
class DockWidget : public KSystemTray
{
   Q_OBJECT
public: 
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level widget.
    * @param name   A text that can be used to identify this widget.
    */
  DockWidget(QWidget *parent=0, const char *name=0);
  /** The destructor. */
  ~DockWidget();
  /** Returns the status of the dock widget. TRUE if it is docked (and visible),
    * otherwise FALSE.
    */
  const bool isDocked() {return docked;}

public slots:
  /** Shows the dock widget.
    *
    * @param loc The %S@h location for which progress information is shown.
    *            If 0, the generic Ksetiwatch icon is displayed.
    */
  void dock(SetiLoc* loc=0);
  /** Hides the dock widget. */
  void undock();
  /** Connects the dock widget to another location.
    *
    * @param loc The SetiLoc object of the new %S@h location. If 0, any
    *            previous connections to the old location are removed.
    */
  void updateDockWidget(SetiLoc* loc);

protected: // methods
  /** Reimplemented for customized painting of the dock widget.
    *
    * @param e Contains event parameters of the paint event.
    */
  void paintEvent(QPaintEvent *e);
  /** Reimplemented to catch internal timer events.
    *
    * The internal timer generates an event every second in order to have
    * a blinking icon in case the state of the location is Stopped,
    * Loading, or Finished.
    *
    * @param e Contains event parameter of the timer event.
    */
  void timerEvent(QTimerEvent* e);

private slots:
  /** Executes the selected command from the right-click popup menu of the
    * dock widget.
    *
    * @param id The unique ID number of the selected popup menu item. See
    *           DockWidget::popupIDs for details.
    */
  void handlePopupCommand(int id);
  /** Prepares the right-click popup menu just before showing it.
    *
    * Checks if the popup menu commands are allowed or make sense in the
    * current situation, and enables or disables the menu items appropriately.
    */
  void preparePopupMenu();
  /** Paints the dock widget.
    *
    * If the dock widget is connected to a %S@h location, progress and state
    * of said location is shown, otherwise the default Ksetiwatch icon is
    * displayed.
    */
  void paintIcon(int st, int ls);
  /** Performs blinking of the state icon.
    *
    * This slot is called whenever an internal timer event occurs. It causes
    * the state icon in the dock widget to periodically toggle between two
    * pixmaps so as to provide a visual feedback. Blinking occurs only if
    * the state of the location is Stopped, Loading, or Finished.
    */
  void slotBlinkIcon();
  /** Updates the progress information in the dock widget.
    *
    * The progress is shown as a mini progress bar.
    */
  void updateProgress();
  /** Updates the state information in the dock widget.
    *
    * A running client is represented by a small green telescope dish, all
    * other states are displayed as blinking icons.
    *
    * @param st The state of the client. If 0, the state is queried directly
    *           from the corresponding SetiLoc object.
    * @param ls Not used in this context.
    */
  void updateState(int st, int ls);

private:
  SetiLoc* location;
  bool docked;
  int newID;
  QPixmap mini_seti;
  QPixmap mini_setigrey;
  QPixmap mini_setistopped;
  QPixmap mini_setifinished;
  QPixmap mini_setiloading;
  QPixmap picsmall_pixmap;
  bool showIcon;
  int loadingState;
  /** Identifiers for the menu items in the right-click popup menu. */
  enum popupIDs {StartClient, StopClient};
};

#endif
