/***************************************************************************/
/** @file     gaussianplot.cpp
  * @brief    A widget that displays a Gaussian profile (implementation).
  *
  * @b begin      : Tue Mar 28 2000<br>
  * @b copyright  : &copy; 2000-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <klocale.h>

#include <qpainter.h>

#include <math.h>

#include "gaussianplot.h"
#include "setiloc.h"

/** The number of data points calculated for the fit curve. The higher the
  * number the smoother the curve. */
#define FITSIZE 128

/*------------------------------------------------------------------------ */
GaussianPlot::GaussianPlot(SetiLoc* loc, QWidget *parent, const char *name )
            : SetiGraph(loc,true,true,parent,name)
{
init();
}

/*------------------------------------------------------------------------ */
GaussianPlot::GaussianPlot(QWidget *parent, const char *name )
            : SetiGraph(0,true,true,parent,name)
{
init();
}

/*------------------------------------------------------------------------ */
GaussianPlot::~GaussianPlot()
{
}

/*------------------------------------------------------------------------ */
void GaussianPlot::init()
{
if(m_location)
  {
  // connect the SetiLoc signal to the new Gaussian slot of this widget.
  connect(m_location, SIGNAL(newGaussian(GaussianScore)),
                this, SLOT(showNewGaussianData(GaussianScore)));

  QString title;
  title = i18n("Location %1: Top Gaussian (right-click for options)").arg(m_location->description());
  setCaption(title);
  }
setBackgroundColor(black);
resize(350, 200);
setMinimumSize(150, 100);

dataLoaded = false;

// prepare data for display
if(m_location)
  showNewGaussianData(m_location->gaussianScore());
}

/*------------------------------------------------------------------------ */
void GaussianPlot::paintWidget(QPaintDevice* pd)
{
QPainter p;

SetiGraph::paintWidget(pd);
p.begin(pd);
p.translate(0, rect().bottom());
if(dataLoaded)
  drawCurve(fitData, SetiGraph::Lines, white, &p);
p.end();
}

/*------------------------------------------------------------------------ */
void GaussianPlot::calculateFitData()
{
double ratio = FITSIZE/64.0;
for(int i=0;i<FITSIZE;i++)
  {
  fitData[i] = gaussian.true_mean +
    gaussian.power*exp(-((i/ratio)-gaussian.fft_index)*((i/ratio)-gaussian.fft_index)/
    (gaussian.sigma*gaussian.sigma));
  }
}

/*------------------------------------------------------------------------ */
void GaussianPlot::showNewGaussianData(GaussianScore score)
{
gaussian = score;
setCurve((const double*)gaussian.data, 64);

// generate the info text
QString it;
it.sprintf("power: %f\n"
           "mean : %f\n"
           "sigma: %f\n"
           "fit  : %f\n"
           "score: %f",
           gaussian.power,
           gaussian.true_mean,
           gaussian.sigma,
           gaussian.chisq,
           gaussian.score
           );
setInfoText(it);

if(fitData.size() != FITSIZE) fitData.resize(FITSIZE);
calculateFitData();
dataLoaded = true;
repaint();
}

#include "gaussianplot.moc"
