/***************************************************************************/
/** @file     gaussianplot.h
  * @brief    A widget that displays a Gaussian profile (declaration).
  *
  * @b begin      : Tue Mar 28 2000<br>
  * @b copyright  : &copy; 2000-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef GAUSSIANPLOT_H
#define GAUSSIANPLOT_H

#include "setigraph.h"
#include "setidefs.h"

/** Displays a Gaussian profile.
  *
  * This object can be constructed in two ways: either with or without
  * specifying a %S@h location. If a SetiLoc object is passed to the
  * constructor of this class, the Gaussian profile shown in the widget
  * always represents the best Gaussian the %S@h client has found in the
  * work unit it processes at that time. The Gaussian profile is
  * automatically updated whenever the client finds a new and better Gaussian
  * signal.
  *
  * If this widget is created without a %S@h location, the Gaussian data
  * have to be explicitly passed with the showNewGaussianData() method.
  */
class GaussianPlot : public SetiGraph
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  GaussianPlot(QWidget* parent=0, const char* name=0);
  /** The constructor.
    *
    * @overload
    *
    * @param loc    The %S@h location for which the best Gaussian (found in
    *               the currently processed work unit) shall be shown.
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  GaussianPlot(SetiLoc* loc, QWidget* parent=0, const char* name=0);
  /** The destructor. */
  ~GaussianPlot();

public slots:
  /** Updates the widget with new Gaussian data.
    *
    * @param score A structure containing the parameters of the new Gaussian
    *              to be shown.
    */
  void showNewGaussianData(GaussianScore score);

protected: // methods
  /** Calculates the Gaussian profile data according to fit parameters. */
  void calculateFitData();
  /** Paints the widget.
    *
    * Reimplemented for customized painting of this widget (drawing the
    * additional fit curve).
    *
    * @param pd A pointer to the object that shall be painted.
    */
  virtual void paintWidget(QPaintDevice* pd);
  /** Initializes the widget.
    *
    * If a %S@h location is given, the Gaussian profile is painted, and a
    * connection is made to the SetiLoc::newGaussian signal of the
    * corresponding SetiLoc object.
    */
  void init();

protected: // attributes
  /** This array stores the data of the fit curve. */
  QMemArray<double> fitData;
  /** A structure containing the Gaussian parameters. */
  GaussianScore gaussian;
  /** Indicates if the data of a Gaussian is loaded. */
  bool dataLoaded;
};

#endif
