/***************************************************************************/
/** @file     ksetiwatch.cpp
  * @brief    The main window widget (implementation).
  *
  * @b begin      : Fre Okt 22 17:39:47 CEST 1999<br>
  * @b copyright  : &copy; 1999-2003 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <kapp.h>
#include <klocale.h>
#include <kstatusbar.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kstddirs.h>
#include <ktabctl.h>
#include <kconfig.h>

#include <qlabel.h>
#include <qmessagebox.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qpopupmenu.h>

#include <unistd.h>

#include "analysis.h"
#include "datainfo.h"
#include "userinfo.h"
#include "completedwulist.h"
#include "options.h"
#include "about.h"
#include "dockwidget.h"
#include "setiloc.h"
#include "ksetiwatch.h"

/*------------------------------------------------------------------------ */
KsetiwatchApp::KsetiwatchApp(bool allowStyles, bool GUIEnabled)
              : KApplication(allowStyles, GUIEnabled), sessionClose(false)
{
}

/*------------------------------------------------------------------------ */
KsetiwatchApp::~KsetiwatchApp()
{
}

/*------------------------------------------------------------------------ */
void KsetiwatchApp::commitData(QSessionManager& sm)
{
sessionClose = true;
KApplication::commitData(sm);
}

/////////////////////////////////////////////////////////////////////////////

QPtrList<SetiLoc> Ksetiwatch::SetiList;
KswOptions        Ksetiwatch::GlobalOptions;

/*------------------------------------------------------------------------ */
Ksetiwatch::Ksetiwatch(const char* name)
           : KMainWindow(0, name), readyToQuit(false), locListCleared(false)

{
QString tlabel[6] = {
                    i18n("Analysis"),
                    i18n("Data Info"),
                    i18n("User Info"),
                    i18n("Completed Work Units"),
                    i18n("Options"),
                    i18n("About")
                    };

// Automatcally delete SetiLoc objects when removing them from the list.
locationList().setAutoDelete(true);

SetiContainer::initWUScore(&globalopts->Record);
readConfig();

QPixmap iconpic;
iconpic = DesktopIcon("ksetiwatch");
setIcon(iconpic);

// KDE 2.1: window title must be given explicitly
setPlainCaption("Ksetiwatch");

// status bar must be initialized before setting the central widget !
statusBar()->insertItem("", 0, 1);
statusBar()->setItemAlignment(0, AlignLeft|AlignVCenter);

ktb = new KTabCtl(this);
ana = new Analysis(ktb);
ana->setMinimumSize(640, 130);
ktb->addTab(ana, tlabel[0]);
di = new DataInfo(ktb);
di->setMinimumSize(640, 130);
ktb->addTab(di, tlabel[1]);
ui = new UserInfo(ktb);
ui->setMinimumSize(640, 130);
ktb->addTab(ui, tlabel[2]);
cwu = new CompletedWUList(ktb);
cwu->setMinimumSize(640, 130);
ktb->addTab(cwu, tlabel[3]);
opt = new Options(ktb);
opt->setMinimumSize(640, 130);
ktb->addTab(opt, tlabel[4]);
abt = new About(ktb);
abt->setMinimumSize(640, 130);
ktb->addTab(abt, tlabel[5]);
setCentralWidget(ktb);

connect(ktb, SIGNAL(tabSelected(int)), this, SLOT(handleTabSelection(int)));

connect(ana,  SIGNAL(listChangeRequested(SetiLoc*, int)),
        this, SLOT(slotUpdateList(SetiLoc*, int)));
connect(this, SIGNAL(listModified(SetiLoc*, int)),
        ana,  SLOT(slotUpdateList(SetiLoc*, int)));
connect(this, SIGNAL(listModified(SetiLoc*, int)),
        di,   SLOT(slotUpdateList(SetiLoc*, int)));
connect(this, SIGNAL(listModified(SetiLoc*, int)),
        ui,   SLOT(slotUpdateList(SetiLoc*, int)));
connect(this, SIGNAL(listModified(SetiLoc*, int)),
        cwu,  SLOT(slotUpdateList(SetiLoc*, int)));

connect(cwu, SIGNAL(listUpdated()), this, SLOT(saveConfig()));

startClients();

ksdock = new DockWidget(this);
connect(ana, SIGNAL(selectionChanged(SetiLoc*)), ksdock, SLOT(updateDockWidget(SetiLoc*)));
connect(opt, SIGNAL(dockStatusChanged()), this, SLOT(toggleIcon()));
// if ksetiwatch was closed in minimized state start it minimized again
if(globalopts->MinimizeToPanel)
  ksdock->dock();

// if a location has been selected the last time, restore it
if(!globalopts->SelectedLocation.isEmpty())
  {
  SetiLoc* loc = getLocation(globalopts->SelectedLocation);
  ana->setSelectedLocation(loc->id());
  }
}

/*------------------------------------------------------------------------ */
Ksetiwatch::~Ksetiwatch()
{
}

/*------------------------------------------------------------------------ */
bool Ksetiwatch::queryClose()
{
if(readyToQuit)
  {
  stopClients();
  return true;
  }
else
  {
  this->hide();
  return false;
  }
}

/*------------------------------------------------------------------------ */
bool Ksetiwatch::queryExit()
{
// When closed by the session manager, this method is called twice (the
// second time via an emitted shuttingDown signal). The boolean switch
// locListCleared ensures that saveConfig() is called only once and
// before the location list is cleared.
if(!locListCleared)
  {
  saveConfig();
  // clear the list, otherwise there may be a segfault on exit
  locationList().clear();
  locListCleared = true;
  }
return true;
}

/*------------------------------------------------------------------------ */
void Ksetiwatch::close()
{
readyToQuit = true;
KMainWindow::close();
}

/*------------------------------------------------------------------------ */
bool Ksetiwatch::event(QEvent *e)
{
switch( (int)e->type() )
  {
  case QEvent::Hide:
    {
    if(globalopts->MinimizeToPanel)
      {
      SetiLoc* loc = ana->selectedLocation();
      ksdock->dock(loc);
      this->hide();
      }
    globalopts->StartMinimized = true;
    kapp->config()->writeEntry("StartMinimized", globalopts->StartMinimized);
    break;
    }
  case QEvent::Show:
    {
    globalopts->StartMinimized = false;
    kapp->config()->writeEntry("StartMinimized", globalopts->StartMinimized);
    break;
    }
  case QEvent::Close:
    {
    KsetiwatchApp* kswApp = dynamic_cast<KsetiwatchApp*>(kapp);
    if(!globalopts->MinimizeToPanel || kswApp->closedBySessionManager())
      {
      readyToQuit = true;
      saveConfig();
      }
    break;
    }
  }
return KMainWindow::event( e );
}

/*------------------------------------------------------------------------ */
void Ksetiwatch::saveConfig()
{
QString entry;
SetiLoc *loc;

KConfig* config = kapp->config();

config->setGroup("General Options");
QRect r(pos().x(), pos().y(), size().width(), size().height());
config->writeEntry("WindowSize", r);
config->writeEntry("RefreshInterval", globalopts->RefreshInterval*1000);
config->writeEntry("TimeFormat", globalopts->hms);
config->writeEntry("ReportSpike", globalopts->ReportSpike);
config->writeEntry("ReportGaussian", globalopts->ReportGaussian);
config->writeEntry("ReportPulse", globalopts->ReportPulse);
config->writeEntry("ReportTriplet", globalopts->ReportTriplet);
config->writeEntry("ShowOnlyRecords", globalopts->ShowRecords);
config->writeEntry("MinimizeToPanel", globalopts->MinimizeToPanel);
config->writeEntry("StartMinimized", globalopts->StartMinimized);
config->writeEntry("DrawGrid", globalopts->DrawGrid);
config->writeEntry("MarkerSize", globalopts->MarkerSize);
config->writeEntry("FillMarker", globalopts->FillMarker);
config->writeEntry("ShowText", globalopts->ShowText);
config->writeEntry("PlaySound", globalopts->PlaySound);
config->writeEntry("Sound_HighSignal", globalopts->HighSignalSound);
loc = ana->selectedLocation();
config->writeEntry("SelectedLocation", loc ? loc->description() : QString::null);

config->setGroup("Seti Locations");
int count = Ksetiwatch::locationList().count();
if(count > 0)
  {
  config->writeEntry("count", count);
  int i = 1;
  for( loc=locationList().first(); loc != 0; loc=locationList().next() )
    {
    LocSettings ls = loc->options();
    entry.sprintf("Location_%d",i);
    config->writeEntry(entry, ls.location.description);
    entry.sprintf("Directory_%d",i);
    config->writeEntry(entry, ls.location.directory);
    entry.sprintf("UseCustomStartExe_%d", i);
    config->writeEntry(entry, ls.location.useCustomStartExe);
    entry.sprintf("CustomStartExe_%d", i);
    config->writeEntry(entry, ls.location.customStartExe);
    entry.sprintf("UseCustomStopExe_%d", i);
    config->writeEntry(entry, ls.location.useCustomStopExe);
    entry.sprintf("CustomStopExe_%d", i);
    config->writeEntry(entry, ls.location.customStopExe);
    entry.sprintf("Color_%d",i);
    config->writeEntry(entry, ls.location.color);
    entry.sprintf("Result_Timestamp_%d",i);
    config->writeEntry(entry, loc->timestamp());
    entry.sprintf("Startup_%d",i);
    config->writeEntry(entry, ls.client.startSeti);
    entry.sprintf("StopAtLogout_%d", i);
    config->writeEntry(entry, ls.client.stopSeti);
    entry.sprintf("KeepClientAlive_%d", i);
    config->writeEntry(entry, ls.client.keepClientAlive);
    entry.sprintf("CommandLine_Arguments_%d",i);
    config->writeEntry(entry, ls.client.arguments);
    entry.sprintf("Arg_StopAfterProcess_%d", i);
    config->writeEntry(entry, ls.client.stopAfterProcess);
    entry.sprintf("Arg_StopAfterSend_%d", i);
    config->writeEntry(entry, ls.client.stopAfterSend);
    entry.sprintf("Arg_Email_%d", i);
    config->writeEntry(entry, ls.client.email);
    entry.sprintf("Arg_Graphics_%d", i);
    config->writeEntry(entry, ls.client.graphics);
    entry.sprintf("Priority_%d", i);
    config->writeEntry(entry, ls.client.priority);
    entry.sprintf("UseProxy_%d", i);
    config->writeEntry(entry, ls.client.useProxy);
    entry.sprintf("ProxyServer_%d", i);
    config->writeEntry(entry, ls.client.proxyServer);
    entry.sprintf("ProxyPort_%d", i);
    config->writeEntry(entry, ls.client.proxyPort);
    entry.sprintf("LogWU_%d",i);
    config->writeEntry(entry, ls.logging.log);
    entry.sprintf("RedirLog_%d", i);
    config->writeEntry(entry, ls.logging.redirectLog);
    entry.sprintf("LogDirectory_%d", i);
    config->writeEntry(entry, ls.logging.logDirectory);
    i++;
    }
  }
config->setGroup("Record Scores");
// this is to avoid rounding errors
entry.sprintf("%f", globalopts->Record.spike.power);
config->writeEntry("Spike", entry);
config->writeEntry("Spike_ChirpRate", globalopts->Record.spike.chirprate);
config->writeEntry("Spike_WUName", globalopts->Record.spike.wu_name);
// this is to avoid rounding errors
entry.sprintf("%f", globalopts->Record.gaussian.score);
config->writeEntry("Gaussian", entry);
config->writeEntry("Gaussian_Power", globalopts->Record.gaussian.power);
config->writeEntry("Gaussian_Chisq", globalopts->Record.gaussian.chisq);
config->writeEntry("Gaussian_ChirpRate", globalopts->Record.gaussian.chirprate);
config->writeEntry("Gaussian_WUName", globalopts->Record.gaussian.wu_name);
// this is to avoid rounding errors
entry.sprintf("%f", globalopts->Record.pulse.score);
config->writeEntry("Pulse", entry);
config->writeEntry("Pulse_Power", globalopts->Record.pulse.power);
config->writeEntry("Pulse_Period", globalopts->Record.pulse.period);
config->writeEntry("Pulse_ChirpRate", globalopts->Record.pulse.chirprate);
config->writeEntry("Pulse_WUName", globalopts->Record.pulse.wu_name);
// this is to avoid rounding errors
entry.sprintf("%f", globalopts->Record.triplet.score);
config->writeEntry("Triplet", entry);
config->writeEntry("Triplet_Power", globalopts->Record.triplet.power);
config->writeEntry("Triplet_Period", globalopts->Record.triplet.period);
config->writeEntry("Triplet_ChirpRate", globalopts->Record.triplet.chirprate);
config->writeEntry("Triplet_WUName", globalopts->Record.triplet.wu_name);

// write the config to disk immediately and not at shutdown
config->sync();
}

/*------------------------------------------------------------------------ */
void Ksetiwatch::readConfig()
{
QString id;

KConfig* config = kapp->config();

config->setGroup("General Options");
QRect sdefault(0, 0, 640, 165);
QRect s = config->readRectEntry("WindowSize", &sdefault);
this->setGeometry(s);
globalopts->RefreshInterval   = config->readNumEntry("RefreshInterval", 15000);
// convert to seconds
globalopts->RefreshInterval  /= 1000;
globalopts->hms               = config->readBoolEntry("TimeFormat", true);
globalopts->MinimizeToPanel   = config->readBoolEntry("MinimizeToPanel", false);
globalopts->DrawGrid          = config->readBoolEntry("DrawGrid", true);
globalopts->StartMinimized    = config->readBoolEntry("StartMinimized", false);
globalopts->ReportSpike       = config->readBoolEntry("ReportSpike", false);
globalopts->ReportGaussian    = config->readBoolEntry("ReportGaussian", false);
globalopts->ReportPulse       = config->readBoolEntry("ReportPulse", false);
globalopts->ReportTriplet     = config->readBoolEntry("ReportTriplet", false);
globalopts->ShowRecords       = config->readBoolEntry("ShowOnlyRecords", false);
globalopts->PlaySound         = config->readBoolEntry("PlaySound", false);
globalopts->HighSignalSound   =
  config->readEntry("Sound_HighSignal", kapp->dirs()->findResource("data", "ksetiwatch/sounds/message.wav"));
globalopts->MarkerSize        = config->readNumEntry("MarkerSize", 2);
globalopts->FillMarker        = config->readBoolEntry("FillMarker", true);
globalopts->ShowText          = config->readBoolEntry("ShowText", true);
globalopts->SelectedLocation  = config->readEntry("SelectedLocation", "");

config->setGroup("Seti Locations");
int count = config->readNumEntry("count");

if(count > 0)
  {
  for(int i=1;i<=count;i++)
    {
    LocSettings ls;
    id.sprintf("Directory_%d", i);
    ls.location.directory = config->readEntry(id);
    id.sprintf("Location_%d", i);
    ls.location.description = config->readEntry(id);
    id.sprintf("UseCustomStartExe_%d", i);
    ls.location.useCustomStartExe = config->readBoolEntry(id, false);
    id.sprintf("CustomStartExe_%d", i);
    ls.location.customStartExe = config->readEntry(id, "./setiathome");
    id.sprintf("UseCustomStopExe_%d", i);
    ls.location.useCustomStopExe = config->readBoolEntry(id, false);
    id.sprintf("CustomStopExe_%d", i);
    ls.location.customStopExe = config->readEntry(id, QString::null);
    id.sprintf("Color_%d", i);
    ls.location.color = config->readColorEntry(id);
    id.sprintf("Startup_%d", i);
    ls.client.startSeti = config->readBoolEntry(id, false);
    id.sprintf("StopAtLogout_%d", i);
    ls.client.stopSeti = config->readBoolEntry(id, false);
    id.sprintf("KeepClientAlive_%d", i);
    ls.client.keepClientAlive = config->readBoolEntry(id, false);
    id.sprintf("CommandLine_Arguments_%d", i);
    ls.client.arguments = config->readEntry(id, "");
    id.sprintf("Arg_StopAfterProcess_%d", i);
    ls.client.stopAfterProcess = config->readBoolEntry(id, false);
    id.sprintf("Arg_StopAfterSend_%d", i);
    ls.client.stopAfterSend = config->readBoolEntry(id, false);
    id.sprintf("Arg_Email_%d", i);
    ls.client.email = config->readBoolEntry(id, false);
    id.sprintf("Arg_Graphics_%d", i);
    ls.client.graphics = config->readBoolEntry(id, false);
    id.sprintf("Priority_%d", i);
    ls.client.priority = config->readNumEntry(id, 1);
    id.sprintf("UseProxy_%d", i);
    ls.client.useProxy = config->readBoolEntry(id, false);
    id.sprintf("ProxyServer_%d", i);
    ls.client.proxyServer = config->readEntry(id, "");
    id.sprintf("ProxyPort_%d", i);
    ls.client.proxyPort = config->readNumEntry(id, 8080);
    id.sprintf("LogWU_%d", i);
    ls.logging.log = config->readBoolEntry(id, true);
    id.sprintf("RedirLog_%d", i);
    ls.logging.redirectLog = config->readBoolEntry(id, false);
    id.sprintf("LogDirectory_%d", i);
    ls.logging.logDirectory = config->readEntry(id, "");
    // Create a new S@h location...
    SetiLoc *loc = new SetiLoc(ls.location.directory, ls.location.description,
                               ls.location.color);
    // ...and set the rest of the options.
    loc->setOptions(ls);
    loc->setRefreshInterval(globalopts->RefreshInterval);
    // Due to a bug in the kdelibs prior to version 2.1 setGroup()
    // has to be called here once again.
    config->setGroup("Seti Locations");
    id.sprintf("Result_Timestamp_%d", i);
    loc->setTimestamp(config->readEntry(id, ""));
    locationList().append(loc);
    }
  }

config->setGroup("Record Scores");
globalopts->Record.spike.power        = config->readDoubleNumEntry("Spike", 0.0);
globalopts->Record.spike.chirprate    = config->readDoubleNumEntry("Spike_ChirpRate", 0.0);
globalopts->Record.spike.wu_name      = config->readEntry("Spike_WUName", "unknown");
globalopts->Record.gaussian.score     = config->readDoubleNumEntry("Gaussian", 0.0);
globalopts->Record.gaussian.power     = config->readDoubleNumEntry("Gaussian_Power", 0.0);
globalopts->Record.gaussian.chisq     = config->readDoubleNumEntry("Gaussian_Chisq", 0.0);
globalopts->Record.gaussian.chirprate = config->readDoubleNumEntry("Gaussian_ChirpRate", 0.0);
globalopts->Record.gaussian.wu_name   = config->readEntry("Gaussian_WUName", "unknown");
globalopts->Record.pulse.score        = config->readDoubleNumEntry("Pulse", 0.0);
globalopts->Record.pulse.power        = config->readDoubleNumEntry("Pulse_Power", 0.0);
globalopts->Record.pulse.period       = config->readDoubleNumEntry("Pulse_Chisq", 0.0);
globalopts->Record.pulse.chirprate    = config->readDoubleNumEntry("Pulse_ChirpRate", 0.0);
globalopts->Record.pulse.wu_name      = config->readEntry("Pulse_WUName", "unknown");
globalopts->Record.triplet.score      = config->readDoubleNumEntry("Triplet", 0.0);
globalopts->Record.triplet.power      = config->readDoubleNumEntry("Triplet_Power", 0.0);
globalopts->Record.triplet.period     = config->readDoubleNumEntry("Triplet_Chisq", 0.0);
globalopts->Record.triplet.chirprate  = config->readDoubleNumEntry("Triplet_ChirpRate", 0.0);
globalopts->Record.triplet.wu_name    = config->readEntry("Triplet_WUName", "unknown");
}

/*------------------------------------------------------------------------ */
void Ksetiwatch::slotUpdateList(SetiLoc* loc, int type)
{
if(loc)
  {
  switch(type)
    {
    case ListAdd:
      locationList().append(loc);
      break;
    case ListEdit:
      break;
    case ListDelete:
      locationList().remove(loc);
      break;
    }
  emit listModified(loc, type);
  }
}

/*------------------------------------------------------------------------ */
void Ksetiwatch::handleTabSelection(int id)
{
switch(id)
  {
  case 0:
    statusBar()->changeItem(i18n("Press the right mouse button to edit the list."), 0);
    break;
  case 1:
    statusBar()->changeItem(i18n("Ready."), 0);
    break;
  case 2:
    statusBar()->changeItem(i18n("Ready."), 0);
    break;
  case 3: // Completed Work Units
    statusBar()->changeItem(i18n("Right click for options."), 0);
    break;
  default:
    statusBar()->changeItem(i18n("Ready."), 0);
    break;
  }
}

/*------------------------------------------------------------------------ */
void Ksetiwatch::startClients()
{
SetiLoc* loc;

for( loc=Ksetiwatch::locationList().first(); loc != 0;
     loc=Ksetiwatch::locationList().next() )
  {
  if(loc->optionStartSeti())
    loc->startClient();
  }
}

/*------------------------------------------------------------------------ */
void Ksetiwatch::stopClients()
{
SetiLoc* loc;

for( loc=Ksetiwatch::locationList().first(); loc != 0;
     loc=Ksetiwatch::locationList().next() )
  {
  if(loc->optionStopSeti() && loc->isClientRunning(true))
    loc->stopClient();
  }
}

/*------------------------------------------------------------------------ */
SetiLoc* Ksetiwatch::getLocation(const QString& name)
{
SetiLoc* loc = 0;
QPtrList<SetiLoc> seli = Ksetiwatch::locationList();

// browse through the list
for(loc=seli.first(); loc != 0; loc=seli.next() )
  {
  if(loc->description() == name) break;
  }
return(loc);
}

/*------------------------------------------------------------------------ */
void Ksetiwatch::toggleIcon()
{
if(globalopts->MinimizeToPanel)
  ksdock->dock(ana->selectedLocation());
else
  ksdock->undock();
}

#include "ksetiwatch.moc"

