/***************************************************************************/
/** @file     ksetiwatch.h
  * @brief    The main window widget (declaration).
  *
  * @b begin      : Fre Okt 22 17:39:47 CEST 1999<br>
  * @b copyright  : &copy; 1999-2003 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef KSETIWATCH_H
#define KSETIWATCH_H

#include <qptrlist.h>

#include <kmainwindow.h>

#include "setidefs.h"

class KTabCtl;
class QListViewItem;
class QPopupMenu;
class Analysis;
class DataInfo;
class UserInfo;
class CompletedWUList;
class Options;
class About;
class DockWidget;
class SetiLoc;

/** All %application-wide options in one class. */
class KswOptions
{
public:
  int     RefreshInterval;    /**< The refresh interval in seconds. Ksetiwatch
                                   looks every @a RefreshInterval seconds into
                                   the state files of the client, and updates
                                   its data structures appropriately. */
  int     StatusFilterValue;  /**< The state of the combo box in the Analysis
                                   tab filtering the locations by their
                                   processing state. */
  bool    hms;                /**< If TRUE, time is displayed in
                                   hours:minutes:seconds form, otherwise
                                   in days:hours:minutes:seconds. */
  bool    MinimizeToPanel;    /**< If TRUE, Ksetiwatch will be minimized to
                                   the system tray on start-up. */
  bool    StartMinimized;     /**< If TRUE, Ksetiwatch will be minimized on
                                   start-up. */
  bool    DrawGrid;           /**< If TRUE, a grid is drawn in all list
                                   views. */
  QString SelectedLocation;   /**< This string stores the name of the %S@h
                                   location that was selected in the Analysis
                                   tab when closing Ksetiwatch. This location
                                   will be restored on the next start-up. */
  bool    ReportSpike;        /**< If TRUE, message boxes about new high
                                   spikes will be shown. */
  bool    ReportGaussian;     /**< If TRUE, message boxes about new high
                                   gaussians will be shown. */
  bool    ReportPulse;        /**< If TRUE, message boxes about new high
                                   pulses will be shown. */
  bool    ReportTriplet;      /**< If TRUE, message boxes about new high
                                   triplets will be shown. */
  bool    ShowRecords;        /**< If TRUE, only new record signals will be
                                   reported via the message boxes. */
  WUScore Record;             /**< This structure stores the data of the
                                   overall record signals. */

  bool    ShowText;           /**< If TRUE, some text about the WU is shown
                                   on the skymap along with the marker. */
  bool    FillMarker;         /**< If TRUE, the marker on the skymap is
                                   filled. */
  int     MarkerSize;         /**< The size of the markers on the skymap. */

  bool    PlaySound;          /**< If TRUE, a sound is played when a message
                                   box reporting a new high signal pops up. */
  QString HighSignalSound;    /**< The path to a WAV audio file that is played
                                   if @a PlaySound is set to TRUE. */
};

/** Simpifies access to the structure with the global options. */
#define globalopts Ksetiwatch::globalOptions()

/** The Ksetiwatch application class.
  *
  * This class inherits KApplication for the sole purpose to allow
  * the distinction between a normal user shutdown and a shutdown
  * initiated by the session manager.
  */
class KsetiwatchApp : public KApplication
{
Q_OBJECT

public:
  /** The standard constructor.
    *
    * @param allowStyles Enables or disables the loading of plugin-based
    *                    styles.
    * @param GUIEnabled  Enables or disables GUI stuff within the application.
    */
  KsetiwatchApp(bool allowStyles = true, bool GUIEnabled = true);
  /** The destructor. */
  ~KsetiwatchApp();
  /** Commits all application-relevant data on shutdown.
    *
    * This function is called when the QSessionManager wants to close the
    * application. It is reimplemented to set the boolean sessionClose flag.
    *
    * @param sm  The QSessionManager object.
    */
  void commitData(QSessionManager& sm);
  /** Returns TRUE if the application is closed by the session manager. */
  bool closedBySessionManager() {return(sessionClose);}

private:
  bool sessionClose;
};

/** The main window of Ksetiwatch.
  *
  * This widget is the main window of Ksetiwatch. It basically consists of
  * a tab widget with several tab pages showing details about the configured
  * %SETI@home locations. Furthermore, it acts as the central instance for
  * maintaining the list of configured %S@h locations, the main configuration
  * file, and the `global' options.
  */
class Ksetiwatch : public KMainWindow
{
Q_OBJECT

public:
  /** This structure defines the layout of a column in the list views of
      Ksetiwatch. */
  struct TableColumn  {
                      QString text;        /**< The title of the column (shown
                                                in the header. */
                      int     width;       /**< The default width of the column
                                                (use -1 for automatic adjustment
                                                of the width). */
                      int     alignment;   /**< The alignment of text within the
                                                column. See Qt::AlignmentFlags
                                                for details. */
                      };

  /** The constructor.
    *
    * @param name   A text that can be used to identify this widget.
    */
  Ksetiwatch(const char* name=0);
  /** The destructor. */
  ~Ksetiwatch();
  /** Returns the start-up mode.
    *
    * @return If TRUE, Ksetiwatch shall be started minimized.
    */
  bool startMinimized() {return globalopts->StartMinimized;}
  /** Returns a pointer to a configured %S@h location.
    *
    * Searches the list of %S@h locations for the one specified by @a name
    * and returns a pointer to it.
    *
    * @param name The name of the %S@h location (as given by the user).
    *
    * @return A pointer to the SetiLoc instance, or NULL if it couldn't be
    *         found.
    */
  static SetiLoc* getLocation(const QString& name);
  /** Returns the %S@h location list.
    *
    * @return A reference to the the list containing all configured %S@h
    *         locations.
    */
  static QPtrList<SetiLoc>& locationList() {return(Ksetiwatch::SetiList);}
  /** Returns a pointer to the global options.
    *
    * @return A pointer to a (static) KswOptions structure.
    */
  static KswOptions* globalOptions() {return(&Ksetiwatch::GlobalOptions);}
  /** These identifiers specify how the list of %S@h locations can be
    * accessed.
    * @arg @c ListAdd:    Add a %S@h location to the list.
    * @arg @c ListEdit:   Update an existing %S@h location in the list.
    * @arg @c ListDelete: Delete a %S@h location from the list.
    * @arg @c ListRemove: Temporarily remove a %S@h location from the list
    *                     without deleting it.
    */
  enum ListId {ListAdd, ListEdit, ListDelete, ListRemove};

public slots: // Public slots
  /** Updates the list of %S@h locations.
    *
    * Adds or deletes the %S@h location @a loc to/from the main list.
    *
    * @param loc  The %S@h location to be added/deleted.
    * @param type Specifies what to do with @a loc. Either
    *             @c Ksetiwatch::ListAdd, or @c Ksetiwatch::ListDelete.
    */
  void slotUpdateList(SetiLoc* loc, int type);
  /** Reimplemented to catch the close event from the dock widget. */
  void close();

protected:
  /** Reimplemented for internal reasons (minimizing to systray, if
    * necessary).
    */
  bool event(QEvent *e);
  /** Saves the configuration on exit.
    *
    * This method is called whenever the session manager closes the
    * %application. It ensures that Ksetiwatch is started in the
    * state it had right before shutting down the KDE session.
    */
  bool queryClose();
  /** Clears the list of locations on exit.
    *
    * Must be done to prevent a segmentation fault when shutting down
    * Ksetiwatch.
    */
  bool queryExit();
  /** Starts the %SETI@home clients on start of Ksetiwatch.
    *
    * Clients are only started if the respective location has enabled
    * this option.
    */
  void startClients();
  /** Stops the %SETI@home clients on stop of Ksetiwatch.
    *
    * Clients are only stopped if the respective location has enabled
    * this option.
    */
  void stopClients();

private:
  static QPtrList<SetiLoc> SetiList;
  static KswOptions        GlobalOptions;

  KTabCtl*         ktb;
  Analysis*        ana;
  DataInfo*        di;
  UserInfo*        ui;
  CompletedWUList* cwu;
  Options*         opt;
  About*           abt;
  DockWidget*      ksdock;
  QPopupMenu*      dockrightpopup;
  bool             readyToQuit;
  bool             locListCleared;

private slots:
  /** Saves the Ksetiwatch configuration. */
  void saveConfig();
  /** Reads the Ksetiwatch configuration from the config file. */
  void readConfig();
  /** Modifies the help text in the status bar. */
  void handleTabSelection(int);
  /** Toggles dock icon on or off when the user makes changes in the options. */
  void toggleIcon();

signals: // Signals
  /** This signal is emitted whenever the main list of %S@h location has been
    * modified.
    *
    * Arguments are a pointer to the SetiLoc object involved and the
    * particular type of action (see Ksetiwatch::ListId).
    */
  void listModified(SetiLoc*, int);
};
#endif
