/***************************************************************************/
/** @file     loggedsigs.h
  * @brief    A dialog displaying logged signals (declaration).
  *
  * @b begin      : Tue May 1 2001<br>
  * @b copyright  : &copy; 2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef LOGGEDSIGS_H
#define LOGGEDSIGS_H

#include <qptrlist.h>

#include "loggedsigstmpl.h"
#include "poplistview.h"
#include "setidefs.h"

class SetiLoc;

const QString ResultLogFileName = "SETIResult.log";
const double  SliceFrequency = 9765.0;
const int     MaxFFTLen = 131072;

/***************************************************************************/
/** @file     loggedsigstmpl.h
  * @brief    The base class for the LoggedSigs dialog (declaration).
  *
  * Layout of the LoggedSigs dialog, automatically generated from the Qt
  * Designer user interface (ui) file @c loggedsigstmpl.ui. Any manual
  * modifications in this file will be overwritten.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  */

/***************************************************************************/
/** @file     loggedsigstmpl.cpp
  * @brief    The base class for the LoggedSigs dialog (implementation).
  *
  * Layout of the LoggedSigs dialog, automatically generated from the Qt
  * Designer user interface (ui) file @c loggedsigstmpl.ui. Any manual
  * modifications in this file will be overwritten.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  */

/***************************************************************************/
/** @class LoggedSigsTmpl
  * @brief The base class for the LoggedSigs dialog.
  *
  * This class is automatically generated from a Qt Designer user interface
  * (ui) file. Any manual changes to it will be overwritten.
  */

/** A customized QListViewItem for the list of spikes in the LoggedSigs
  * dialog.
  */
class SpikeListViewItem : public ColorListViewItem
{
public:
  /** The constructor.
    *
    * @param parent The parent list view.
    */
  SpikeListViewItem(QListView* parent) : ColorListViewItem(parent) {}
  /** The destructor. */
  ~SpikeListViewItem() {}
  /** Reimplemented to allow customized sorting of the items in the list.
    *
    * @param column    The column for which a key is returned.
    * @param ascending Indicates the sorting order (usage not recommended).
    *
    * @return A key that can be used for sorting by column @a column.
    */
  QString key(int column, bool ascending) const;
};

/** A dialog that displays all logged signals of a particular work unit.
  *
  * This dialog reads all logged signals from the @c SETIResult.log file and
  * displays them in a tab widget. The user can conveniently browse
  * through all logged signals by use of a few navigation buttons.
  */
class LoggedSigs : public LoggedSigsTmpl
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param locName The name of the %S@h location (as given by the user).
    * @param wu      The name of the work unit.
    * @param parent  The parent widget. If 0, this widget becomes a top-level
    *                widget.
    * @param name    A text that can be used to identify this widget.
    */
  LoggedSigs(const QString& locName, const QString& wu,
             QWidget* parent=0, const char* name=0);
  /** The destructor. */
  ~LoggedSigs();
  /** Defines the work unit whose signals shall be read from the log
    * file.
    *
    * @param name The name of the work unit in a QString.
    */
  void setWorkUnit(const QString& name) {wuname = name;}
  /** Returns the name of the work unit whose signals are shown. */
  QString wuName() const {return wuname;}

public slots: // Public slots

protected: // Protected methods
  /** Reimplemented to tell the list view whether it shall draw grid lines.
    *
    * @param e An object containing details about the paint event.
    */
  void paintEvent(QPaintEvent* e);
  /** Depending on the current gaussian shown, enable or disable the
    * navigation buttons. */
  void changeGaussianBtnState();
  /** Depending on the current pulse shown, enable or disable the
    * navigation buttons. */
  void changePulseBtnState();
  /** Depending on the current triplet shown, enable or disable the
    * navigation buttons. */
  void changeTripletBtnState();
  /** Prints a table with all important gaussian parameters in the text
    * view.
    *
    * The parameters for the Gaussian at position @a idx in the list will
    * be shown.
    *
    * @param idx The position in the list of Gaussians.
    */
  void setGaussianText(int idx);
  /** Prints a table with all important pulse parameters in the text
    * view.
    *
    * The parameters for the pulse at position @a idx in the list will
    * be shown.
    *
    * @param idx The position in the list of pulses.
    */
  void setPulseText(int idx);
  /** Prints a table with all important triplet parameters in the text
    * view.
    *
    * The parameters for the triplet at position @a idx in the list will
    * be shown.
    *
    * @param idx The position in the list of triplets.
    */
  void setTripletText(int idx);

protected slots: // Protected slots
  /** Adds listview items to the table of spikes. */
  void slotFillSpikeTable();
  /** Initializes the tab widgets and sets the display to the first
    * signals of the list. */
  void slotShowFirstSignals();
  /** Jumps to the next signal in the list. The type of the signal is
    * determined by the tab currently selected. */
  void slotNextSignal();
  /** Jumps to the previous signal in the list. The type of the signal is
    * determined by the tab currently selected. */
  void slotPreviousSignal();
  /** Jumps to the last signal in the list. The type of the signal is
    * determined by the tab currently selected. */
  void slotLastSignal();
  /** Jumps to the first signal in the list. The type of the signal is
    * determined by the tab currently selected. */
  void slotFirstSignal();
  /** Enables or disables the save button depending on which tab is
    * selected. */
  void slotChangeSaveBtnState(QWidget*);
  /** Saves a graph of a signal to disk. The graph has a standard size of
    * 350x200 pixel.*/
  void slotSave();
  /** Shows the help page for this dialog. */
  void slotShowHelp();

private: // Private methods
  /** Reads all logged signals from @c SETIResult.log and sorts them into
      the lists. */
  bool readLoggedSigs();
  /** Clears all entries in the lists of signals. */
  void clearLists();

private: // Private attributes
  QPtrList<SpikeScore>    spikeList;
  QPtrList<GaussianScore> gaussianList;
  QPtrList<PulseScore>    pulseList;
  QPtrList<TripletScore>  tripletList;
  QString  wuname;
  SetiLoc* location;
};

#endif
