/***************************************************************************/
/** @file     poplistview.cpp
  * @brief    A list view with a right-mouse-button popup menu
  *           (implementation).
  *
  * @b begin      : Thu Oct 28 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <qglobal.h>
#include <qpainter.h>
#include <qframe.h>
#include <qpoint.h>
#include <qcursor.h>

#include "poplistview.h"

/*------------------------------------------------------------------------ */
ColorListViewItem::ColorListViewItem(QListView* parent, const QColor& color,
                                     bool separator)
                  : KListViewItem(parent), m_separator(separator)
{
init(parent, color);
}

/*------------------------------------------------------------------------ */
ColorListViewItem::ColorListViewItem(QListView* parent, QListViewItem* after,
                                      const QColor& color, bool separator)
                  : KListViewItem(parent, after), m_separator(separator)
{
init(parent, color);
}

/*------------------------------------------------------------------------ */
ColorListViewItem::ColorListViewItem(QListViewItem* parent, const QColor& color,
                                     bool separator)
                  : KListViewItem(parent), m_separator(separator)
{
init(parent->listView(), color);
}

/*------------------------------------------------------------------------ */
ColorListViewItem::ColorListViewItem(QListViewItem* parent, QListViewItem* 
after,                                      const QColor& color, bool separator)
                  : KListViewItem(parent, after), m_separator(separator)
{
init(parent->listView(), color);
}

/*------------------------------------------------------------------------ */
ColorListViewItem::~ColorListViewItem()
{
}

/*------------------------------------------------------------------------ */
void ColorListViewItem::init(QListView* parent, const QColor& color)
{
m_listView   = parent;
if(color.isValid())
  m_color = color;
else
  m_color = m_listView->colorGroup().color(QColorGroup::Text);
}

/*------------------------------------------------------------------------ */
void ColorListViewItem::paintCell(QPainter* p, const QColorGroup& cg,
                                  int column, int width, int align)
{
QColorGroup my_cg(cg);

my_cg.setColor(QColorGroup::Text, m_color);
KListViewItem::paintCell(p, my_cg, column, width, align);

if(dynamic_cast<popListView*>(m_listView)->isGridEnabled())
  {
  p->setPen( cg.mid() );
  p->moveTo(0, height()-1);
  p->lineTo(width-1, height()-1);
  p->lineTo(width-1, 0);
  if(m_separator)
    {
    QPen pen;
    pen.setWidth(2);
    p->setPen(pen);
    p->moveTo(0, 0);
    p->lineTo(width-1, 0);
    }
  }
}

/*------------------------------------------------------------------------ */
popListView::popListView(QWidget *parent, bool grid, const char *name)
            :KListView(parent,name), drawGrid(grid)
{
setAllColumnsShowFocus(TRUE);
popup = new KPopupMenu(this);
triggerUpdate();

lastRootItem = 0;
lastChildItem = 0;

setFrameStyle( QFrame::Box | QFrame::Sunken );
setLineWidth( 1 );
setMidLineWidth( 0 );

if(QT_VERSION < 0x030005)
  {
  connect(this,
    SIGNAL(mouseButtonClicked(int, QListViewItem*, const QPoint&, int)),
    SLOT(slotMouseButtonClicked(int, QListViewItem*, const QPoint&, int)));
  }
else
  {
  connect(this,
    SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)),
    SLOT(slotShowContextMenu(QListViewItem*, const QPoint&, int)));
  }
}

/*------------------------------------------------------------------------ */
popListView::~popListView()
{
}

/*------------------------------------------------------------------------ */
void popListView::setGridEnabled(bool grid)
{
drawGrid = grid;
update();
}

/*------------------------------------------------------------------------ */
QListViewItem* popListView::insertRootItem(const QString& aName,
                                           QPixmap* icon)
{
QListViewItem* item;

// Make sure the entry gets added AFTER the last entry.
if( lastRootItem != 0 )
  {
  item =  new ColorListViewItem( this, lastRootItem );
  item->setText( 0, aName );
  }
else
  {
  item = new ColorListViewItem( this );
  item->setText( 0, aName );
  }

if(icon != 0)
  item->setPixmap(0, *icon );

// Save this as the last entry.
lastRootItem = item;

return item;
}

/*------------------------------------------------------------------------ */
QListViewItem* popListView::insertChildItem(const QString& aName,
                                            QPixmap* icon,
                                            QListViewItem *parent)
{
QListViewItem *item;

if(parent != 0)
  {
  item = new ColorListViewItem( parent, lastChildItem );
  item->setText( 0, aName );
  }
else
  {
  item = new QListViewItem( lastRootItem, lastChildItem );
  item->setText( 0, aName );
  }
if(icon != 0)
  item->setPixmap( 0, *icon );

// Save this as the last entry.
lastChildItem = item;

return item;
}

/*------------------------------------------------------------------------ */
void popListView::clear()
{
lastRootItem = 0;
lastChildItem = 0;
QListView::clear();
}

/*------------------------------------------------------------------------ */
int popListView::insertPopupItem(const QString& name, int id, int index)
{
return(popup->insertItem(name, id, index));
}

/*------------------------------------------------------------------------ */
int popListView::insertPopupItem(const QPixmap& icon, const QString& name,
                                 int id, int index)
{
return(popup->insertItem(QIconSet(icon, QIconSet::Small), name, id, index));
}

/*------------------------------------------------------------------------ */
int popListView::insertPopupItem(const QString& name, QPopupMenu* pop,
                                 int id, int index)
{
return(popup->insertItem(name, pop, id, index));
}

/*------------------------------------------------------------------------ */
int popListView::insertPopupItem(const QPixmap& icon, const QString& name,
                                 QPopupMenu* pop, int id, int index)
{
return(popup->insertItem(QIconSet(icon, QIconSet::Small), name, pop,
                         id, index));
}

/*------------------------------------------------------------------------ */
void popListView::slotMouseButtonClicked(int but, QListViewItem* it,
                                         const QPoint&, int)
{
if(it == 0 && pstyle == ShowAnywhere)
  {
  setSelected(currentItem(), false);
  emit selectionChanged(0);
  }

if(but == Qt::RightButton && popup->count() > 0)
  {
  if(pstyle == ShowAnywhere || (it && pstyle == OverItem))
    popup->popup(QCursor::pos());
  }
}

/*------------------------------------------------------------------------ */
void popListView::slotShowContextMenu(QListViewItem* it, const QPoint& pos,
                                      int col)
{
slotMouseButtonClicked(Qt::RightButton, it, pos, col);
}

/*------------------------------------------------------------------------ */
KPopupMenu *popListView::popupMenu()
{
return(popup);
}

/*------------------------------------------------------------------------ */
void popListView::setPopupStyle(PopupStyle p)
{
pstyle = p;
}

/*------------------------------------------------------------------------ */
popListView::PopupStyle popListView::popupStyle()
{
return pstyle;
}

#include "poplistview.moc"

