/***************************************************************************/
/** @file     poplistview.h
  * @brief    A list view with a right-mouse-button popup menu
  *           (declaration).
  *
  * @b begin      : Thu Oct 28 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef POPLISTVIEW_H
#define POPLISTVIEW_H

#include <qpalette.h>

#include <klistview.h>
#include <kpopupmenu.h>

/**  A customized QListViewItem with additional features.
  *
  *  Adds grid lines and colored text.
  *
  *  @warning Only to be used with the popListView class.
  */
class ColorListViewItem : public KListViewItem
{
public:
  /** The constructor.
    *
    * @param parent    The parent list view.
    * @param color     The color of the text in the list view item. If no
    *                  color is specified, the normal text color of the
    *                  parent widget is taken.
    * @param separator If true, a thick line is drawn at the top edge of
    *                  the item to separate it from othe list view items.
    */
  ColorListViewItem(QListView* parent, const QColor& color = QColor(),
                    bool separator = false);
  /** The constructor.
    *
    * @overload
    *
    * @param parent    The parent list view.
    * @param after     The list view item after which the new item will be
    *                  sorted into the list.
    * @param color     The color of the text in the list view item. If no
    *                  color is specified, the normal text color of the
    *                  parent widget is taken.
    * @param separator If true, a thick line is drawn at the top edge of
    *                  the item to separate it from the list view items.
    */
  ColorListViewItem(QListView* parent, QListViewItem* after,
                    const QColor& color=QColor(), bool separator = false);
  /** The constructor.
    *
    * @overload
    * @param parent    The parent list view item.
    * @param color     The color of the text in the list view item. If no
    *                  color is specified, the normal text color of the
    *                  parent widget is taken.
    * @param separator If true, a thick line is drawn at the top edge of
    *                  the item to separate it from othe list view items.
    */
  ColorListViewItem(QListViewItem* parent, const QColor& color = QColor(),
                    bool separator = false);
  /** The constructor.
    *
    * @overload
    *
    * @param parent    The parent list view item.
    * @param after     The list view item after which the new item will be
    *                  sorted into the list.
    * @param color     The color of the text in the list view item. If no
    *                  color is specified, the normal text color of the
    *                  parent widget is taken.
    * @param separator If true, a thick line is drawn at the top edge of
    *                  the item to separate it from the list view items.
    */
  ColorListViewItem(QListViewItem* parent, QListViewItem* after,
                    const QColor& color=QColor(), bool separator = false);
  /** The destructor. */
  ~ColorListViewItem();

protected: // methods
  /** Initializes the list view item.
    *
    * @param parent The parent list view.
    * @param color  The color of the text in the list view item.
    */
  void init(QListView* parent, const QColor& color);
  /** Reimplemented to allow customized painting of one column of an item.
    *
    * @param p      A QPainter object open on the relevant paint device.
    * @param cg     The color group which is used for painting.
    * @param column The column number within the item which is to be painted.
    * @param width  The width of the column.
    * @param align  The alignment of text within the column.
    */
  virtual void paintCell(QPainter* p, const QColorGroup& cg, int column,
                         int width, int align);

protected: // attributes
  QListView* m_listView;
  QColor     m_color;
  bool       m_separator;
};

/** A list view with a right-mouse-button popup menu.
  *
  * This class combines a normal QListView with a popup menu which appears
  * when the right mouse button is clicked.This allows the manipulation of
  * list view items via the menu items in the popup menu.
  *
  * There are two modes for the appearance of the popup menu. Use
  * @c popListView::OverItem in order to allow the popup menu to appear only
  * when a list view item has been clicked, or @c popListView::ShowAnywhere if
  * the context menu is allowed to pop up anywhere.
  */
class popListView : public KListView
{
Q_OBJECT

public: 
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  popListView(QWidget *parent, bool grid=false, const char *name=0);
  /** The destructor. */
  ~popListView();
  /** Adds a list view item at root level.
    *
    * Optionally, an icon can be displayed.
    *
    * @param aName The text to be shown in the list view.
    * @param icon  A pointer to an icon that shall be shown.
    *
    * @return A pointer to the new list view item.
    */
  QListViewItem* insertRootItem( const QString& aName, QPixmap *icon=0 );
  /** Adds a child list view item.
    *
    * The list view item will be the child of @a parent. Optionally, an icon
    * can be displayed.

    * @param aName  The text to be shown in the list view.
    * @param icon   A pointer to an icon that shall be shown.
    * @param parent The parent the new item shall be added to. If 0, the item
    *               will be added to the last added root item.
    *
    * @return A pointer to the new list view item.
    */
  QListViewItem* insertChildItem(const QString& aName, QPixmap *icon=0,
                                 QListViewItem *parent=0 );
  /** Clears all items in the list view. */
  void clear();
  /** Adds a menu item to the right-click popup menu.
    *
    * The menu item will show the text @a name.
    *
    * @param name  The text of the menu item.
    * @param id    The ID number of the item. If negative, the identifier
    *              will be automatically generated.
    * @param index An index value. If -1, the item will be appended to
    *              the menu.
    *
    * @return The menu item identifier.
    */
  int insertPopupItem(const QString& name, int id=-1, int index=-1);
  /** Adds a menu item to the right-click popup menu.
    *
    * @overload
    * The menu item will show the text @a name, and the icon @a icon.
    *
    * @param icon  A pixmap with the icon that shall be shown.
    * @param name  The text of the menu item.
    * @param id    The ID number of the item. If negative, the identifier
    *              will be automatically generated.
    * @param index An index value. If -1, the item will be appended to
    *              the menu.
    *
    * @return The menu item identifier.
    */
  int insertPopupItem(const QPixmap& icon, const QString& name,
                      int id=-1, int index=-1);
  /** Adds a menu item to the right-click popup menu.
    *
    * @overload
    * The menu item has a sub-menu @a pop, and displays the text @a name.
    *
    * @param name  The text of the menu item.
    * @param pop   A pointer to the sub-menu.
    * @param id    The ID number of the item. If negative, the identifier
    *              will be automatically generated.
    * @param index An index value. If -1, the item will be appended to
    *              the menu.
    *
    * @return The menu item identifier.
    */
  int insertPopupItem(const QString& name, QPopupMenu* pop,
                      int id=-1, int index=-1);
  /** Adds a menu item to the right-click popup menu.
    *
    * @overload
    * The menu item has a sub-menu @a pop, and displays the text @a name
    * and the icon @a icon.
    *
    * @param icon  A pixmap with the icon that shall be shown.
    * @param name  The text of the menu item.
    * @param pop   A pointer to the sub-menu.
    * @param id    The ID number of the item. If negative, the identifier
    *              will be automatically generated.
    * @param index An index value. If -1, the item will be appended to
    *              the menu.
    *
    * @return The menu item identifier.
    */
  int insertPopupItem(const QPixmap& icon, const QString& name,
                      QPopupMenu* pop, int id=-1, int index=-1);
  /** Defines an accelerator key.
    *
    * Defines an accelerator key for the menu item with the identifier @a id.
    *
    * @param key The accelerator key, which is a combination of the key code
    *            (e.g. Key_A) and modifiers like SHIFT, CTRL, or ALT.
    * @param id  The identifier of the menu item.
    */
  void setAccel(int key, int id) {popup->setAccel(key, id);}
  /** Adds a separator to the popup menu.
    *
    * The separator becomes the last item in the popup menu.
    */
  inline void insertSeparator() {if(popup) popup->insertSeparator();}
  /** Sets the title of the popup menu.
    *
    * @param title The title of the popup menu.
    */
  inline void popupSetTitle(const QString& title) {if(popup) popup->insertTitle(title);}  
  /** Returns a pointer to the KPopupMenu object. */
  KPopupMenu* popupMenu();
  /** Two different popup styles. */
  enum PopupStyle {ShowAnywhere, OverItem};
  /** Sets the behaviour of the popup menu
    *
    * @param p The new style. This can either be @c popListView::ShowAnywhere
    *          to allow the popup menu anywhere in the list view, or
    *          @c popListView::OverItem to display the menu only when the
    *          click occurred over a list view item.
    */
  void setPopupStyle(PopupStyle p);
  /** Returns the current popup style */
  PopupStyle popupStyle();
  /** Enables or disables grid lines in the list view.
    *
    * @param grid TRUE if grid lines shall be drawn, otherwise FALSE.
    */
  void setGridEnabled(bool grid);
  /** Returns whether list view items are drawn with grid lines. */
  bool isGridEnabled() {return(drawGrid);}
  
protected:
  /** The last child item added to the tree. */
  QListViewItem* lastChildItem;
  /** The last root-item added to the tree. */
  QListViewItem* lastRootItem;
  /** Stores the display style of the popup menu. */
  PopupStyle pstyle;
  /** The user-defined popup menu. */
  KPopupMenu* popup;
  /** Specifies whether to draw a grid inside the list view. */
  bool drawGrid;

protected slots:
  /** Handles mouse button clicks correctly.
    *
    * This slot is called whenever the user clicked with either mouse button
    * onto the list view.
    *
    * @param but The mouse button which the user pressed.
    * @param it  The pointer to the clicked list view item or 0, if the user
    *            didn't click on an item.
    * @param pos The position of the mouse cursor.
    * @param col The list view column into which the user clicked (this
    *            argument is only valid, if @a it is not 0).
    */
  void slotMouseButtonClicked(int but, QListViewItem* it,
                              const QPoint& pos, int col);
  /** Implemented as a workaround for Qt 3.0.5 (mouse button signals aren't
    * emitted).
    *
    * @param it  The pointer to the clicked list view item or 0, if the user
    *            didn't click on an item.
    * @param pos The position of the mouse cursor.
    * @param col The list view column into which the user clicked (this
    *            argument is only valid, if @a it is not 0).
    */
  void slotShowContextMenu(QListViewItem* it, const QPoint& pos, int col);
};

#endif
