/***************************************************************************/
/** @file     pulseplot.h
  * @brief    A widget that displays a pulse profile (declaration).
  *
  * @b begin      : Mon Oct 23 2000<br>
  * @b copyright  : &copy; 2000-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef PULSEPLOT_H
#define PULSEPLOT_H

#include "setigraph.h"
#include "setidefs.h"

/** Displays a pulse profile.
  *
  * This object can be constructed in two ways: either with or without
  * specifying a %S@h location. If a SetiLoc object is passed to the
  * constructor of this class, the pulse profile shown in the widget
  * always represents the best pulse the %S@h client has found in the
  * work unit it processes at that time. The pulse profile is
  * automatically updated whenever the client finds a new and better pulse
  * signal.
  *
  * If this widget is created without a %S@h location, the pulse data
  * have to be explicitly passed with the showNewPulseData() method.
  */
class PulsePlot : public SetiGraph
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  PulsePlot(QWidget *parent=0, const char *name=0);
  /** The constructor.
    *
    * @overload
    *
    * @param loc    The %S@h location for which the best pulse (found in
    *               the currently processed work unit) shall be shown.
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  PulsePlot(SetiLoc* loc, QWidget *parent=0, const char *name=0);
  /** The destructor. */
  ~PulsePlot();

public slots:
  /** Updates the widget with new pulse data.
    *
    * @param score A structure containing the parameters of the new pulse
    *              to be shown.
    */
  void showNewPulseData(PulseScore score);

protected: // methods
  /** Initializes the widget.
    *
    * If a %S@h location is given, the pulse profile is painted, and a
    * connection is made to the SetiLoc::newPulse signal of the
    * corresponding SetiLoc object.
    */
  void init();

protected: // attributes
  /** This array stores the pulse data (actually, two copies of it to make
      it look like a repeating signal). */
  QMemArray<double> pulseData;
  /** A structure containing the pulse parameters. */
  PulseScore pulse;
  /** Indicates if the data of a pulse is loaded. */
  bool dataLoaded;
};

#endif
