/***************************************************************************/
/**
  * @file  seticontainer.cpp
  * @brief The SetiContainer class (implementation).
  *
  * @b version    : 0.5 - 2002.01.29<br>
  * @b begin      : Fri Nov 17 2000<br>
  * @b copyright  : &copy; 2000-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This file contains the implementation of the SetiContainer class. More
  * details can be found directly in the documentation of the SetiContainer
  * class.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <math.h>
#include <errno.h>
#include <sys/file.h>
#include <signal.h>
#include <stdio.h>
#include <ctype.h>

#include <qfile.h>
#include <qfileinfo.h>
#include <qdatetime.h>
#include <qtextstream.h>
#include <qdir.h>

#include <kprocess.h>

#include "seticontainer.h"

// needed to prevent rounding errors when comparing doubles
#define DIGITS (double)1.0e-6

/*------------------------------------------------------------------------ */
SetiContainer::SetiContainer(const QString& dir, int refresh)
{
// store refresh interval in ms
m_refreshIval   = refresh*1000;

// set the current directory and initialize everything
setDirectory(dir);

// start the timer in order to periodically refresh the data
m_refTimer = startTimer(m_refreshIval);

// start timers that check periodically the state of the client
m_sttTimer1 = startTimer(500);            // every 500 or 2500 ms
m_sttTimer2 = startTimer(120000);         // every 2 minutes

m_startProc = new KProcess();
m_stopProc  = new KProcess();
}

/*------------------------------------------------------------------------ */
SetiContainer::~SetiContainer()
{
delete m_startProc;
delete m_stopProc;
}

/*------------------------------------------------------------------------ */
void SetiContainer::setDirectory(const QString& dir)
{
// make a back-up
m_setiDirectory = dir;

// read the version number of the client
m_sahVersion = readClientVersion();
// returning zero means that the version file couldn't be found
m_versionRead = m_sahVersion > 0;

m_stfFileName = dir + "/state.sah";
m_wufFileName = dir + "/work_unit.sah";
m_uifFileName = dir + "/user_info.sah";
m_rsfFileName = dir + "/result.sah";
m_rhdFileName = dir + "/result_header.sah";
m_wtpFileName = dir + "/wtemp.sah";

// initialize all structures and read new data
initAllData();

// initialize m_oldProgress variable for passive monitoring
m_oldProgress = m_stfData.prog;

// reset client state and pid
m_cltState  = Stopped;
m_cltPid    = 0;
m_procPid   = 0;
m_localClt  = false;
m_fastCheck = true;
}

/*------------------------------------------------------------------------ */
void SetiContainer::initAllData(bool sig)
{
// fill all file data members with zeros
initStateFileData();
initWorkUnitData();
initUserInfoData();

// clear the timestamps
m_stfTimeStamp = QString::null;
m_uifTimeStamp = QString::null;
m_wufTimeStamp = QString::null;
m_pidTimeStamp = QString::null;

// read all data
updateSahData(sig);
}

/*------------------------------------------------------------------------ */
void SetiContainer::setRefreshInterval(int refresh)
{
m_refreshIval = refresh*1000; // in ms

// restart the timer with the new refresh interval
killTimer(m_refTimer);
m_refTimer = startTimer(m_refreshIval);
}

/*------------------------------------------------------------------------ */
void SetiContainer::timerEvent(QTimerEvent* e)
{
if(e->timerId() == m_refTimer)
  updateSahData();
else if(e->timerId() == m_sttTimer2)
  checkClientStatePassive();
else if(e->timerId() == m_sttTimer1)
  {
  bool oldFastCheck = m_fastCheck;
  if((int)cpuTime() > 600 && (int)remainingTime() > 600)
    m_fastCheck = false;
  else
    m_fastCheck = true;
  if(m_fastCheck != oldFastCheck)
    {
    //qDebug("SetiContainer (%s): CPU Time=%d, remaining Time=%d, status=%d",
    //       (const char*)directory(), (int)cpuTime(), (int)remainingTime(),
    //       (int)m_fastCheck);
    killTimer(m_sttTimer1);
    if(m_fastCheck)
      m_sttTimer1 = startTimer(500);
    else
      m_sttTimer1 = startTimer(2500);
    }
  checkClientState();
  }
}

/*------------------------------------------------------------------------ */
int SetiContainer::updateSahData(bool sig)
{
int err;
int filesMissing = 0;

// do this complicated check to avoid disk access as much as possible
if(m_versionRead == false)
  {
  m_sahVersion  = readClientVersion();
  if(m_sahVersion > 0)
    m_versionRead = true;
  }
else
  {
  if(QFile::exists(m_setiDirectory + "/version.sah") == false &&
     QFile::exists(m_setiDirectory + "/version.txt") == false)
    {
    m_sahVersion = 0;
    m_versionRead = false;
    }
  }

err = readUserInfoFile(sig);
if(err)
  {
  filesMissing++;
  #ifndef NDEBUG
    qDebug("SetiContainer (%s): user_info.sah missing.",
           (const char*)directory());
  #endif
  }

err = readWorkUnitFile(sig);
if(err)
  {
  filesMissing++;
  #ifndef NDEBUG
    qDebug("SetiContainer (%s): work_unit.sah missing.",
          (const char*)directory());
  #endif
  }

err = readStateFile(sig);
if(err)
  {
  filesMissing++;
  #ifndef NDEBUG
    qDebug("SetiContainer (%s): state.sah missing.",
          (const char*)directory());
  #endif
  }

// all three files are missing, switch to Stopped
if(filesMissing == 3)
  {
  if(m_cltState != Stopped)
    {
    m_cltState = Stopped;
    emit stateChanged(m_cltState);
    }
  err = 1;
  }

return(err);
}

/*------------------------------------------------------------------------ */
int SetiContainer::readSahFile(const QString& file, QStringList& list, const QString& endId)
{
int ret(0);
QFile sah(file);

// clear the list first
list.clear();

if(sah.open(IO_ReadOnly))
  {
  QTextStream t(&sah);
  QString s;
  while( !t.atEnd() )
    {
    s = t.readLine();
    // if reading a line fails or the eof identifier is found stop here
    if (s.isEmpty() || (endId && endId == s)) break;
    list.append(s);
    }
  sah.close();
  }
else
  ret = 1;

return(ret);
}

/*------------------------------------------------------------------------ */
int SetiContainer::readStateFile(bool sig)
{
int ret(0);

// Return with error if file couldn't be found.
if(exists(SC_StateFile) == false) return(1);

// Read the time stamp of the file...
QFileInfo fi(m_stfFileName);
QDateTime dt = QDateTime(fi.lastModified());
QString   ts = dt.toString();
// ...and compare it with the stored one.
if(ts != m_stfTimeStamp)
  {
  m_stfTimeStamp = ts;
  ret = readSahFile(m_stfFileName, m_stateFile);
  // If successfully read, scan the file for the data.
  if(ret == 0)
    scanStateFile(sig);
  }

return(ret);
}

/*------------------------------------------------------------------------ */
int SetiContainer::readWorkUnitFile(bool sig)
{
int ret(0);

// Return with error if file couldn't be found.
if(exists(SC_WorkUnitFile) == false) return(1);

// Read the time stamp of the file...
QFileInfo fi(m_wufFileName);
QDateTime dt = QDateTime(fi.lastModified());
QString   ts = dt.toString();
// ...and compare it with the stored one.
if(ts != m_wufTimeStamp)
  {
  m_wufTimeStamp = ts;
  ret = readSahFile(m_wufFileName, m_workunitFile, "end_seti_header");
  // If successfully read, scan the file for the data.
  if(ret == 0)
    scanWorkUnitData(sig);
  }

return(ret);
}

/*------------------------------------------------------------------------ */
int SetiContainer::readUserInfoFile(bool sig)
{
int ret(0);

// Return with error if file couldn't be found.
if(exists(SC_UserInfoFile) == false) return(1);

// Read the time stamp of the file...
QFileInfo fi(m_uifFileName);
QDateTime dt = QDateTime(fi.lastModified());
QString   ts = dt.toString();
// ...and compare it with the stored one.
if(ts != m_uifTimeStamp)
  {
  m_uifTimeStamp = ts;
  ret = readSahFile(m_uifFileName, m_userinfoFile);
  // If successfully read, scan the file for the data.
  if(ret == 0)
    scanUserInfoData(sig);
  }

return(ret);
}

/*------------------------------------------------------------------------ */
QString SetiContainer::readEntry(int sah, const QString& e)
{
QString val;

switch(sah)
  {
  case SC_StateFile:
    val = readEntry(m_stfFileName, e);
    break;
  case SC_UserInfoFile:
    val = readEntry(m_uifFileName, e);
    break;
  case SC_WorkUnitFile:
    val = readEntry(m_wufFileName, e);
    break;
  case SC_ResultFile:
    val = readEntry(m_rsfFileName, e);
    break;
  default:
    val = QString::null;
    break;
  }

return(val);
}


/*------------------------------------------------------------------------ */
QString SetiContainer::readEntry(const QString& fn, const QString& e)
{
QString v = QString::null;
QString l,s;
int eq(-1);
bool found(false);
QFile sah(fn);

if(sah.open(IO_ReadOnly))
  {
  QTextStream t(&sah);
  while( !t.eof() )
    {
    s = t.readLine();
    // the next line is to prevent hangs when monitoring NFS directories
    // (suggested by Frode Tenneboe)
    if (s.isEmpty()) break;
    eq = s.find('=');
    if(eq != -1)
      {
      l = s.left(eq);
      if(l == e) {found = true; break;}
      }
    }
  if(found && s.length()-eq-1 > 0)
    v = s.right( s.length()-eq-1 );
  sah.close();
  }

return( v );
}

/*------------------------------------------------------------------------ */
QString SetiContainer::readEntry(QStringList& list, const QString& e)
{
QString v = QString::null;
QString s;
int pos;
bool found(false);

QStringList::Iterator it;
QStringList::Iterator end(list.end());
for(it=list.begin(); it!=end; ++it)
  {
  // copy the contents of the iterator to a string
  s = (*it);
  pos = s.find(e);
  // accept the found entry only if it is at position 0
  if(pos == 0)
    {
    found = true;
    break;
    }
  }

if(found)
  {
  // find the equal char
  pos = s.find('=');
  // and take the part to the right
  if(pos > 0 && s.length()-pos-1 > 0)
    v = s.right(s.length()-pos-1);
  }

return(v);
}

/*------------------------------------------------------------------------ */
void SetiContainer::initWUScore(WUScore* score)
{
score->spike.power     = 0.0;
score->spike.score     = 0.0;
score->spike.bin       = 0;
score->spike.fft_index = 0;
score->spike.fft_len   = 0;
score->spike.chirprate = 0.0;
score->spike.ra        = 0.0;
score->spike.dec       = 0.0;
score->spike.frequency = 0.0;
score->spike.wu_name   = "";

score->gaussian.score     = 0.0;
score->gaussian.power     = 0.0;
score->gaussian.bin       = 0;
score->gaussian.chisq     = 0.0;
score->gaussian.true_mean = 0.0;
score->gaussian.sigma     = 0.0;
score->gaussian.fft_index = 0;
score->gaussian.fft_len   = 0;
score->gaussian.chirprate = 0.0;
score->gaussian.ra        = 0.0;
score->gaussian.dec       = 0.0;
score->gaussian.frequency = 0.0;
score->gaussian.wu_name   = "";
for(int i=0;i<64;i++) score->gaussian.data[i] = 0.0;
score->gaussian.data_len  = 0;

score->pulse.score     = 0.0;
score->pulse.power     = 0.0;
score->pulse.mean      = 0.0;
score->pulse.period    = 0.0;
score->pulse.freq_bin  = 0;
score->pulse.time_bin  = 0;
score->pulse.fft_len   = 0;
score->pulse.chirprate = 0.0;
score->pulse.ra        = 0.0;
score->pulse.dec       = 0.0;
score->pulse.frequency = 0.0;
score->pulse.wu_name   = "";
for(int i=0;i<512;i++) score->pulse.data[i] = 0;
score->pulse.data_len  = 0;

score->triplet.score      = 0.0;
score->triplet.power      = 0.0;
score->triplet.mean       = 0.0;
score->triplet.period     = 0.0;
score->triplet.bperiod    = 0.0;
score->triplet.tpotind0_0 = 0;
score->triplet.tpotind0_1 = 0;
score->triplet.tpotind1_0 = 0;
score->triplet.tpotind1_1 = 0;
score->triplet.tpotind2_0 = 0;
score->triplet.tpotind2_1 = 0;
score->triplet.freq_bin   = 0;
score->triplet.time_bin   = 0.0;
score->triplet.scale      = 0.0;
score->triplet.fft_len    = 0;
score->triplet.chirprate  = 0.0;
score->triplet.ra         = 0.0;
score->triplet.dec        = 0.0;
score->triplet.frequency  = 0.0;
score->triplet.wu_name    = "";
for(int i=0;i<512;i++) score->triplet.data[i] = 0;
score->triplet.data_len   = 0;
}

/*------------------------------------------------------------------------ */
void SetiContainer::initStateFileData()
{
m_stfData.ncfft       = 0;
m_stfData.cr          = 0.0;
m_stfData.fl          = 0;
m_stfData.cpu         = 0.0;
m_stfData.prog        = 0.0;
m_stfData.potfreq     = 0;
m_stfData.potactivity = 0;
m_stfData.outfilepos  = 0;
initWUScore(&m_stfData.max);
}

/*------------------------------------------------------------------------ */
void SetiContainer::initWorkUnitData()
{
m_wufData.task                = "";
m_wufData.version             = 0;;
m_wufData.name                = "";
m_wufData.data_type           = "";
m_wufData.data_class          = 0;
m_wufData.splitter_version    = "";
m_wufData.start_ra            = 0.0;
m_wufData.start_dec           = 0.0;
m_wufData.end_ra              = 0.0;
m_wufData.end_dec             = 0.0;
m_wufData.angle_range         = 0.0;
m_wufData.time_recorded       = "";
m_wufData.subband_center      = 0.0;
m_wufData.subband_base        = 0.0;
m_wufData.subband_sample_rate = 0.0;
m_wufData.fft_len             = 0;
m_wufData.ifft_len            = 0;
m_wufData.subband_number      = 0;
m_wufData.receiver            = "";
m_wufData.nsamples            = 0;
m_wufData.tape_version        = "";
m_wufData.num_positions       = 0;
m_wufData.coordinates.clear();
}

/*------------------------------------------------------------------------ */
void SetiContainer::initUserInfoData()
{
m_uifData.id               = 0;
m_uifData.key              = 0;
m_uifData.email_addr       = "";
m_uifData.name             = "";
m_uifData.url              = "";
m_uifData.country          = "";
m_uifData.postal_code      = 0;
m_uifData.show_name        = false;
m_uifData.show_email       = false;
m_uifData.venue            = 0;
m_uifData.register_time    = "";
m_uifData.last_wu_time     = "";
m_uifData.last_result_time = "";
m_uifData.nwus             = 0;
m_uifData.nresults         = 0;
m_uifData.total_cpu        = 0.0;
m_uifData.params_index     = 0;
}

/*------------------------------------------------------------------------ */
void SetiContainer::scanStateFile(bool sig)
{
QString val;

val = readEntry(m_stateFile, "ncfft");
m_stfData.ncfft = val.toInt();

val = readEntry(m_stateFile, "cr");
m_stfData.cr = val.toDouble();

val = readEntry(m_stateFile, "fl");
m_stfData.fl = val.toInt();

val = readEntry(m_stateFile, "cpu");
m_stfData.cpu = val.toDouble();

double oldprog = m_stfData.prog;
val = readEntry(m_stateFile, "prog");
m_stfData.prog = val.toDouble();
double diff = fmod(m_stfData.prog-oldprog, DIGITS);
if(diff > 0.0 && sig)
  {
  emit progressIncreased();
  // switch only to Running if the old progress value is non-zero (to
  // prevent a freshly mounted location from automatically entering
  // the Running state)
  if(m_cltState != Running && oldprog > 0.0)
    {
    m_cltState = Running;
    emit stateChanged(m_cltState);
    }
  }
if(diff < 0.0 && sig) emit progressDecreased();

val = readEntry(m_stateFile, "potfreq");
m_stfData.potfreq = val.toInt();

val = readEntry(m_stateFile, "potactivity");
m_stfData.potactivity = val.toInt();

val = readEntry(m_stateFile, "outfilepos");
m_stfData.outfilepos = val.toInt();

scanSpikeData(sig);
scanGaussianData(sig);
scanPulseData(sig);
scanTripletData(sig);

}

/*------------------------------------------------------------------------ */
void SetiContainer::scanSpikeData(bool sig)
{
QString val;

// read the highest peak so far
double oldspike = m_stfData.max.spike.power;
val = readEntry(m_stateFile, "bs_power");
m_stfData.max.spike.power = val.toDouble();

double diff = fmod(m_stfData.max.spike.power-oldspike, DIGITS);
if(diff > 0.0)
  {
  // read the binary value of the highest peak
  val = readEntry(m_stateFile, "bs_bin");
  m_stfData.max.spike.bin = val.toInt();

  // read the corresponding chirp rate of the highest peak
  val = readEntry(m_stateFile, "bs_chirp_rate");
  m_stfData.max.spike.chirprate = val.toDouble();

  // read the fft index of the highest peak
  val = readEntry(m_stateFile, "bs_fft_ind");
  m_stfData.max.spike.fft_index = val.toInt();

  // read the fft length of the highest peak
  val = readEntry(m_stateFile, "bs_fft_len");
  m_stfData.max.spike.fft_len = val.toInt();

  // read the score of the highest peak
  val = readEntry(m_stateFile, "bs_score");
  m_stfData.max.spike.score = val.toDouble();

  m_stfData.max.spike.wu_name = wuName();

  if(sig) emit newSpike(m_stfData.max.spike);
  }
}

/*------------------------------------------------------------------------ */
void SetiContainer::scanGaussianData(bool sig)
{
QString val;

// read the strongest gaussian according to its score value
double oldgaussian = m_stfData.max.gaussian.score;
val = readEntry(m_stateFile, "bg_score");
m_stfData.max.gaussian.score = val.toDouble();

double diff = fmod(m_stfData.max.gaussian.score-oldgaussian, DIGITS);
if(diff > 0.0)
  {
  // read the binary value of the highest gaussian
  val = readEntry(m_stateFile, "bg_bin");
  m_stfData.max.gaussian.bin = val.toInt();

  // read the gaussian power
  val = readEntry(m_stateFile, "bg_power");
  m_stfData.max.gaussian.power = val.toDouble();

  // read the gaussian chi square
  val = readEntry(m_stateFile, "bg_chisq");
  m_stfData.max.gaussian.chisq = val.toDouble();

  // read the fft length of the highest gaussian
  val = readEntry(m_stateFile, "bg_fft_len");
  m_stfData.max.gaussian.fft_len = val.toInt();

  // read the corresponding chirp rate of the highest gaussian
  val = readEntry(m_stateFile, "bg_chirp_rate");
  m_stfData.max.gaussian.chirprate = val.toDouble();

  // read the sigma value of the top Gaussian
  val = readEntry(m_stateFile, "bg_sigma");
  m_stfData.max.gaussian.sigma = val.toDouble();

  // read the mean value of the top Gaussian
  val = readEntry(m_stateFile, "bg_true_mean");
  m_stfData.max.gaussian.true_mean = val.toDouble();

  // read the fft index of the top Gaussian
  val = readEntry(m_stateFile, "bg_fft_ind");
  m_stfData.max.gaussian.fft_index = val.toInt();

  // read the data for the Gaussian graph
  scanGaussianGraphData();

  m_stfData.max.gaussian.wu_name = wuName();

  if(sig) emit newGaussian(m_stfData.max.gaussian);
  }
}

/*------------------------------------------------------------------------ */
void SetiContainer::scanGaussianGraphData()
{
QString val;

for(int i=0;i<=63;i++)
  {
  val = readEntry(m_stateFile, QString("bg_pot %1").arg(i));
  m_stfData.max.gaussian.data[i] = val.toDouble();
  }
m_stfData.max.gaussian.data_len = 64;
}

/*------------------------------------------------------------------------ */
void SetiContainer::scanPulseData(bool sig)
{
QString val;

// read the score of the top pulse
double oldpulse = m_stfData.max.pulse.score;
val = readEntry(m_stateFile, "bp_score");
m_stfData.max.pulse.score = val.toDouble();

double diff = fmod(m_stfData.max.pulse.score-oldpulse, DIGITS);
if(diff > 0.0)
  {
  // read the power of the top pulse
  val = readEntry(m_stateFile, "bp_power");
  m_stfData.max.pulse.power = val.toDouble();

  // read the mean value of the top pulse
  val = readEntry(m_stateFile, "bp_mean");
  m_stfData.max.pulse.mean = val.toDouble();

  // read the period of the top pulse
  val = readEntry(m_stateFile, "bp_period");
  m_stfData.max.pulse.period = val.toDouble();

  // read the chirp rate of the top pulse
  val = readEntry(m_stateFile, "bp_chirp_rate");
  m_stfData.max.pulse.chirprate = val.toDouble();

  // read the freq_bin value of the top pulse
  val = readEntry(m_stateFile, "bp_freq_bin");
  m_stfData.max.pulse.freq_bin = val.toInt();

  // read the time_bin value of the top pulse
  val = readEntry(m_stateFile, "bp_time_bin");
  m_stfData.max.pulse.time_bin = val.toInt();

  // read the fft length of the top pulse
  val = readEntry(m_stateFile, "bp_fft_len");
  m_stfData.max.pulse.time_bin = val.toInt();

  // read the graph data
  if(scanPulseGraphData() == false)
    debug("Pulse data could not be read.");

  m_stfData.max.pulse.wu_name = wuName();

  if(sig) emit newPulse(m_stfData.max.pulse);
  }
}

/*------------------------------------------------------------------------ */
bool SetiContainer::scanPulseGraphData()
{
bool status = false;
int  dl;

QString buf = readEntry(m_stateFile, "bp_pot");
dl = readDataString(&m_stfData.max.pulse.data[0], buf, 512);
if(dl > 0)
  {
  m_stfData.max.pulse.data_len = dl;
  status = true;
  }

return(status);
}

/*------------------------------------------------------------------------ */
void SetiContainer::scanTripletData(bool sig)
{
QString val;

// read the score of the top triplet
double oldtriplet = m_stfData.max.triplet.score;
val = readEntry(m_stateFile, "bt_score");
m_stfData.max.triplet.score = val.toDouble();

double diff = fmod(m_stfData.max.triplet.score-oldtriplet, DIGITS);
if(diff > 0.0)
  {
  // read the power of the top triplet
  val = readEntry(m_stateFile, "bt_power");
  m_stfData.max.triplet.power = val.toDouble();

  // read the mean value of the top pulse
  val = readEntry(m_stateFile, "bt_mean");
  m_stfData.max.triplet.mean = val.toDouble();

  // read the period of the top pulse
  val = readEntry(m_stateFile, "bt_period");
  m_stfData.max.triplet.period = val.toDouble();

  // read the chirp rate of the top pulse
  val = readEntry(m_stateFile, "bt_chirp_rate");
  m_stfData.max.triplet.chirprate = val.toDouble();

  // read the index values
  val = readEntry(m_stateFile, "bt_tpotind0_0");
  m_stfData.max.triplet.tpotind0_0 = val.toInt();
  val = readEntry(m_stateFile, "bt_tpotind0_1");
  m_stfData.max.triplet.tpotind0_1 = val.toInt();
  val = readEntry(m_stateFile, "bt_tpotind1_0");
  m_stfData.max.triplet.tpotind1_0 = val.toInt();
  val = readEntry(m_stateFile, "bt_tpotind1_1");
  m_stfData.max.triplet.tpotind1_1 = val.toInt();
  val = readEntry(m_stateFile, "bt_tpotind2_0");
  m_stfData.max.triplet.tpotind2_0 = val.toInt();
  val = readEntry(m_stateFile, "bt_tpotind2_1");
  m_stfData.max.triplet.tpotind2_1 = val.toInt();

  // read the bperiod value of the top pulse
  val = readEntry(m_stateFile, "bt_bperiod");
  m_stfData.max.triplet.bperiod = val.toDouble();

  // read the freq_bin value of the top pulse
  val = readEntry(m_stateFile, "bt_freq_bin");
  m_stfData.max.triplet.freq_bin = val.toInt();

  // read the time_bin value of the top pulse
  val = readEntry(m_stateFile, "bt_time_bin");
  m_stfData.max.triplet.time_bin = val.toDouble();

  // read the scale of the top pulse
  val = readEntry(m_stateFile, "bt_scale");
  m_stfData.max.triplet.scale = val.toDouble();

  // read the fft length of the top pulse
  val = readEntry(m_stateFile, "bt_fft_len");
  m_stfData.max.triplet.fft_len = val.toInt();

  // read the graph data
  if(scanTripletGraphData() == false)
    debug("Triplet data could not be read.");

  m_stfData.max.triplet.wu_name = wuName();

  if(sig) emit newTriplet(m_stfData.max.triplet);
  }
}

/*------------------------------------------------------------------------ */
bool SetiContainer::scanTripletGraphData()
{
bool status = false;
int  dl;
QString buf;

buf = readEntry(m_stateFile, "bt_pot");
dl = readDataString(&m_stfData.max.triplet.data[0], buf, 512);
if(dl > 0)
  {
  m_stfData.max.triplet.data_len = dl;
  status = true;
  }

return(status);
}

/*------------------------------------------------------------------------ */
int SetiContainer::readDataString(unsigned short int data[],
                                  QString& str, int cnt)
{
char byte[2];        // stores a byte value
int  i = 0;

if(!str.isEmpty())
  {
  QTextStream ts(&str, IO_ReadOnly);
  unsigned int val;
  while(!ts.atEnd() && i < cnt)
    {
    ts >> byte[0] >> byte[1];
    if(isxdigit(byte[0]) && isxdigit(byte[1]))
      sscanf(byte, "%x", &val);
    else
      break;
    data[i] = (unsigned short int)val;
    i++;
    }
  }

return(i);
}

/*------------------------------------------------------------------------ */
void SetiContainer::scanUserInfoData(bool sig)
{
QString val;

// read user id
val = readEntry(m_userinfoFile, "id");
m_uifData.id = val.toInt();

// read the key
val = readEntry(m_userinfoFile, "key");
m_uifData.key = val.toInt();

// read the user's email address
m_uifData.email_addr = readEntry(m_userinfoFile, "email_addr");

// read the user name
m_uifData.name = readEntry(m_userinfoFile, "name");

// read the user's url
m_uifData.url = readEntry(m_userinfoFile, "url");

// read the user's country
m_uifData.country = readEntry(m_userinfoFile, "country");

// read the postal code
val = readEntry(m_userinfoFile, "postal_code");
m_uifData.postal_code = val.toInt();

// show the name?
val = readEntry(m_userinfoFile, "show_name");
if(val == "yes")
  m_uifData.show_name = true;
else
  m_uifData.show_name = false;

// show the email address?
val = readEntry(m_userinfoFile, "show_email");
if(val == "yes")
  m_uifData.show_email = true;
else
  m_uifData.show_email = false;

// read the venue parameter (whatever it is good for)
val = readEntry(m_userinfoFile, "venue");
m_uifData.venue = val.toInt();

// read the register time
m_uifData.register_time = readEntry(m_userinfoFile, "register_time");

// read last wu time
m_uifData.last_wu_time = readEntry(m_userinfoFile, "last_wu_time");

// read last result time
m_uifData.last_result_time = readEntry(m_userinfoFile, "last_result_time");

// read number of sent wus (not used any more)
val = readEntry(m_userinfoFile, "nwus");
m_uifData.nwus = val.toInt();

// read number of completed wus
val = readEntry(m_userinfoFile, "nresults");
m_uifData.nresults = val.toInt();

// read the total cpu time
val = readEntry(m_userinfoFile, "total_cpu");
m_uifData.total_cpu = val.toDouble();

// read the parameter index value (what's that?)
val = readEntry(m_userinfoFile, "params_index");
m_uifData.params_index = val.toInt();

// finally, send emit a signal to notify others of the change
if(sig) emit updatedUserInfoData();
}

/*------------------------------------------------------------------------ */
void SetiContainer::scanWorkUnitData(bool sig)
{
QString val;

// read the task (seti, of course)
m_wufData.task = readEntry(m_workunitFile, "task");

// read the version (of what?)
val = readEntry(m_workunitFile, "version");
m_wufData.version = val.toInt();

// read the name of the wu
m_wufData.name = readEntry(m_workunitFile, "name");

// read the data type
m_wufData.data_type = readEntry(m_workunitFile, "data_type");

// read the data class
val = readEntry(m_workunitFile, "data_class");
m_wufData.data_class = val.toInt();

// read the splitter version
m_wufData.splitter_version = readEntry(m_workunitFile, "splitter_version");

// read the start ra value
val = readEntry(m_workunitFile, "start_ra");
m_wufData.start_ra = val.toDouble();

// read the start dec value
val = readEntry(m_workunitFile, "start_dec");
m_wufData.start_dec = val.toDouble();

// read the end ra value
val = readEntry(m_workunitFile, "end_ra");
m_wufData.end_ra = val.toDouble();

// read the end dec value
val = readEntry(m_workunitFile, "end_dec");
m_wufData.end_dec = val.toDouble();

// read the angle range
val = readEntry(m_workunitFile, "angle_range");
m_wufData.angle_range = val.toDouble();

// read the time recorded
m_wufData.time_recorded = readEntry(m_workunitFile, "time_recorded");

// read the subband center frequency
val = readEntry(m_workunitFile, "subband_center");
m_wufData.subband_center = val.toDouble();

// read the subband base frequency
val = readEntry(m_workunitFile, "subband_base");
m_wufData.subband_base = val.toDouble();

// read the subband sample rate
val = readEntry(m_workunitFile, "subband_sample_rate");
m_wufData.subband_sample_rate = val.toDouble();

// read the fft length
val = readEntry(m_workunitFile, "fft_len");
m_wufData.fft_len = val.toInt();

// read the ifft (?) length
val = readEntry(m_workunitFile, "ifft_len");
m_wufData.ifft_len = val.toInt();

// read the subband number
val = readEntry(m_workunitFile, "subband_number");
m_wufData.subband_number = val.toInt();

// read the receiver's id
m_wufData.receiver = readEntry(m_workunitFile, "receiver");

// read the number of samples
val = readEntry(m_workunitFile, "nsamples");
m_wufData.nsamples = val.toInt();

// read the tape version
m_wufData.tape_version = readEntry(m_workunitFile, "tape_version");

// read the number of sky positions
val = readEntry(m_workunitFile, "num_positions");
m_wufData.num_positions = val.toInt();

// read the coordinates
m_wufData.coordinates.clear();
if(m_wufData.num_positions > 0)
  {
  QString e;
  for(int i=0; i<m_wufData.num_positions; i++)
    {
    val = readEntry(m_workunitFile, (e = "coord%1").arg(i));
    if(!val.isEmpty())
      m_wufData.coordinates.append(val);
    }
  }

// finally, send emit a signal to notify others of the change
if(sig) emit updatedWorkUnitData();
}

/*------------------------------------------------------------------------ */
int SetiContainer::readClientVersion()
{
int version(0);

QString versionFile(m_setiDirectory + "/version.sah");

if(QFile::exists(versionFile))
  {
  QString entry = readEntry(versionFile, "major_version");
  version = 100*entry.toInt();
  entry = readEntry(versionFile, "minor_version");
  version += entry.toInt();
  }
else if(QFile::exists(m_setiDirectory + "/version.txt"))
  version = 100;

return(version);
}


/*------------------------------------------------------------------------ */
double SetiContainer::progressRate()
{
double prograte = (cpuTime() > 0.0) ? (3600.0/cpuTime())*progress() : 0.0;
return(prograte);
}

/*------------------------------------------------------------------------ */
double SetiContainer::remainingTime()
{
double timeleft = (progressRate() > 0.0) ?
                    3600.0*(100.0 - progress())/progressRate() : 0.0;
return(timeleft);
}

/*------------------------------------------------------------------------ */
QString SetiContainer::remainingTimeAsString()
{
double tl = remainingTime();
return(convertTime(tl, true));
}

/*------------------------------------------------------------------------ */
QString SetiContainer::convertTime(double time, bool hms)
{
unsigned int d, h, m, s;
QString t;

d = static_cast<unsigned int>(time/86400.0);
h = static_cast<unsigned int>(time) % 86400 / 3600;
m = static_cast<unsigned int>(time) % 3600 / 60;
s = static_cast<unsigned int>(time) % 60;

// display with days?
if( hms || (hms == false && d == 0))
  {
  h += d*24;
  t.sprintf("%u:%02u:%02u", h, m, s);
  }
else
  {
  if(d >= 365)
    t.sprintf("%.4f yrs", time/31536000.0);
  else
    t.sprintf("%ud:%02u:%02u:%02u", d, h, m, s);
  }

return(t);
}

/*------------------------------------------------------------------------ */
QString SetiContainer::convertRAToString(double ra)
{
int h, m, s;
QString val;

h = static_cast<int>(ra);
m = static_cast<int>((ra - h)*60);
s = static_cast<int>((ra - h)*3600 - m*60);
val.sprintf("%2dh %02dmin %02ds", h, m, s);

return(val);
}

/*------------------------------------------------------------------------ */
QString SetiContainer::convertDecToString(double dec)
{
int d, m, s;
QString val;

d = static_cast<int>(dec);
m = static_cast<int>((dec - d)*60);
s = static_cast<int>((dec - d)*3600 - m*60);
val.sprintf("%2d%02d'%02d\"", d, m, s);

return(val);
}

/*------------------------------------------------------------------------ */
QString SetiContainer::timeRecordedString()
{
QString tr = timeRecorded();

int k = tr.find('(');
int l = tr.find(')');
if(k > -1 && l > -1 && l > k)
  tr = tr.mid(k+1, l-k-1);
else
  tr = "";

return(tr);
}

/*------------------------------------------------------------------------ */
double SetiContainer::averageCPUTime()
{
double avgcpu = (numberOfResults() > 0) ? totalCPUTime()/(double)numberOfResults() : 0.0;
return(avgcpu);
}

/*------------------------------------------------------------------------ */
QString SetiContainer::lastResultTimeString()
{
QString lrt = lastResultTime();

int k = lrt.find('(');
int l = lrt.find(')');
if(k > -1 && l > -1 && l > k)
  lrt = lrt.mid(k+1, l-k-1);
else
  lrt = "";

return(lrt);
}

/*------------------------------------------------------------------------ */
QString SetiContainer::registerTimeString()
{
QString rt = registerTime();

int k = rt.find('(');
int l = rt.find(')');
if(k > -1 && l > -1 && l > k)
  rt = rt.mid(k+1, l-k-1);
else
  rt = "";

return(rt);
}

/*------------------------------------------------------------------------ */
bool SetiContainer::exists(int sah)
{
bool ex;

switch(sah)
  {
  case SC_StateFile:
    ex = QFile::exists(m_stfFileName);
    break;
  case SC_UserInfoFile:
    ex = QFile::exists(m_uifFileName);
    break;
  case SC_WorkUnitFile:
    ex = QFile::exists(m_wufFileName);
    break;
  case SC_ResultFile:
    ex = QFile::exists(m_rsfFileName);
    break;
  case SC_WtempFile:
    ex = QFile::exists(m_wtpFileName);
    break;
  case SC_ResultHeaderFile:
    ex = QFile::exists(m_rhdFileName);
    break;
  default:
    ex = false;
    break;
  }

return(ex);
}

/*------------------------------------------------------------------------ */
double SetiContainer::teraFlops(double ar, int version)
{
double tf(0.0);

if(version == 0) version = clientVersion();

if(version >= 200 && version <= 204)
  {
  if(ar >= 0.2255 && ar <= 1.1274)
    tf = 1.45*pow(ar, -0.0826);
  else
    tf = 1.34;
  }
else if(version >= 300 && version < 303)
  {
  if(ar < 0.2255)
    tf = 2.37;

  if(ar >= 0.2255 && ar <= 1.1274)
    tf = 2.58*pow(ar, -0.1531);

  if(ar > 1.1274)
    tf = 2.23*pow(ar, -0.0115);
  }
else if(version >= 303)
  {
  if(ar < 0.2255)
    tf = 3.54*exp(0.0327*ar);

  if(ar >= 0.2255 && ar <= 1.1274)
    tf = 3.74*pow(ar, -0.1075);

  if(ar > 1.1274)
    tf = 3.37*pow(ar, -0.0065);
  }
else
  tf = 1.5;

return(tf);
}

/*------------------------------------------------------------------------ */
double SetiContainer::megaFlopsPerSecond()
{
return((10.0*teraFlops(angleRange())*progressRate())/3.6);
}

/*------------------------------------------------------------------------ */
int SetiContainer::checkClientState()
{
// for delayed switching from Finished to Connecting
static int count = 0;
// make a back-up of the old state
int oldState = m_cltState;

if(exists(SC_WtempFile) && !exists(SC_ResultHeaderFile))
  {
  if(/*!exists(SC_ResultHeaderFile) && !exists(SC_WorkUnitFile) &&*/
     exists(SC_ResultFile))
    {
    // Finished
    if(isClientRunning())
      {
      /* This indicates that we can control the client locally, so
         store the pid. */
      m_localClt = true;
      // Follow the sequence Running->Finished->Connecting
      if(oldState == Running || oldState == Stopped)
        {
        m_cltState = Finished;
        count    = 0;
        }
      else if(oldState == Finished)
        {
        // count up to 3 (~1.5 s) until switching to Connecting
        count++;
        if(count >= 3)
          m_cltState = Connecting;
        }
      }
    else
      {
      m_cltPid    = (pid_t)0;
      m_cltState  = Finished;
      }
    }
  else
    {
    // Loading
    // check the state of a local client and switch to stop if it was killed
    if(isClientRunning())
      {
      m_localClt = true;
      m_cltState = Loading;
      }
    else
      {
      if(m_localClt)
        {
        m_cltPid   = (pid_t)0;
        m_cltState = Stopped;
        }
      else
        {
        // never switch from Running or Stopped to Loading directly, do at least
        // once a Finished
        if(oldState == Running || oldState == Stopped)
          m_cltState = Finished;
        else
          m_cltState = Loading;
        }
      }
    }
  }
else // wtemp.sah doesn't exist
  {
  switch(m_cltState)
    {
    case Finished:
    case Loading:
    case Connecting:
      // only update if a new WU is present
      if(exists(SC_WorkUnitFile))
        {
        // set all entries to zero and read the new data
        initAllData(true);
        // make sure that the timing data are read the first time
        m_stfData.prog = -1.0;
        // switch to Running in any case
        m_cltState = Running;
        // notify others that a new WU has arrived
        emit newWorkUnit(m_wufData);
        }
      break;
    case Running:
      if(m_localClt && !isClientRunning())
        {
        m_cltState = Stopped;
        m_cltPid   = (pid_t)0;
        }
      break;
    case Stopped:
      if(isClientRunning())
        {
        /* This indicates that we can control the client locally, so
           store the pid. */
        m_localClt = true;
        if(exists(SC_WorkUnitFile))
          m_cltState = Running;
        else
          m_cltState = Connecting;
        }
      break;
    default:
      #ifndef NDEBUG
        qDebug("SetiContainer (%s): Invalid client state.",
               (const char*)directory());
      #endif
      break;
    }
  }

// emit a signal if state has changed or is Loading
if((m_cltState != oldState) || m_cltState == Loading)
  {
  //qDebug("SetiContainer (%s): rs:%d rh:%d wu:%d wt:%d ns:%d os:%d",
  //       (const char*)directory(),
  //       (int)rsfEx, (int)rhdEx, (int)wufEx, (int)wtpEx, m_cltState, oldState);
  switch(m_cltState)
    {
    case Finished:
      {
      updateSahData(false);
      emit stateChanged(m_cltState);
      break;
      }
    case Loading:
      {
      // get the file size of wtemp.sah
      QFileInfo fi(m_wtpFileName);
      // approximate file size of WUs is 360000
      int loaded = (int)( 100*((double)fi.size()/360000.0) );
      if(loaded > 100)
        loaded = 100;
      // emit a signal every time to keep others up-to-date with the
      // actual size of wtemp.sah
      emit stateChanged(m_cltState, loaded);
      break;
      }
    default:
      {
      emit stateChanged(m_cltState);
      break;
      }
    }
  }

return(m_cltState);
}

/*------------------------------------------------------------------------ */
int SetiContainer::getClientPid()
{
pid_t   pid = -1;
QString pidfn = directory() + "/pid.sah";
QFile   pidfile(pidfn);

if(QFile::exists(pidfn))
  {
  // Read the time stamp of the file...
  QFileInfo fi(pidfn);
  QDateTime dt = QDateTime(fi.lastModified());
  QString   ts = dt.toString();
  if(ts != m_pidTimeStamp)
    {
    m_pidTimeStamp = ts;
    if(pidfile.open(IO_ReadOnly))
      {
      QTextStream t(&pidfile);
      t >> pid;
      pidfile.close();
      }
    }
  }

return(pid);
}

/*------------------------------------------------------------------------ */
bool SetiContainer::startClient(const QString& startexe)
{
//bool running;

if( !startexe.isEmpty() && !startexe.isNull() )
  {
  // Prepare the process
  m_startProc->clearArguments();
  *m_startProc << QStringList::split(' ', startexe);
  // we have to change to the setiathome directory to properly start the client
  pid_t oldPid = getClientPid();
  QString curdir = QDir::currentDirPath();
  QDir::setCurrent(directory());
  m_startProc->start(KProcess::DontCare, KProcess::Stdout);
  QDir::setCurrent(curdir);
  // wait 0.2 secs at maximum
  int i = 20;
  while((m_cltPid = getClientPid()) == oldPid && i > 0)
    {
    usleep(10000);
    i--;
    }
  // and check if the process is running
  if(m_startProc->isRunning())
    m_procPid = m_startProc->pid();
  else
    m_procPid = 0;
  }

return((m_procPid > 0) || (m_cltPid > 0));
}

/*------------------------------------------------------------------------ */
bool SetiContainer::stopClient(const QString& stopexe)
{
int ret(0);

if(!stopexe.isNull() && !stopexe.isEmpty())
  {
  m_stopProc->clearArguments();
  *m_stopProc << QStringList::split(' ', stopexe);
  m_stopProc->start(KProcess::DontCare, KProcess::NoCommunication);
  ret = 0;
  }
else
  {
  if(m_procPid > 0)
    {
    ret = ::kill(m_procPid, SIGTERM);
    m_procPid = 0;
    }
  if(m_cltPid > 0 && m_cltPid != m_procPid)
    {
    if(isClientRunning(true))
      {
      ret = ::kill(m_cltPid, SIGTERM);
      m_cltPid = 0;
      }
    }
  }

return(ret == 0);
}


/*------------------------------------------------------------------------ */
bool SetiContainer::isClientRunning(bool checkCmdLine)
{
// read from pid.sah if pid is zero
if(m_cltPid <= 0)
  m_cltPid = getClientPid();

if(m_procPid > 0 && m_procPid != m_cltPid)
  return(m_startProc->isRunning());

// check the command line to see whether it is really the S@h client
if(checkCmdLine && m_cltPid > 0)
  {
  QString procstr = QString("/proc/%1/cmdline").arg(m_cltPid);
  QFile procfile(procstr);
  if(procfile.open(IO_ReadOnly))
    {
    QString cmd;
    QTextStream t(&procfile);
    t >> cmd;
    procfile.close();
    // We assume that if there's somewhere the expression "setiathome" in the
    // cmd line, it is a S@h client.
    // If "setiathome" is not found return false.
    if(cmd.find("setiathome") == -1)
      return(false);
    }
  }

// don't send a signal but only check for error;
// if zero, process is alive
if(m_cltPid > 0 && ::kill(m_cltPid, 0) == 0)
  return(true);
else
  return(false);
}

/*------------------------------------------------------------------------ */
int SetiContainer::checkClientStatePassive()
{
/* Evaluate the status of this location (either running or stopped)
   by checking every 2 minutes if the progress value has increased. */
int oldState = m_cltState;
if(!m_localClt && m_cltState == Running)
  {
  // this is to avoid a stopped process as being indicated as running
  // during the first check
  if(m_oldProgress > -1.0)
    {
    // now compare
    double diff = fmod(m_stfData.prog-m_oldProgress, DIGITS);
    if(diff == 0.0)
      m_cltState = Stopped;
    }
  m_oldProgress = m_stfData.prog;
  }

if(m_cltState != oldState) emit stateChanged(m_cltState);

return(m_cltState);
}

