/***************************************************************************/
/**
  * @file  seticontainer.h
  * @brief The SetiContainer class (declaration).
  *
  * @b version    : 0.5pre - 2002.01.20<br>
  * @b begin      : Fri Nov 17 2000<br>
  * @b copyright  : &copy; 2000-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This file contains the implementation of the SetiContainer class. More
  * details can be found directly in the documentation of the SetiContainer
  * class.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef SETICONTAINER_H
#define SETICONTAINER_H

#include <sys/types.h>

#include <qobject.h>

#include "setidefs.h"

class KProcess;

/**
  * The SetiContainer class is a C++ class that allows convenient access to
  * all the information in the state files of a %SETI@home client. It
  * periodically examines these files and updates its own data members
  * accordingly. Other programs can use this class to extract information
  * about the progress of the client, the found signals, or the work unit
  * being processed. Additional functions offer more advanced info (like
  * the amount of TeraFlops in a work unit, estimated time of completion,
  * etc). The SetiContainer class is based on the Qt toolkit, and extensively
  * uses Qt's Signal/Slot technique.
  *
  * Using the SetiContainer is very simple. Here is a short code snippet:
  *
  * <PRE>
  * Seticontainer* sec = new SetiContainer("/home/crunchking/setiathome/");
  *
  * double  progress = sec->progress();
  * QString wuname   = sec->wuName();
  * cout << wuname << ": " << progress << " %";
  * </PRE>
  *
  * This snippet creates a SetiContainer class which monitors the %SETI@home
  * client in the given directory. The name of the work unit and the client's
  * progress are read and printed to stdout. By default, the SetiContainer
  * class refreshes its data every 30 seconds. To change the refresh interval
  * to 60 seconds, just do
  *
  * <PRE>
  * sec->setRefreshInterval(60);
  * </PRE>
  *
  * SetiContainer offers more advanced functions: it calculates the amount
  * of floating point operations (Flops) required to process the WU in
  * dependence on the angle range, calculates the estimated time to complete
  * the work unit, or converts the given time values from seconds into a
  * nicely formatted string.
  *
  * Another way of using this class is to inherit from it, and to extend it
  * with all the features you would like to have. Have a look at the
  * Ksetiwatch source code (http://ksetiwatch.sourceforge.net) to see how it
  * is done.
  *
  * All information from the client's output files is organized in a logical
  * manner using dedicated structures. For example, there are the
  * WorkUnitData structure, the UserInfoData structure, or the
  * WUScore structure (which itself consists of four other structures:
  * SpikeScore, GaussianScore, PulseScore, and TripletScore). By proper naming
  * of these structures in your code, the resulting code is very readable.
  * Here is an example; consider we have two WUScore structures, one holds
  * the record signals, the other contains some newly found signals:
  *
  * <PRE>
  * WUScore record;
  * WUScore new;
  * ...
  * ...
  * ...
  * if(new.pulse.score > record.pulse.score)
  *   {
  *   record.pulse = new.pulse;
  *   }
  * ...
  * </PRE>
  *
  * This piece of code checks if the score value of the new pulse is larger
  * than that of the record pulse, and -if this is true- copies the entire
  * pulse data to the record WUScore.
  *
  * SetiContainer makes extensive use of Qt's signal/slot mechanism. For
  * example, the progressIncreased() signal is emitted whenever the client
  * has updated its state file with new data, or the newPulse() signal is
  * emitted when a new high pulse signal was found. Connecting to a signal
  * looks like this:
  *
  * <PRE>
  * connect(this,     SIGNAL(progressIncreased()),
  *         mywidget, SLOT(slotUpdateTimeData()));
  * </PRE>
  *
  * Using this approach, the GUI elements are only updated when required,
  * resulting in very efficient code.
  *
  * In addition, the SetiContainer class provides a sophisticated way to
  * check the state of the %SETI@home client. It can distinguish between
  * four different states: Running, Stopped, Finished, and Loading.
  * Whenever the state changes, the stateChanged() signal is
  * emitted.
  *
  * @author Gordon Machel <gmachel@users.sourceforge.net>
  */
class SetiContainer : public QObject
{
Q_OBJECT

public:
  /** Constructs a Seti container class.
    *
    * @param dir     The working folder of the %SETI@home client.
    * @param refresh The refresh interval. This class looks every @a refresh
    *                seconds if the data needs to be updated. Default is
    *                30 seconds.
    */
  SetiContainer(const QString& dir, int refresh=30);
  /** The destructor. */
  ~SetiContainer();
  /** Sets the refresh interval to @a refresh seconds.
    *
    * @param refresh The refresh interval in seconds.
    */
  void setRefreshInterval(int refresh);
  /** Returns the %SETI@home directory. */
  QString directory() const {return m_setiDirectory;}
  /** Sets the location's directory. */
  void setDirectory(const QString& d);
  /** Returns the current refresh interval time in seconds. */
  int refreshInterval() {return(m_refreshIval/1000);}
  /** Returns the number of the currently processed Fast Fourier
    * Transformation.
    */
  int fftNumber() {return(m_stfData.ncfft);}
  /** Returns the length (in data points) of the currently processed Fast
    * Fourier Transformation.
    */
  int fftLength() {return(m_stfData.fl);}
  /** Returns the current chirp rate (or drift rate) in Hz/s. */
  double chirpRate() {return(m_stfData.cr);}
  /** Returns the cpu time (in seconds) spent on the current work unit. */
  double cpuTime() {return(m_stfData.cpu);}
  /** Returns the size of the file @c outfile.sah which collects data about
    * interesting signals to be returned to the %SETI@home server. If this
    * value has increased the client has found a new interesting signal.
    */
  int outfileSize() {return(m_stfData.outfilepos);}
  /** This is a parameter used internally by the %SETI@home client. */
  int potFrequency() {return(m_stfData.potfreq);}
  /** This is a parameter used internally by the %SETI@home client. */
  int potActivity() {return(m_stfData.potactivity);}
  /** Returns the progress of the client in per cent. */
  double progress() {return(100.0*(m_stfData.prog < 0.0 ? 0.0 : m_stfData.prog));}
  /** Returns the client's progress rate (in % per hour) based on the
    * actual percentage done and the corresponding cpu time.
    */
  double progressRate();
  /** Returns the estimated time (in seconds) needed to finish the work
    * unit.
    */
  double remainingTime();
  /** Returns the estimated time needed to finish the work unit in form of a
    * formatted string.
    */
  QString remainingTimeAsString();
  /** Returns a WUScore structure containing the highest signals found in
    * the current work unit so far. See WUScore for details about
    * the members of the WUScore structure.
    */
  WUScore wuScore() {return(m_stfData.max);}
  /** Returns a SpikeScore structure containing the highest spike found in
    * the current work unit so far. See SpikeScore for details about
    * the members of the SpikeScore structure.
    */
  SpikeScore spikeScore() {return(m_stfData.max.spike);}
  /** Returns a GaussianScore structure containing the highest gaussian found
    * in the current work unit so far. See GaussianScore for details
    * about the members of the GaussianScore structure.
    */
  GaussianScore gaussianScore() {return(m_stfData.max.gaussian);}
  /** Returns a PulseScore structure containing the highest pulse found
    * in the current work unit so far. See PulseScore for details
    * about the members of the PulseScore structure.
    */
  PulseScore pulseScore() {return(m_stfData.max.pulse);}
  /** Returns a TripletScore structure containing the highest triplet found
    * in the current work unit so far. See TripletScore for details
    * about the members of the TripletScore structure.
    */
  TripletScore tripletScore() {return(m_stfData.max.triplet);}
  /** Returns the user's id number. */
  int userID() {return(m_uifData.id);}
  /** Returns the user's id key. */
  int key() {return(m_uifData.key);}
  /** Returns the user's email address. */
  QString userEmailAddress() {return(m_uifData.email_addr);}
  /** Returns the user's name. */
  QString userName() {return(m_uifData.name);}
  /** Returns the user's homepage address. */
  QString userHomepage() {return(m_uifData.url);}
  /** Returns the user's country. */
  QString userCountry() {return(m_uifData.country);}
  /** Returns the user's postal code. */
  int userPostalCode() {return(m_uifData.postal_code);}
  /** If true the user's name is allowed to be shown on the %SETI@home
    * web pages.
    */
  bool showName() {return(m_uifData.show_name);}
  /** If true the user's email address is allowed to be shown on the
    * %SETI@home web pages.
    */
  bool showEmail() {return(m_uifData.show_email);}
  /** Returns the venue parameter (whatever it is good for). */
  int venue() {return(m_uifData.venue);}
  /** Returns the time the user registered with %SETI@home in form of a plain
    * string (as stored in user_info.sah).
    */
  QString registerTime() {return(m_uifData.register_time);}
  /** Returns the date and time the user registered with %SETI@home in
    * a nicely formatted string. An empty string is returned when the
    * information could not be found.
    */
  QString registerTimeString();
  /** Returns the time the last work unit was sent to the user in form of a
    * plain string (as stored in user_info.sah). Note: this information is
    * not updated any more.
    */
  QString lastWUTime() {return(m_uifData.last_wu_time);}
  /** Returns the time the user has sent his/her last work unit result to
    * the %SETI@home server in form of a plain string (as stored in
    * user_info.sah).
    */
  QString lastResultTime() {return(m_uifData.last_result_time);}
  /** Returns the date and time the last result was sent in a nicely
    * formatted string. An empty string is returned when the information
    * could not be found.
    */
  QString lastResultTimeString();
  /** Returns the number of work units sent to the user. Note: this
    * information is not updated any more.
    */
  int numberOfWUs() {return(m_uifData.nwus);}
  /** Returns the number of results the user has sent back to the %SETI@home
    * server.
    */
  int numberOfResults() {return(m_uifData.nresults);}
  /** Returns the total cpu time (in seconds) the user has processed so far.
    */
  double totalCPUTime() {return(m_uifData.total_cpu);}
  /** Returns the average CPU time (in seconds) required to process one
    * work unit.
    */
  double averageCPUTime();
  /** Returns a parameter index value. No idea what this is good for. */
  double paramsIndex() {return(m_uifData.params_index);}
  /** Returns the task identifier string. We are all in the "seti" business,
    * aren't we?
    */
  QString task() {return(m_wufData.task);}
  /** Returns an internal version number. This is not the version number of
    * the %SETI@home client.
    */
  int wuVersion() {return(m_wufData.version);}
  /** Returns the name of the work unit. */
  QString wuName() {return(m_wufData.name);}
  /** Returns the data type of the attached work unit data. */
  QString wuDataType() {return(m_wufData.data_type);}
  /** Returns the data class of the attached work unit data. */
  int wuDataClass() {return(m_wufData.data_class);}
  /** Returns the version number of the splitter (software?). */
  QString splitterVersion() {return(m_wufData.splitter_version);}
  /** Returns the right ascension (RA) value where the telescope has
    * started to scan the sky for this work unit.
    */
  double startRA() {return(m_wufData.start_ra);}
  /** Returns the declination value where the telescope has
    * started to scan the sky for the current work unit.
    */
  double startDec() {return(m_wufData.start_dec);}
  /** Returns the right ascension (RA) value where the telescope has
    * stopped to scan the sky for the current work unit.
    */
  double endRA() {return(m_wufData.end_ra);}
  /** Returns the declination value where the telescope has
    * stopped to scan the sky for the current work unit.
    */
  double endDec() {return(m_wufData.end_dec);}
  /** Returns the angle range which the telescope has scanned for the current
    * work unit.
    */
  double angleRange() {return(m_wufData.angle_range);}
  /** Calculates the amount of floating point operations that need to be
    * processed to finish a work unit depending on the angle range. This
    * function is based on the work of Lawrence Kirby and Roelof Engelbrecht.
    *
    * @param ar      The angle range of the work unit.
    * @param version The version of the %SETI@home client as an integer value
    *                (300 is "3.00", 303 is "3.03"). If omitted, this method
    *                uses the version number found in `version.sah'.
    *
    * @return The number of floating point operations in TeraFlops.
    */
  double teraFlops(double ar, int version=0);
  /** Returns the progress rate in MegaFlops/s. */
  double megaFlopsPerSecond();
  /** Returns the time the work unit was recorded (as stored in
    * in the work_unit.sah file).
    */
  QString timeRecorded() {return(m_wufData.time_recorded);}
  /** Returns the date and the time the work unit was recorded in a nicely
    * formatted string. An empty string is returned when the information
    * could not be found.
    */
  QString timeRecordedString();
  /** Returns the subband center frequency in GHz. */
  double centerFrequency() {return(m_wufData.subband_center/1.0e9);}
  /** Returns the subband base frequency in GHz. */
  double baseFrequency() {return(m_wufData.subband_base/1.0e9);}
  /** Returns the sample rate in Hz. */
  double sampleRate() {return(m_wufData.subband_sample_rate);}
  /** This parameter is used internally by the %SETI@home client. */
  int wuFFTLength() {return(m_wufData.fft_len);};
  /** This parameter is used internally by the %SETI@home client. */
  int wuIFFTLength() {return(m_wufData.ifft_len);}
  /** Returns the subband number. */
  int subbandNumber() {return(m_wufData.subband_number);}
  /** Returns the identifier string of the facility that recorded
    * the work unit. "ao1420" defines the Arecibo Radio Telescope.
    */
  QString receiver() {return(m_wufData.receiver);}
  /** Returns the number of samples in the work unit. */
  int numberOfSamples() {return(m_wufData.nsamples);}
  /** Returns the version number of the tape storing the work unit. */
  QString tapeVersion() {return(m_wufData.tape_version);}
  /** Returns the number of sky coordinates that define the path of
    * the telescope over the sky during acquisition of the work unit data.
    */
  int numberOfCoords() {return(m_wufData.num_positions);}
  /** Returns a string list with all the sky coordinates that define
    * the path of the telescope over the sky during acquisition of the work
    * unit data.
    */
  QStringList coords() {return(m_wufData.coordinates);}
  /** Returns the version number of the %SETI@home client. It is
    * returned as an integer formed of both the major and the minor
    * version number. Example: for client version 3.03, a 303 is
    * returned.
    */
  int clientVersion() {return(m_sahVersion);}
  /** Checks if the file represented by @a sah exists.
    *
    * @param sah Can be one of the following: @p SC_StateFile,
    *            @p SC_UserInfoFile, @p SC_WorkUnitFile, @p SC_ResultFile,
    *            @p SC_ResultHeaderFile, or @p SC_WtempFile.
    */
  bool exists(int sah);
  bool startClient(const QString& startexe);
  bool stopClient(const QString& stopexe);
  /** Checks if the %SETI@home client is running by trying to signal
    * the process @em pid.
    *
    * @param pid          The process id of the client. If omitted the pid
    *                     will be read from @c pid.sah.
    * @param checkCmdLine If true, this method checks whether the process
    *                     defined by @a pid is really that of a %SETI@home
    *                     client.
    *
    * @return TRUE if client is running, FALSE if it is stopped or on error.
    */
  bool isClientRunning(bool checkCmdLine = false);
  /** Returns the state of the location. */
  int clientState() const {return m_cltState;}
  /** Sets the state of the location. Setting it to -1, triggers a refresh
    * during the next update cycle. */
  void setClientState(int st) {m_cltState = st;}
  /** This function allows low-level access to the entries in the %S@h
    * output files. It reads the entry @a e from the file @a sah. Use this
    * function only if you know what you are doing!
    *
    * @param sah The file from which the entry shall be read. You can choose
    *            from four values: SC_StateFile, SC_UserInfoFile,
    *            SC_WorkUnitFile, and SC_ResultFile.
    * @param e   The entry which shall be retrieved.
    */
  QString readEntry(int sah, const QString& e);
  /** Initializes all members and data arrays of the WUScore structure .
    *
    * @param score A pointer to the WUScore structure to be cleared.
    */
  static void initWUScore(WUScore* score);
  /** Converts the time (in seconds) to a string.
    *
    * @param time The time in seconds.
    * @param hms  Defines the kind of presentation. If true, the time will be
    *             given in hours:minutes:seconds, if false in
    *             days:hours:minutes:seconds.
    */
  static QString convertTime(double time, bool hms);
  /** Converts the right ascension (RA) value to a readable string of the
    * form hours:minutes:seconds.
    *
    * @param ra The RA value.
    *
    * @return The converted string.
    */
  static QString convertRAToString(double ra);
  /** Converts the declination value to a readable string of the
    * form degrees:minutes:seconds.
    *
    * @param dec The declination value.
    *
    * @return The converted string.
    */
  static QString convertDecToString(double dec);
  /**
   * Status definition values
   */
  enum SetiStatus {Stopped, Running, Finished, Loading, Connecting};
  /** The identifiers for the different output files of the %SETI@home
    * client.
    */
  enum sahID {SC_StateFile, SC_UserInfoFile,
              SC_WorkUnitFile, SC_ResultFile,
              SC_ResultHeaderFile, SC_WtempFile};

public slots:
  /** Updates the data from all %SETI@home files.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    *
    * @return 0 in case of success, 1 on error.
    */
  int updateSahData(bool sig=true);

protected:
  int           m_refreshIval;
  int           m_sahVersion;
  bool          m_versionRead;
  QString       m_setiDirectory;
  QString       m_stfFileName;
  QString       m_wufFileName;
  QString       m_uifFileName;
  QString       m_rsfFileName;
  QString       m_rhdFileName;
  QString       m_wtpFileName;
  QStringList   m_stateFile;
  QStringList   m_workunitFile;
  QStringList   m_userinfoFile;
  QString       m_stfTimeStamp;
  QString       m_wufTimeStamp;
  QString       m_uifTimeStamp;
  QString       m_pidTimeStamp;
  StateFileData m_stfData;
  UserInfoData  m_uifData;
  WorkUnitData  m_wufData;
  int           m_cltState;

  KProcess*     m_startProc;
  KProcess*     m_stopProc;
  pid_t         m_cltPid;
  pid_t         m_procPid;
  bool          m_localClt;

protected: // Protected methods
  /** Reads the state file and puts it into a string list.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    *
    * @return 0 in case of success, 1 on error.
    */
  int readStateFile(bool sig=true);
  /** Reads the work unit file and puts it into a string list.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    *
    * @return 0 in case of success, 1 on error.
    */
  int readWorkUnitFile(bool sig=true);
  /** Reads the user info file and puts it into a string list.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    *
    * @return 0 in case of success, 1 on error.
    */
  int readUserInfoFile(bool sig=true);
  /** Reads a %SETI@home sah file into a string list.
    *
    * @param file  The file name.
    * @param list  A pointer to a QStrList where the lines are stored.
    * @param endId The end-of-file identifier. If a line identical to
    *              this string is found, reading stops. When this parameter
    *              is omitted, the entire file is read.
    *
    * @return 0 in case of success, 1 on error.
    */
  int readSahFile(const QString& file, QStringList& list,
                  const QString& endId=0);
  /** Reads an entry given by @a e from file @a fn. Reads it directly from
    * disk.
    *
    * @param fn The file name.
    * @param e  The entry identifier string.
    *
    * @return The corresponding entry, or an empty string if the entry could
    *         not be found.
    */
  QString readEntry(const QString& fn, const QString& e);
  /** Reads an entry given by @a e from the string list @a list.
    *
    * @param list A pointer to the string list containing a sah file.
    * @param e    The entry identifier string.
    *
    * @return The corresponding entry, or an empty string if the entry could
    *         not be found.
    */
  static QString readEntry(QStringList& list, const QString& e);
  /** Initializes all state file data members. */
  void initStateFileData();
  /** Initializes all work unit data members. */
  void initWorkUnitData();
  /** Initializes all user info data members. */
  void initUserInfoData();
  /** Scans the state file and puts the data in the StateFileData
    * structure.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    */
  void scanStateFile(bool sig=true);
  /** Scans the state file for spike data.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    */
  void scanSpikeData(bool sig=true);
  /** Scans the state file for basic Gaussian data.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    */
  void scanGaussianData(bool sig=true);
  /** Scans the state file for basic pulse data.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    */
  void scanPulseData(bool sig=true);
  /** Scans the state file for basic triplet data.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    */
  void scanTripletData(bool sig=true);
  /** Scans the state file for Gaussian graph data ('bg_pot xx'). */
  void scanGaussianGraphData();
  /** Scans the state file for pulse graph data ('bp_pot'). */
  bool scanPulseGraphData();
  /** Scans the state file for triplet graph data ('bt_pot'). */
  bool scanTripletGraphData();
  /** Scans the user info file and puts the data in the UserInfoData
    * structure.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    */
  void scanUserInfoData(bool sig=true);
  /** Scans the state file and puts the data in the stateFileData
    * structure.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    */
  void scanWorkUnitData(bool sig=true);
  /** Returns the version number of the %SETI@home client. It is
    * returned as an integer formed of both the major and the minor
    * version number. Example: for client version 3.03, a 303 is
    * returned.
    */
  int readClientVersion();
  /** Fills all data members (all structs) with zeros and reads the client's
    * data from disk.
    *
    * @param sig If true, Qt's signals are allowed to be emitted when
    *            necessary.
    */
  void initAllData(bool sig=false);
  /** The event handler for timer events. This method is called when
    * the internal refresh timer triggers the event to synchronize
    * the internal data structures with the %SETI@home files.
    */
  virtual void timerEvent(QTimerEvent* e);
  /** Checks the status of the client.
    *
    * @return One of the following values:
    *         @em Stopped, @em Running, @em Finished, or @em Loading.
    */
  int checkClientState();
  /** Retrieves the pid of the %S@h client.
    *
    * @return The pid, or -1 on error.
    */
  int getClientPid();
  /** This function is called every two minutes by a timer event. It checks
    * if the client has made any progress within the two minutes, and switches
    * the state to Stopped if nothing has happened. This is needed for passive
    * monitoring of remote clients on other computers, where no valid process
    * id could be obtained.
    *
    * @return The current state of the client.
    */
  int checkClientStatePassive();
  /** Reads @p cnt values from the data string @p str.
    *
    * @param data A pointer to the array where the converted shall be stored.
    * @param str  The string with data in hexadecimal form.
    * @param cnt  The number of data points to be read from the string.

    * @return True if ok, false on error.
    */
  static int readDataString(unsigned short int data[], QString& str, int cnt);
  /** No descriptions */
  void checkFileExistence();

private:
  int    m_refTimer;
  int    m_sttTimer1;
  int    m_sttTimer2;
  double m_oldProgress;
  bool   m_fastCheck;

signals: // Signals
  /** This signal is emitted when the client has updated the state file
    * with new data.
    */
  void progressIncreased();
  /** This signal is emitted when the progress value has been found to
    * be decreased. This is a rather unusual event, and should only
    * happen when the client has started to reprocess the current
    * work unit from the beginning (probably after updating the %S@h
    * client).
    */
  void progressDecreased();
  /** This signal is emitted when a new high spike has been found in the
    * work unit.
    *
    * @param sps The SpikeScore structure with the new data.
    */
  void newSpike(SpikeScore sps);
  /** This signal is emitted when a new high gaussian has been found in the
    * work unit.
    *
    * @param gas The GaussianScore structure with the new data.
    */
  void newGaussian(GaussianScore gas);
  /** This signal is emitted when a new high pulse has been found in the
    * work unit.
    *
    * @param pus The PulseScore structure with the new data.
    */
  void newPulse(PulseScore pus);
  /** This signal is emitted when a new high triplet has been found in the
    * work unit.
    *
    * @param trs The TripletScore structure with the new data.
    */
  void newTriplet(TripletScore trs);
  /** This signal is emitted when a new work unit has been loaded.
    *
    */
  void newWorkUnit(WorkUnitData wud);
  /** This signal is emitted when the work unit data have changed during
    * runtime for some reason, e.g. after mounting a network share.
    *
    */
  void updatedWorkUnitData();
  /** This signal is emitted when the user info data have changed during
    *runtime for some reason, e.g. after mounting a network share.
    *
    */
  void updatedUserInfoData();
  /** This signal is emitted when the state of the client has changed.
    *
    * @param state    The new state.
    * @param loadsize If state is 'Loading', this parameter passes the current
    *                 file size of the work unit being loaded. For all other
    *                 states it is zero (not used).
    */
  void stateChanged(int state, int loadsize=0);
};

#endif
