/***************************************************************************/
/**
  * @file  setidefs.h
  * @brief Definitions of %SETI@home-related structures.
  *
  * @b version    : 0.5pre - 2002.01.20<br>
  * @b begin      : Thu May 31 2001<br>
  * @b copyright  : &copy; 2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This file contains the definitions of several structures collecting
  * all the information which can be retrieved from the %SETI@home state
  * (sah) files. In particular, there are structures for the different
  * types of signals (SpikeScore, GaussianScore, PulseScore, TripletScore),
  * and for the user and work unit data (UserInfoData and WorkUnitData,
  * respectively).
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef SETIDEFS_H
#define SETIDEFS_H

#include <qstring.h>
#include <qstringlist.h>

/** This structure stores all relevant data of a spike. Spikes are radio
  * signals occurring at single frequencies that are strong enough to be
  * distinguished from general noise. To extract these signals from raw
  * radiowave data, baseline smoothing, data de-chirping, and Fast Fourier
  * Transforms (FFTs) need to be performed.*/
struct SpikeScore {
                  /** The spike power. */
                  double  power;
                  /** The score of the spike. */
                  double  score;
                  /** An integer value representing the spike. */
                  int     bin;
                  /** The fft index value. */
                  int     fft_index;
                  /** The fft length (number of data points). */
                  int     fft_len;
                  /** The chirp rate at which this spike was found. See
                    * StateFileData for a detailed explanation. */
                  double  chirprate;
                  /** The sky position where the spike was detected (right
                    * ascension). */
                  double  ra;
                  /** The sky position where the spike was detected
                    * (declination). */
                  double  dec;
                  /** The frequency at which the signal was found. */
                  double  frequency;
                  /** The name of work unit in which the spike was found. */
                  QString wu_name;
                  };

/** This structure stores all relevant data of a gaussian signal. Gaussians
  * are signals whose power should increase and then decrease with a
  * bell-shaped curve in time. The idea behind this is that Signals from a
  * distant transmitter should get stronger and then weaker as the
  * telescope's focal point moves across that area of the sky. The gaussian
  * test is only applied for frequency resolutions greater than or equal
  * to 0.59 Hz. Since noise can sometimes randomly simulate a gaussian,
  * the %SETI@home client only returns gaussians stronger than 3.2 times the
  * average noise level with a fit (chisq) less than 8.8. */
struct GaussianScore {
                     /** The gaussian power. This value tells us how strong
                       * the signal is relative to noise. */
                     double  power;
                     /** The score of the gaussian. This value is determined
                       * by calculating (power/chisq). */
                     double  score;
                     /** An integer value representing the gaussian. */
                     int     bin;
                     /** A fit value indicating how well a rising and
                       * falling signal fits an ideal gaussian (bell curve)
                       * profile. A lower fit value means a better fit. */
                     double  chisq;
                     /** The fitted mean value. */
                     double  true_mean;
                     /** Sigma represents the width of the gaussian. */
                     double  sigma;
                     /** The fft index value. */
                     int     fft_index;
                     /** The fft length (number of data points). */
                     int     fft_len;
                     /** The chirp rate at which this gaussian was found. See
                       * StateFileData for a detailed explanation. */
                     double  chirprate;
                     /** This array stores the gaussian shape. */
                     double  data[64];
                     /** The amount of data values in the array (should be
                       * always 64 for gaussians). */
                     int     data_len;
                     /** The sky position (right ascension). */
                     double  ra;
                     /** The sky position (declination). */
                     double  dec;
                     /** The frequency at which the signal was found. */
                     double  frequency;
                     /** The name of work unit in which the gaussian was
                       * found. */
                     QString wu_name;
                     };

/** This structure stores all relevant data of a pulse. Pulses are a series
  * of evenly spaced spikes (in time). To find such a series, the %SETI@home
  * client applies a special test called a "fast folding algorithm". Pulses
  * are searched for all frequency resolutions greater than or equal to
  * 0.59 Hz. Any pulse with a score of greater than 1 will be reported when
  * the client returns a result to Berkeley.*/
struct PulseScore {
                  /** The power of the pulse relative to the noise level. */
                  double  power;
                  /** The score of the pulse. Because both interference
                    * signals (RFI) and random noise can simulate a pulsed
                    * signal, a minimum threshold for acceptance is set.
                    * This threshold is calculated dynamically and depends
                    * upon the period and the number of times the data has
                    * been folded. The score is the ratio of the pulse
                    * amplitude to this threshold value. */
                  double  score;
                  /** The mean value of the data. */
                  double  mean;
                  /** The period of the pulse in seconds. */
                  double  period;
                  /** Details unknown. */
                  int     freq_bin;
                  /** Details unknown. */
                  double  time_bin;
                  /** The fft length (number of data points). */
                  int     fft_len;
                  /** The chirp rate at which this pulse was found. See
                    * StateFileData for a detailed explanation. */
                  double  chirprate;
                  /** This array stores the pulse profile. */
                  unsigned short int data[512];
                  /** The amount of data values in the array (variable
                    * for pulses). */
                  int     data_len;
                  /** The sky position (right ascension). */
                  double  ra;
                  /** The sky position (declination). */
                  double  dec;
                  /** The frequency at which the signal was found. */
                  double  frequency;
                  /** The name of work unit in which the pulse was found. */
                  QString wu_name;
                  };

/** This structure stores all relevant data of a triplet. Triplets are sets
  * of three evenly spaced spikes. The %SETI@home client tests for triplets
  * by looking at every pair of spikes above a certain threshold power.
  * It then looks for another spike precisely between the two spikes. If
  * one is found, a triplet is logged and sent back to the %SETI@home
  * server. */
struct TripletScore {
                    /** The power of the triplet relative to the noise
                      * level. */
                    double  power;
                    /** The score of the triplet. Currently the score is
                      * identical to the power. */
                    double  score;
                    /** The mean value of the data. */
                    double  mean;
                    /** The period of the triplet. This value tells us how
                      * far apart the three spikes are (in seconds). */
                    double  period;
                    /** Details unknown. */
                    double  bperiod;
                    /** Marks the position of the first pulse of the triplet
                      * in the data set. */
                    int     tpotind0_0;
                    /** Always identical to @em tpotind0_0. */
                    int     tpotind0_1;
                    /** Marks the position of the second pulse of the triplet
                      * in the data set. */
                    int     tpotind1_0;
                    /** Always identical to @em tpotind1_0. */
                    int     tpotind1_1;
                    /** Marks the position of the third pulse of the triplet
                      * in the data set. */
                    int     tpotind2_0;
                    /** Always identical to @em tpotind2_0. */
                    int     tpotind2_1;
                    /** Details unknown. */
                    int     freq_bin;
                    /** Details unknown. */
                    double  time_bin;
                    /** Details unknown. */
                    double  scale;
                    /** The fft length (number of data points). */
                    int     fft_len;
                    /** The chirp rate at which this pulse was found. See
                      * StateFileData for a detailed explanation. */
                    double  chirprate;
                    /** This array stores the triplet profile. */
                    unsigned short int data[512];
                    /** The amount of data values in the array (should be
                      * always 512 for triplets). */
                    int     data_len;
                    /** The sky position (right ascension). */
                    double  ra;
                    /** The sky position (declination). */
                    double  dec;
                    /** The frequency at which the signal was found. */
                    double  frequency;
                    /** The name of work unit in which the triplet was found. */
                    QString wu_name;
                    };

/** The WUScore structure collects all spike, gaussian, pulse, and triplet
  * data of a work unit. */
struct WUScore  {
                /** A SpikeScore structure with the spike data. */
                SpikeScore    spike;
                /** A GaussianScore structure with the gaussian data. */
                GaussianScore gaussian;
                /** A PulseScore structure with the pulse data. */
                PulseScore    pulse;
                /** A TripletScore structure with the triplet data. */
                TripletScore  triplet;
                };

/** The StateFileData structure contains all information that can be found
  * in the @c state.sah file. */
struct StateFileData {
                     /** The number of processed Fast Fourier
                       * Transformations at the time of the last write to the
                       * state file. The FFT decomposes a signal taken over a
                       * certain period of time into its frequency components.
                       */
                     int     ncfft;
                     /** The chirp (or drift) rate in Hz/s. Assuming that the
                       * extraterrestrial signals are coming from a moving
                       * object (relative to the Earth), the signals are
                       * received with a certain frequency change. This is
                       * the well-known Doppler effect, which can also be
                       * observed e.g. for an approaching car horn. The amount
                       * of the frequency shift depends on the relative
                       * velocity of the extraterrestrial object with respect
                       * to Earth. Unfortunately we don't exactly know what the
                       * relative velocity is (since it varies from object to
                       * object), so the client needs to search a range of
                       * values (-50 Hz/s to +50 Hz/s for version 3.03).
                       */
                     double  cr;
                     /** The FFT length at the time of the last write to the
                       * state file. */
                     int     fl;
                     /** The CPU time in seconds. */
                     double  cpu;
                     /** The percentage done by the client. This is a value
                       * between 0 and 1. */
                     double  prog;
                     /** Used by the client to pick up the processing at the
                       * point where it stopped at shut-down. */
                     int     potfreq;
                     /** Used by the client to pick up the processing at the
                       * point where it stopped at shut-down. */
                     int     potactivity;
                     /** Represents the size of the file `outfile.sah', which
                       * contains signals to be returned to the server. */
                     int     outfilepos;
                     /** The maximum signals found in the work unit. */
                     WUScore max;
                     };

/** The UserInfoData structure contains all user data. All the
  * information is  retrieved from the @c user_info.sah file. */
struct UserInfoData {
                    /** The user's unique ID number. */
                    int     id;
                    /** The user's key (?). */
                    int     key;
                    /** The user's email address. */
                    QString email_addr;
                    /** The name of the user. */
                    QString name;
                    /** The url address of the user's homepage (if given). */
                    QString url;
                    /** The user's home country. */
                    QString country;
                    /** The postal code of the user's home town. */
                    int     postal_code;
                    /** This flag decides whether the user's name is allowed
                      * to be shown on the %SETI@home web pages. */
                    bool    show_name;
                    /** This flag decides whether the user's email address is
                      * allowed to be shown on the %SETI@home web pages. */
                    bool    show_email;
                    /** This value specifies the general location the user is
                        `working' for %SETI@home (home, business,
                        university, etc.). */
                    int     venue;
                    /** This string contains the time and date the user
                      * registered with %SETI@home. */
                    QString register_time;
                    /** This string contains the time and date the last
                      * work unit was sent to the user. This information is
                      * not updated any more. */
                    QString last_wu_time;
                    /** This string contains the time and date the user has
                      * sent back the last completed work unit to the
                      * server. */
                    QString last_result_time;
                    /** The number of work units sent to the user.  This
                      * information is not updated any more. */
                    int     nwus;
                    /** The number of results the user has sent back to the
                      * server. Equivalent to the number of completed work
                      * units. */
                    int     nresults;
                    /** The total CPU time (in seconds) which was needed to
                      * complete all the work units. */
                    double  total_cpu;
                    /** Details unknown. */
                    int     params_index;
                    };

/** This structure holds all work-unit-related data. All the information is
  * retrieved from the @c work_unit.sah file. */
struct WorkUnitData {
                    /** What has to be done with the data? Always "seti". */
                    QString     task;
                    /** This is an internal version number (not the one of
                      * the client!). */
                    int         version;
                    /** The name of the work unit. */
                    QString     name;
                    /** An internal value. Details unknown. */
                    QString     data_type;
                    /** An internal value. Details unknown. */
                    int         data_class;
                    /** An internal version number. Has to do with the
                      * splitter that splits the data stream from Arecibo
                      * into handy work units. */
                    QString     splitter_version;
                    /** This is the right ascension (RA) start value. Defines
                      * where the telescope has started to scan the sky for
                      * this work unit. */
                    double      start_ra;
                    /** This is the declination start value. Defines
                      * where the telescope has started to scan the sky for
                      * this work unit. */
                    double      start_dec;
                    /** This is the right ascension (RA) end value. Defines
                      * where the telescope has stopped to scan the sky for
                      * this work unit. */
                    double      end_ra;
                    /** This is the declination end value. Defines
                      * where the telescope has stopped to scan the sky for
                      * this work unit. */
                    double      end_dec;
                    /** The angle range (together with the above RA and
                      * declination values) defines the part of the sky that
                      * has been scanned for this work unit. A large angle
                      * range means that the telescope has swept very quickly
                      * between the start and end position. */
                    double      angle_range;
                    /** This string contains the date and time this work
                      * unit was recorded. */
                    QString     time_recorded;
                    /** The center frequency of the subband covered by this
                      * work unit. */
                    double      subband_center;
                    /** The base frequency of the subband covered by this
                      * work unit. */
                    double      subband_base;
                    /** The rate at which the data are sampled. */
                    double      subband_sample_rate;
                    /** Details unknown. */
                    int         fft_len;
                    /** Details unknown. */
                    int         ifft_len;
                    /** The %SETI@home search covers a 2.5 MHz band. This
                      * frequency band is broken down into 256 subbands.
                      * This value tells you the subband number of this work
                      * unit. */
                    int         subband_number;
                    /** This string contains an identifier for the receiver.
                      * "ao1420" is the Arecibo Telescope. */
                    QString     receiver;
                    /** The number of samples in the work unit. */
                    int         nsamples;
                    /** Details unknown. */
                    QString     tape_version;
                    /** The number of positions that define the path of the
                      * telescope over the sky. */
                    int         num_positions;
                    /** A string list containing the coordinates that define
                      * the path of the telescope. */
                    QStringList coordinates;
                    };

#endif

