/***************************************************************************/
/** @file     setigraph.cpp
  * @brief    The base widget for all signal graphs (implementation).
  *
  * @b begin      : Thu Oct 19 2000<br>
  * @b copyright  : &copy; 2000-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <math.h>
#include <float.h>

#include <klocale.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <kglobalsettings.h>

#include <qpopupmenu.h>
#include <qpainter.h>
#include <qcursor.h>

#include "setigraph.h"
#include "setiloc.h"

/*------------------------------------------------------------------------ */
SetiGraph::SetiGraph(SetiLoc* loc, bool shwPop, bool allowModes,
                     QWidget *parent, const char *name)
         : QWidget(parent,name)
{
// Make a backup of the SetiLoc class and the boolean switches
m_location = (SetiLoc*)loc;
if(m_location)
  setName(m_location->wuName());
else
  setName("");
m_showPopup    = shwPop;
m_allowModeSel = allowModes;

if(m_showPopup)
  {
  m_gpOptions = new KPopupMenu(this, "Seti Graph");
  m_gpOptions->setCheckable(true);
  if(m_allowModeSel)
    {
    m_gpOptions->insertItem(i18n("Bar Chart"), BarChart);
    m_gpOptions->insertItem(i18n("Scatter Chart"), ScatterChart);
    m_markerLines = new QPopupMenu(this);
    m_markerLines->setCheckable(true);
    m_markerLines->insertItem(i18n("Marker"), 0);
    m_markerLines->insertItem(i18n("Connecting Lines"), 1);
    m_gpOptions->insertItem(i18n("Style"), m_markerLines, Style);
    m_gpOptions->insertSeparator();
    connect(m_markerLines, SIGNAL(activated(int)), SLOT(handleMarkerLines(int)));
    }
  m_gpOptions->insertItem(i18n("Show Info"), ShowInfo);
  m_infoPos = new QPopupMenu(this);
  m_infoPos->setCheckable(true);
  m_infoPos->insertItem(i18n("Top-Left"), 0);
  m_infoPos->insertItem(i18n("Top-Right"), 1);
  m_gpOptions->insertItem(i18n("Text Position"), m_infoPos, TextPosition);
  m_gpOptions->insertSeparator();
  m_gpOptions->insertItem(i18n("Save Graph..."), SaveGraph);
  connect(m_gpOptions, SIGNAL(activated(int)), SLOT(handlePopupCommand(int)));
  connect(m_gpOptions, SIGNAL(aboutToShow()), SLOT(checkPopupStatus()));
  connect(m_infoPos, SIGNAL(activated(int)), SLOT(handleInfoPos(int)));
  }

// default settings
m_plotMode = Bars;
m_showInfo = true;
m_marker   = true;
m_lines    = true;
m_textPos  = TopLeft;
m_infoText = "";
setMargin(0, 0, 10, 0);
}

/*------------------------------------------------------------------------ */
SetiGraph::~SetiGraph()
{
}

/*------------------------------------------------------------------------ */
void SetiGraph::paintEvent(QPaintEvent*)
{
paintWidget(this);
}

/*------------------------------------------------------------------------ */
void SetiGraph::paintWidget(QPaintDevice* pd)
{
QPainter p;
p.begin(pd);
p.translate(0, rect().bottom());

if(m_yData.size() > 0)
  {
  drawCurve(m_yData, m_plotMode, Qt::red, &p);
  if(m_showInfo) printInfo(m_infoText, &p);
  }
else
  {
  p.setPen(Qt::white);
  p.drawText(0, -height()/2, width(), 16, Qt::AlignCenter,
             i18n("No graph available"));
  }
p.end();
}

/*------------------------------------------------------------------------ */
void SetiGraph::drawCurve(const QMemArray<double>& d,
                          int mode, const QColor& c, QPainter* p)
{
// leave the function if the size of the array is zero
int dsize = d.size();
if(dsize == 0)
  return;

QBrush bru;
bru.setStyle(SolidPattern);
bru.setColor(c);

QPen pen;
pen.setColor(c);

p->setPen(pen);
p->setBrush(bru);

double xfactor = width() - m_leftMargin - m_rightMargin;
double yfactor = height() - m_topMargin - m_bottomMargin;
switch(mode)
  {
  case Bars: // bar chart
    {
    int x = m_leftMargin;
    int y = -(int)(m_bottomMargin + (yfactor*d[0])/m_yRange);
    p->moveTo(x, y);
    x = m_leftMargin + (int)(xfactor/dsize);
    p->lineTo(x, y);
    for(int i=1;i<dsize;i++)
      {
      y = -(int)((yfactor*d[i])/m_yRange);
      p->lineTo(x, y);
      x = m_leftMargin + (int)((i+1)*xfactor/dsize);
      p->lineTo(x, y);
      }
    break;
    }
  case Lines: // scatter chart
  case Markers:
  case MarkersAndLines:
    {
    int x = (int)(m_leftMargin + (0.5*width())/dsize);
    int y = -(int)(m_bottomMargin + (yfactor*d[0])/m_yRange);
    p->moveTo(x, y);
    if(mode == Markers || mode == MarkersAndLines)
      p->drawEllipse(x-3, y-3, 6, 6);
    for(int i=1;i<dsize;i++)
      {
      x = (int)(m_leftMargin + ((i+0.5)*width())/dsize);
      y = -(int)(m_bottomMargin + (yfactor*d[i])/m_yRange);
      if(mode == Lines || mode == MarkersAndLines)
        p->lineTo(x, y);
      else
        p->moveTo(x, y);
      if(mode == Markers || mode == MarkersAndLines)
        p->drawEllipse(x-3, y-3, 6, 6);
      }
    break;
    }
  }
}

/*------------------------------------------------------------------------ */
double SetiGraph::maxValue(const QMemArray<double>& array) const
{
double maxval(-DBL_MAX);

for(unsigned int i=0;i<array.size();i++)
  if(array[i] > maxval)
    maxval = array[i];

return maxval;
}

/*------------------------------------------------------------------------ */
double SetiGraph::minValue(const QMemArray<double>& array) const
{
double minval(DBL_MAX);

for(unsigned int i=0;i<array.size();i++)
  if(array[i] < minval)
    minval = array[i];

return minval;
}

/*------------------------------------------------------------------------ */
void SetiGraph::mousePressEvent(QMouseEvent* e)
{
if(e->button() == RightButton && m_showPopup)
  m_gpOptions->popup(QCursor::pos());
}

/*------------------------------------------------------------------------ */
void SetiGraph::handlePopupCommand(int id)
{
switch(id)
  {
  case BarChart:
    m_plotMode = Bars;
    break;
  case ScatterChart:
    if(m_marker && !m_lines)
      m_plotMode = Markers;
    if(!m_marker && m_lines)
      m_plotMode = Lines;
    if(m_marker && m_lines)
      m_plotMode = MarkersAndLines;
    break;
  case ShowInfo:
    m_showInfo = !m_showInfo;
    break;
  case SaveGraph: // save the Gaussian widget to disk
    slotSaveGraph();
    break;
  }
repaint();
}

/*------------------------------------------------------------------------ */
void SetiGraph::checkPopupStatus()
{
m_gpOptions->setItemChecked(BarChart, false);
m_gpOptions->setItemChecked(ScatterChart, false);
m_gpOptions->setItemChecked(ShowInfo, false);
if(m_plotMode == Bars)
  m_gpOptions->setItemChecked(BarChart, true);
else
  m_gpOptions->setItemChecked(ScatterChart, true);

if(m_plotMode != Bars)
  {
  m_gpOptions->setItemEnabled(Style, true);
  m_markerLines->setItemChecked(0, m_marker);
  m_markerLines->setItemChecked(1, m_lines);
  }
else
  m_gpOptions->setItemEnabled(Style, false);
m_gpOptions->setItemChecked(ShowInfo, m_showInfo);
m_gpOptions->setItemEnabled(TextPosition, m_showInfo);
if(m_showInfo == true)
  {
  m_infoPos->setItemChecked(0, false);
  m_infoPos->setItemChecked(1, false);
  m_infoPos->setItemChecked(m_textPos, true);
  }
}

/*------------------------------------------------------------------------ */
void SetiGraph::handleMarkerLines(int id)
{
switch(id)
  {
  case 0:
    if(m_lines == true)
      m_marker = !m_marker;
    break;
  case 1:
    if(m_marker == true)
      m_lines = !m_lines;
    break;
  }
if(m_marker && !m_lines)
  m_plotMode = Markers;
if(!m_marker && m_lines)
  m_plotMode = Lines;
if(m_marker && m_lines)
  m_plotMode = MarkersAndLines;

repaint();
}

/*------------------------------------------------------------------------ */
void SetiGraph::handleInfoPos(int id)
{
m_textPos = id;
repaint();
}

/*------------------------------------------------------------------------ */
void SetiGraph::setTextPosition(int pos)
{
if(pos > TopRight)
  pos = TopRight;
handleInfoPos(pos);
}

/*------------------------------------------------------------------------ */
void SetiGraph::printInfo(const QString& info, QPainter* p)
{
QFont font = KGlobalSettings::fixedFont();
font.setPointSize(6);
p->setFont(font);

QString wu;
wu = "WU: " + this->name() + "\n";

p->setPen(Qt::red);
switch(m_textPos)
  {
  case 0: // top-left
    {
    p->drawText(0, -height(), width(), 16, AlignLeft|DontClip, wu);
    p->setPen(Qt::white);
    p->drawText(0, -height()+16, width(), height()-16, AlignLeft|DontClip, info);
    break;
    }
  case 1: // top-right
    {
    p->drawText(0, -height(), width(), 16, AlignRight|DontClip,wu);
    p->setPen(Qt::white);
    p->drawText(0, -height()+16, width(), height()-16, AlignRight|DontClip, info);
    break;
    }
  }
}

/*------------------------------------------------------------------------ */
void SetiGraph::setInfoText(const QString& info)
{
if(!info.isNull())
  m_infoText = info;
}

/*------------------------------------------------------------------------ */
void SetiGraph::setCurve(const double* dat, int len)
{
m_yData.duplicate((const double*)dat, len);
m_xMin   = 0.0;
m_xMax   = (double)m_yData.size();
m_xRange = m_xMax - m_xMin;
m_yMin   = 0.0;
m_yMax   = maxValue(m_yData);
m_yRange = m_yMax - m_yMin;
}

/*------------------------------------------------------------------------ */
void SetiGraph::setCurve(const QMemArray<double>& dat)
{
m_yData.duplicate(dat);
m_xMin   = 0.0;
m_xMax   = (double)m_yData.size();
m_xRange = m_xMax - m_xMin;
m_yMin   = 0.0;
m_yMax   = maxValue(m_yData);
m_yRange = m_yMax - m_yMin;
}

/*------------------------------------------------------------------------ */
void SetiGraph::setPlotMode(int mode)
{
if(mode > MarkersAndLines)
  mode = MarkersAndLines;
m_plotMode = mode;
}

/*------------------------------------------------------------------------ */
void SetiGraph::setMargin(int left, int right, int top, int bottom)
{
m_leftMargin   = left;
m_rightMargin  = right;
m_topMargin    = top;
m_bottomMargin = bottom;
}

/*------------------------------------------------------------------------ */
void SetiGraph::setName(const QString& wun)
{
m_wuName = wun;
}

/*------------------------------------------------------------------------ */
QString SetiGraph::name() const
{
return(m_wuName);
}

/*------------------------------------------------------------------------ */
double SetiGraph::maximumValue()
{
return(m_yData.size()>0 ? m_yMax : 0.0);
}

/*------------------------------------------------------------------------ */
void SetiGraph::showInfoText(bool show)
{
m_showInfo = show;
}

/*------------------------------------------------------------------------ */
QPixmap SetiGraph::createPixmap()
{
QPixmap pm(this->size());
pm.fill(Qt::black);
paintWidget(&pm);
return(pm);
}

/*------------------------------------------------------------------------ */
void SetiGraph::slotSaveGraph()
{
QString fn;
QPixmap gg = createPixmap();
if(m_location)
  fn = KFileDialog::getSaveFileName(m_location->directory(), "*.png *.PNG");
else
  fn = KFileDialog::getSaveFileName(QString::null, "*.png *.PNG");
if(!fn.isEmpty())
  {
  if(strcmp((const char*)fn.right(4),".png") &&
     strcmp((const char*)fn.right(4),".PNG"))
    {
    fn.append(".png");
    }
  if(QFile::exists(fn))
    {
    int ret(0); // 1 is equivalent to cancel
    ret = KMessageBox::warningYesNo(this,
                                    i18n("File already exists. Overwrite?"));
    if(ret == KMessageBox::No) return;  // <----- outta here
    }
  if(gg.save(fn, "PNG") == false)
    {
    KMessageBox::error(this, i18n("Could not save the image. Probably\n"
                                  "you don't have write permission."));
    }
  }
}

#include "setigraph.moc"

