/***************************************************************************/
/** @file     setigraph.h
  * @brief    The base widget for all signal graphs (declaration).
  *
  * @b begin      : Thu Oct 19 2000<br>
  * @b copyright  : &copy; 2000-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef SETIGRAPH_H
#define SETIGRAPH_H

#include <qwidget.h>
#include <qmemarray.h>

class KPopupMenu;
class QPopupMenu;
class SetiLoc;

/** The base widget for all signal graphs
  *
  * This is a generic widget class for the display of gaussian, pulse, and
  * triplet graphs. It provides a customizable view of the signal data, and
  * allows to save a picture of the graph to disk. The data is set with the
  * setCurve() methods.
  */
class SetiGraph : public QWidget
{
Q_OBJECT

public:
  /** The constructor.
    *
    * Creates a graph for a signal of location @a loc. If the %S@h client of
    * location @a loc detects a higher signal, the contents of this widget
    * will automatically be updated.
    *
    * @param loc        The %S@h location for which the signal shall be shown.
    *                   If 0, this widget will not be connected to a location,
    *                   and thus won't receive any update events.
    * @param shwPop     If TRUE, the right-mouse-button popup menu with
    *                   configuration options is enabled.
    * @param allowModes If TRUE, menu item for the selection of different
    *                   drawing modes is enabled in the right-click popup
    *                   menu.
    * @param parent     The parent widget. If 0, this widget becomes a
    *                   top-level widget.
    * @param name       A text that can be used to identify this widget.
    */
  SetiGraph(SetiLoc* loc, bool shwPop, bool allowModes, QWidget *parent=0,
            const char *name=0);
  /** The destructor. */
  ~SetiGraph();
  /** Sets the info text.
    *
    * The text @a info will be shown in form of a legend in either the
    * top-left or top-right corner of the widget.
    *
    * @param info The info string.
    */
  void setInfoText(const QString& info);
  /** Sets the data of the graph.
    *
    * @param data A QArray of doubles representing the data of the signal.
    */
  void setCurve(const QMemArray<double>& dat);
  /** Sets the data of the graph.
    *
    * @overload
    *
    * @param data The pointer to an array of doubles representing the data
    *             of the signal.
    * @param len  The length of the array.
    */
  void setCurve(const double* dat, int len);
  /** Sets the position of the legend.
    *
    * @param pos The new position, either @c TopRight or @c TopLeft.
    */
  void setTextPosition(int pos);
  /** Sets the plot mode.
    *
    * @param mode The new mode, either @c Bars, @c Lines, @c Markers,
    *             @c MarkersAndLines, or @c XYPlot ).
    */
  void setPlotMode(int mode);
  void setMargin(int left, int right, int top, int bottom);
  int  leftMargin() {return(m_leftMargin);}
  int  rightMargin() {return(m_rightMargin);}
  int  topMargin() {return(m_topMargin);}
  int  bottomMargin() {return(m_bottomMargin);}
  /** Sets the name of the work unit.
    *
    * The name will be displayed in the legend.
    *
    * @param wun A string containing the name of the work unit.
    */
  void setName(const QString& wun);
  /** Returns the name of the work unit currently displayed. */
  QString name() const;
  /** Returns the maximum value of the data array. */
  double maximumValue();
  double xRange() {return(m_xRange);}
  double yRange() {return(m_yRange);}
  /** Returns the size of the data array. */
  int arraySize() {return(m_yData.size());}
  /** Sets the flag that controls the display of the legend.
    *
    * @param show If TRUE, the legend shall be shown.
    */
  void showInfoText(bool show);
  /** Returns the flag that controls the display of the info text.
    *
    * @return TRUE, if the legend is displayed, otherwise FALSE.
    */
  bool isInfoTextShown() {return m_showInfo;}
  /** Returns a picture of the graph. */
  QPixmap createPixmap();
  /** Identifiers for the different plotting modes.
    *
    * @c Bars is the default way of plotting the data, @c Lines draws
    * a line chart, @c Markers draws a scatter chart with circle markers,
    * and @c MarkersAndLines is a combination of the latter two.
    */
  enum modeIds {Bars, Lines, Markers, MarkersAndLines};
  /** Identifiers specifying the position of the legend in the graph. */
  enum TextPosIds {TopLeft, TopRight};

public slots:
  /** Grabs the contents of the widget and saves it to disk.
    *
    * Calling this slot will open a file dialog for the selection of a file
    * name.
    */
  void slotSaveGraph();

protected: // methods
  /** Reimplemented for customized painting of the widget.
    *
    * @param e Contains event parameters of the paint event.
    */
  virtual void paintEvent(QPaintEvent*);
  /** Paints the widget.
    *
    * The curve is drawn in the specified mode, and optionally a legend with
    * basic info about the data is printed .
    *
    * @param pd A pointer to the object that shall be painted.
    */
  virtual void paintWidget(QPaintDevice* pd);
  /** Draws the curve.
    *
    * @param d    The curve data.
    * @param mode The drawing mode. See SetiGraph::modeIds for a list of
                  allowed modes.
    * @param c    The color of the curve.
    * @param p    A pointer to an open painter object.
    */
  void drawCurve(const QMemArray<double>& d,
                 int mode, const QColor& c, QPainter* p);
  /** An event handler for mouse press events.
    *
    * Required to catch the right-mouse-button click for the context menu.
    *
    * @param e An object with parameters describing the mouse event.
    */
  virtual void mousePressEvent(QMouseEvent* e);
  /** Prints out the info text (or legend).
    *
    * @param info A string containing the text.
    * @param p    A pointer to an open painter object.
    */
  void printInfo(const QString& info, QPainter* p);
  /** Scans a data array for the maximum value.
    *
    * @param array An array with the data.
    *
    * @return The maximum value in the array.
    */
  double maxValue(const QMemArray<double>& array) const;
  /** Scans a data array for the minimum value.
    *
    * @param array An array with the data.
    *
    * @return The minimum value in the array.
    */
  double minValue(const QMemArray<double>& array) const;

protected slots:
  /** Executes the selected command from the right-click popup menu.
    *
    * @param id The unique ID number of the selected popup menu item. See
    *           SetiGraph::popupIds for details.
    */
  void handlePopupCommand(int id);
  /** Prepares the right-click popup menu just before showing it.
    *
    * Checks if the popup menu commands are allowed or make sense in the
    * current situation, and enables or disables the menu items appropriately.
    */
  void checkPopupStatus();
  /** Repaints the widget with the selected position for the legend.
    *
    * @param id The unique ID number of the selected popup menu item. See
    *           SetiGraph::TextPosIds for details.
    */
  void handleInfoPos(int id);
  /** Determines the plot mode and repaints the widget.
    *
    * @param id The unique ID number of the selected popup menu item. A value
    *           of 0 toggles markers on or off, a value of 1 toggles the
    *           drawing of lines on or off.
    */
  void handleMarkerLines(int id);

protected: // attributes
  /** Identifiers for the menu items in the right-click popup menu. */
  enum PopupIds {BarChart, ScatterChart, Style, ShowInfo, TextPosition, SaveGraph};
  SetiLoc*          m_location;
  bool              m_showPopup;
  bool              m_allowModeSel;
  KPopupMenu*       m_gpOptions;
  QPopupMenu*       m_infoPos;
  QPopupMenu*       m_markerLines;
  int               m_plotMode;
  bool              m_showInfo;
  bool              m_marker;
  bool              m_lines;
  int               m_textPos;
  double            m_xMin;
  double            m_xMax;
  double            m_xRange;
  QMemArray<double> m_yData;
  double            m_yMin;
  double            m_yMax;
  double            m_yRange;
  int               m_leftMargin;
  int               m_rightMargin;
  int               m_topMargin;
  int               m_bottomMargin;
  QString           m_infoText;
  QString           m_wuName;
};

#endif
