/***************************************************************************/
/** @file     setiloc.h
  * @brief    Represents a %SETI@home location (declaration).
  *
  * @b begin      : Wed Oct 27 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef SETILOC_H
#define SETILOC_H

#include <qstring.h>
#include <qcolor.h>

#include "seticontainer.h"

class SetiGraph;
class KTextBrowser;

struct LocationSet
  {
  QString description;        /**< A descriptive name for the location. */
  QString directory;          /**< The working directory of the %S@h
                                   location. */
  bool    useCustomStartExe;  /**< If TRUE, the executable specified in
                                   LocSettings::customStartExe is used to
                                   start the %S@h client. */
  QString customStartExe;     /**< Path and file name of a user-defined
                                   executable to start the %S@h client.
                                   Only used if LocSettings::useCustomStartExe
                                   is TRUE, otherwise the standard %S@h client
                                   in LocSettings::directory is used. */
  bool    useCustomStopExe;   /**< If TRUE, the executable specified in
                                   LocSettings::customStopExe is used to stop
                                   the %S@h client. */
  QString customStopExe;      /**< Path and file name of a user-defined
                                   executable to stop the %S@h client. Only
                                   used if LocSettings::useCustomStopExe is
                                   TRUE, otherwise the %S@h client will be
                                   stopped by sending a SIGTERM signal. */
  QColor  color;              /**< The color used for the markers in the
                                   skymap. */
  };

struct ClientSet
  {
  QString arguments;          /**< A string with the command-line arguments
                                   passed to the %S@h client. */
  bool    startSeti;          /**< If TRUE, the %S@h client is started upon
                                   start-up of Ksetiwatch. */
  bool    stopSeti;           /**< If TRUE, the %S@h client is stopped upon
                                   shut-down of Ksetiwatch. */
  bool    keepClientAlive;    /**< If TRUE, Ksetiwatch  tries to restart the
                                   %S@h client whenever it stops running
                                   without user intervention. */
  bool    stopAfterProcess;   /**< If TRUE, the client is stopped after
                                   processing a WU. */
  bool    stopAfterSend;      /**< If TRUE, the client is stopped after sending
                                   the results of the last WU. */
  bool    email;              /**< If TRUE, an email is sent to the user in
                                   case of an error. */
  bool    graphics;           /**< If TRUE, the client produces a data stream
                                   for the @c xsetiathome graphics program. */
  int     priority;           /**< The priority of the %S@h client. Ranges
                                   from 1 to 19. Higher values indicate a
                                   higher priority. A value of 1 corresponds to
                                   a nice level of 19. */
  bool    useProxy;           /**< If TRUE, a proxy server is used to connect
                                   to the %S@h server. */
  QString proxyServer;        /**< The proxy server via which to connect to
                                   the %S@h server. */
  int     proxyPort;          /**< The port of the proxy server. */
  };

struct LoggingSet
  {
  bool    log;                /**< If TRUE, the results of a completed WU are
                                   logged. */
  bool    redirectLog;        /**< If TRUE, the results of a completed WU are
                                   redirected to the directory specified in
                                   LocSettings::logDirectory. If FALSE, the
                                   results are stored in the working directory
                                   of the client. */
  QString logDirectory;       /**< The directory where the results of the
                                   completed WUs are stored. Only used if
                                   LocSettings::log is TRUE. */
  };

/** Settings and options of a %S@h location. */
struct LocSettings
  {
  LocationSet location;
  ClientSet   client;
  LoggingSet  logging;
  };

/** Represents a %SETI@home location.
  *
  * This is a central class in the Ksetiwatch application. For every %S@h
  * location the user wishes to monitor, an object based on this class is
  * created. It extends the features of the SetiContainer class (which
  * basically does the monitoring of the sah files and data structures of
  * the client) by the ability to start and stop the %S@h client, and to
  * log the results of completed work units.
  *
  * Upon creation of a SetiLoc object, a unique identifier string is
  * generated. This is because %S@h locations can in principle have the
  * same name, but different working directories. Use the ID string
  * (obtained via id()) to distinguish between the different locations.
  */
class SetiLoc : public SetiContainer
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param dir The working directory of the %SETI@home client.
    * @param loc A descriptive name of the %S@h location.
    * @param col The color which is used for the WU markers on the skymap.
    */
  SetiLoc(const QString& dir = QString::null,
          const QString& loc = QString::null,
          const QColor& col = QColor::QColor(Qt::red));
  /** The destructor. */
  ~SetiLoc();
  /** Returns the unique ID string.
    *
    * Upon creation of a SetiLoc object, a unique identifier string is
    * generated. This ID string can be used to safely identify the correct
    * location.
    */
  const QString& id() const;
  /** Sets all options of the location.
    *
    * @param ls A structure containing the new settings.
    */
  void setOptions(const LocSettings& ls);
  /** Returns the settings of the location. */
  LocSettings options() const {return(sets);}
  /** Returns the descriptive name of the location. */
  QString description() const {return sets.location.description;}
  /** Returns the color used for the location. */
  QColor color() const {return sets.location.color;}
  /** Returns TRUE, if the %S@h client is started at start-up of
    * Ksetiwatch. */
  bool optionStartSeti() const {return sets.client.startSeti;}
  /** Returns TRUE, if the %S@h client is stopped at shut-down of
    * Ksetiwatch. */
  bool optionStopSeti() const {return sets.client.stopSeti;}
  /** Returns TRUE, if the %S@h client shall be restarted whenevever it
    * shuts down unexpectedly. */
  bool optionKeepClientAlive() const {return sets.client.keepClientAlive;}
  /** Sets the time stamp of the last WU logged by Ksetiwatch.
    *
    * The time stamp is used to prevent multiple logging of the same WU.
    *
    *  @param ts A string containing the time stamp.
    */
  void setTimestamp(const QString& ts) {sresult_timestamp = ts;}
  /** Returns the time stamp of the last WU logged by Ksetiwatch. */
  QString timestamp() const {return sresult_timestamp;}
  /** Returns the path to the log directory. */
  QString logDirectory();
  /** Identifiers for the three different graph widgets. */
  enum GraphIds {GaussianGraph, PulseGraph, TripletGraph, ClientOutput};
  /** Displays a window with one of the three different signal graphs.
    *
    * @param id  The identifier of the graph. See SetiLoc::GraphIds.
    */
  void showGraph(int id);
  /** Returns the state of the graph widget specified by identifier @a id.
    *
    * @param id The identifier of the graph. See SetiLoc::GraphIds.
    */
  bool isGraphVisible(int id) const {return(graphwidget[id] != 0);}
  /** Starts the %SETI@home client.
    *
    * @return TRUE if started successfully, otherwise FALSE.
    */
  bool startClient();
  /** Stops the %SETI@home client.
    *
    * @return TRUE if stopped successfully, otherwise FALSE.
    */
  bool stopClient();
  /** TRUE if the client has been stopped upon user request. */
  bool clientStoppedByRequest() {return(stoppedByRequest);}
  /** Initializes the structure that stores the score of the best signals of
    * the WU in progress. */
  void initMaxScore();
  /** Sets the structure containing the best signals to @a score.
    *
    * @param score A WUScore structure with the new signal parameters.
    */
  void setMaxScore(const WUScore& score) {mymax = score;}
  /** Returns a structure with the parameters of the best signals found in
    * the WU in progress. */
  const WUScore& maxScore() const {return(mymax);}
  /** Writes triplet data (power, period, and profile) to a temporary file.
    * This information will be merged into the result log later on.
    *
    * @param triplet A pointer to the triplet score structure.
    */
  void storeTripletData(const TripletScore& triplet);
  /** Checks if the client has completed a work unit.
    *
    * If the client has created the file @c result.sah, and
    * @c result_header.sah and @c work_unit.sah are missing, the WU will
    * be logged.
    */
  void checkWUStatus();
  /** Returns the parameters of a logged spike signal.
    *
    * The %S@h client stores the parameters of signals which are to be
    * returned to the server in a single line of the form
    * @c "spike: power=...". This method parses such a line and extracts the
    * parameters and optionally the profile of the signal
    *
    * This method can be applied to lines in the files @c outfile.sah and
    * @c result.sah (created by the %S@h client), and @c SETIResult.log
    * (created by Ksetiwatch).
    *
    * @param line The line containing the parameters of the spike.
    *
    * @return A SpikeScore structure with the parameters extracted from
    *         @a line.
    */
  static SpikeScore loggedSpikeSignal(QString& line);
  /** Returns the parameters of a logged gaussian signal.
    *
    * See loggedSpikeSignal() for details.
    *
    * @param line The line containing the parameters of the gaussian.
    *
    * @return A GaussianScore structure with the parameters extracted from
    *         @a line.
    */
  static GaussianScore loggedGaussianSignal(QString& line);
  /** Returns the parameters of a logged pulse signal.
    *
    * See loggedSpikeSignal() for details.
    *
    * @param line The line containing the parameters of the pulse.
    *
    * @return A PulseScore structure with the parameters extracted from
    *         @a line.
    */
  static PulseScore loggedPulseSignal(QString& line);
  /** Returns the parameters of a logged triplet signal.
    *
    * See loggedSpikeSignal() for details.
    *
    * @param line The line containing the parameters of the triplet.
    *
    * @return A TripletScore structure with the parameters extracted from
    *         @a line.
    */
  static TripletScore loggedTripletSignal(QString& line);

protected: //methods
  /** Logs all important results of a processed WU.
    *
    * The results are written to the file @c SETILog.csv. This file is
    * compatible to the csv file generated by SETIWatch.
    */
  void logWorkUnit();
  /** Creates a string that can be appended to a line of the log file
    * @c SETILog.csv.
    *
    * This is a helper method for logWorkUnit(). It takes a string obtained
    * from a sah file by readEntry(), and prepares it in a way that it can
    * be added to the log file.
    *
    * @param entry A string obtained by reading an item from a sah file.
    * @param type  Either 0 for a numerical value, or 1 for a string.
    *
    * @return A string containing an element prepared for the csv file.
    */
  QString constructEntry(const QString& entry, int type);
  /** Logs the signals found in the result file @c result.sah.
    *
    * The information about the signals ( a one-line string) is copied
    * verbatim to the file @c SETIResult.log. The actual numerical parameters
    * can be parsed with loggedSpikeSignal(), loggedGaussianSignal(),
    * loggedPulseSignal(), and loggedTripletSignal().
    */
  void logResults();
  /** Takes the given string with triplet data and compares it with the
    * triplets stored in @c .triplets.dat. If a match is found a string
    * with data describing the triplet profile is returned. The motivation
    * behind this is to provide a possibility to log triplet graph data
    * (which the client itself doesn't offer).
    *
    * @param tdata A string with triplet information as is provided by
    *              the @c result.sah file.
    * @return A string with additional information about the triplet profile
    *         (graph data), or an empty string if no match could be found.
    */
  QString loggedTripletProfile(const QString& tdata);
  /** Determines the index value (i.e the position) of the fitted gaussian.
    *
    * This is done by calculating the least-square sum over the data set.
    * We  need this method because the client doesn't store the index value
    * of logged gaussians (in @c outfile.sah or @c result.sah).
    *
    * @param g A pointer to the GaussianScore structure.
    *
    * @return The index value.
    */
  static int calcGaussianFFTIndex(GaussianScore* g);

protected slots:
  /** Initializes the processing of a new WU.
    *
    * In addition to resetting the structure with the best signal scores, any
    * open graph window is hidden, the hidden temporary file @c .triplets.dat
    * is deleted, and a signal is sent to the skymap window to update its list
    * of displayed locations.
    *
    * This slot should only be called when starting the processing of a new
    * (downloaded) WU.
    */
  void slotInitMaxScore(WorkUnitData wud);
  /** No descriptions */
  void slotDumpClientOutput(KProcess*, char*, int);

protected:
  /** The unique id string of the location. */
  QString        thisId;
  /** The settings and options of the location */
  LocSettings    sets;
  /** The time stamp of the result file. Required to prevent multiple
    * logging. */
  QString        sresult_timestamp;
  /** Stores the best signal scores found for the WU in progress. */
  WUScore        mymax;
  SetiGraph*     graphwidget[3];
  /** This widget catches the output of the %S@h client. */
  KTextBrowser*  clientOutput;
  /** Only TRUE if the client has been stopped via the menu action in the
    * popup menu. */
  bool stoppedByRequest;

signals:
  /** This signal is emitted whenever a new WU has been started. Arguments
    * are a structure with the new work unit data, the name of the
    * corresponding %S@h location, and the color used for the location.
    *
    * This signal is equivalent to the newWorkUnit() signal of the
    * SetiContainer class, only with extended number of arguments.
    */
  void newWorkUnit(WorkUnitData wud, const QString& locname,
                   const QColor& col);
  /** This signal is emitted whenever a work unit has been logged. The
    * argument is a pointer to the SetiLoc object.
    */
  void workUnitLogged(SetiLoc* loc);

};

#endif
