/***************************************************************************/
/** @file     skymap.cpp
  * @brief    The skymap window (implementation).
  *
  * @b begin      : Fri Nov 12 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * The skymap window is actually composed of two parts: a widget displaying
  * the skmap picture and a scroll view object containing the aforementioned
  * widget as a child. This guarantees efficient and flicker-free scrolling.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <math.h>

#include <kapp.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kiconloader.h>
#include <kdialog.h>

#include <qglobal.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qcursor.h>
#include <qtooltip.h>
#include <qsignalmapper.h>
#include <qstringlist.h>
#include <qmemarray.h>
#include <qlayout.h>

#include "ksetiwatch.h"
#include "skymap.h"
#include "datainfo.h"
#include "setiloc.h"
#include "telescopepath.h"

////////////////////////////////////////////////////////////////////
// LocButton methods
LocButton::LocButton(QWidget* parent, const char* name)
         : QButton(parent, name)
{
}

LocButton::~LocButton()
{
}

/*------------------------------------------------------------------------ */
void LocButton::setColor(const QColor& col)
{
m_color = col;
update();
}

/*------------------------------------------------------------------------ */
void LocButton::paintEvent(QPaintEvent*)
{
QPainter p;
QBrush bru;
QPen pen;

p.begin( this );
pen.setColor(m_color);
bru.setColor(m_color);
if(globalopts->FillMarker)
  bru.setStyle(SolidPattern);
else
  bru.setStyle(NoBrush);
p.setPen(pen);
p.setBrush(bru);
p.drawEllipse(rect());
p.end();
}

////////////////////////////////////////////////////////////////////
// SkyMap methods
SkyMap* SkyMap::map = 0;

SkyMap::SkyMap(QWidget* parent, const char*)
       : QScrollView(parent), pathDlg(0), path(0)
{
SeLi.setAutoDelete(true);

// create a signal mapper for the different buttons
buttonSignals = new QSignalMapper(this);
connect(buttonSignals, SIGNAL(mapped(const QString&)),
                 this, SLOT(showTelescopePath(const QString&)));

// load the skymap
starmap = UserIcon("starmap");
QSize s(starmap.size());

setCaption(i18n("Skymap (right click for menu)"));
setMinimumSize(100, 100);

pic_options = new KPopupMenu(this, "Skymap");
pic_options->insertTitle(i18n("Skymap"));
pic_options->setCheckable(true);
pic_options->insertItem(i18n("Clear"), Clear);
pic_options->insertItem(i18n("Show Text"), ShowText);
pic_options->insertSeparator();
pic_options->insertItem(i18n("Small Marker"), SmallMarker);
pic_options->insertItem(i18n("Normal Marker"), NormalMarker);
pic_options->insertItem(i18n("Large Marker"), LargeMarker);
pic_options->insertSeparator();
pic_options->insertItem(i18n("Fill Marker"), FillMarker);
connect(pic_options, SIGNAL(activated(int)), SLOT(handlePopupCommand(int)));
connect(pic_options, SIGNAL(aboutToShow()), SLOT(checkPopupStatus()));

pic = new QLabel(this);
pic->installEventFilter(this);
pic->setFixedSize(s);
pic->setBackgroundPixmap(starmap);
this->addChild(pic);

QWidget* d = QApplication::desktop();
int w = d->width();                   // returns screen width
int h = d->height();                  // returns screen height
int w_sm(0), h_sm(0);                 // width and height of starmap window
// +4 had to be added to avoid scroll bars in max mode (Qt 2.2)
if(s.width() < w)
  w_sm = s.width()+4;
else
  {
  w_sm = w - 8;
  h_sm = 16;
  }
if(s.height() < h)
  h_sm += s.height()+4;
else
  h_sm = h;	
resize(w_sm, h_sm);
setMaximumSize(w_sm, h_sm);
}

/*------------------------------------------------------------------------ */
SkyMap::~SkyMap()
{
clearMap();
}

/*------------------------------------------------------------------------ */
bool SkyMap::event(QEvent* e)
{
switch(e->type())
  {
  case QEvent::MouseButtonPress:
    {
    QMouseEvent* me = dynamic_cast<QMouseEvent*>(e);
    if(me && me->button() == RightButton)
      pic_options->popup(QCursor::pos());
    break;
    }
  case QEvent::Close:
    {
    if(pathDlg != 0)
      pathDlg->close();
    break;
    }
  case QEvent::Hide:
    {
    if(pathDlg != 0)
      {
      isPathDlgHidden = pathDlg->isHidden();
      pathDlg->hide();
      }
    break;
    }
  case QEvent::Show:
    {
    if(pathDlg != 0 && isPathDlgHidden == false)
      {
      pathDlg->show();
      }
    break;
    }
  default:
    break;
  }

return(QScrollView::event(e));
}

/*------------------------------------------------------------------------ */
bool SkyMap::eventFilter(QObject* o, QEvent* e)
{
if(e->type() == QEvent::Paint)
  {
  QLabel* pic     = dynamic_cast<QLabel*>(o);
  QPaintEvent* pe = dynamic_cast<QPaintEvent*>(e);
  if(pic && pe)
    paintLocations(pe);
  return(true);
  }
return(QScrollView::eventFilter(o, e));
}

/*------------------------------------------------------------------------ */
void SkyMap::paintLocations(QPaintEvent*)
{
QPainter p;
QBrush bru;
QPen pen;
QString id;
int ms = globalopts->MarkerSize;

p.begin( pic );
for(SkyPos* pos=SeLi.first(); pos != 0; pos=SeLi.next() )
  {
  QPoint lp = locationPosition(pos->ra, pos->dec);
  pen.setColor(pos->color);
  p.setPen(pen);
  if(pos->pathButton == 0)
    {
    bru.setColor(pos->color);
    if(globalopts->FillMarker)
      bru.setStyle(SolidPattern);
    else
      bru.setStyle(NoBrush);
    p.setBrush(bru);
    p.drawEllipse(lp.x()-ms*3, lp.y()-ms*3, ms*6, ms*6);
    }
  if(globalopts->ShowText)
    {
    id = pos->location + "\n" + pos->time_recorded;
    QRect r = p.boundingRect(0, 0, 10000, 10000, 0, (const char *)id);
    p.drawText(lp.x()-(r.width()/2), lp.y()-r.height()-ms*3-1,
               r.width(), r.height(), AlignCenter|DontClip, id);
    }
  }
p.end();
}

/*------------------------------------------------------------------------ */
SkyMap* SkyMap::instance()
{
if(map == 0)
  map = new SkyMap();
return(map);
}

/*------------------------------------------------------------------------ */
SkyMap* SkyMap::showMap()
{
SkyMap* m = instance();
if(m)
  {
  m->show();
  if(m->isActiveWindow() == false)
    {
    if(m->isMinimized())
      {
      m->hide();
      m->showNormal();
      }
    m->raise();
    }
  }
return(m);
}

/*------------------------------------------------------------------------ */
void SkyMap::addLocation(SetiLoc* loc)
{
if(!loc->timeRecorded().isEmpty())
  {
  SkyPos *spos = new SkyPos;

  // fill the SkyPos structure
  spos->location = loc->description();
  spos->time_recorded = loc->timeRecordedString();
  spos->color = loc->color();
  spos->ra = loc->startRA();
  spos->dec = loc->startDec();

  setLocationButton(spos);
  buttonSignals->setMapping(spos->pathButton, spos->location);
  connect(spos->pathButton, SIGNAL(clicked()), buttonSignals, SLOT(map()));

  // and add it to the list
  SeLi.append(spos);
  pic->update();
  }

connect(loc, SIGNAL(newWorkUnit(WorkUnitData, const QString&, const QColor&)),
     this, SLOT(updateLocation(WorkUnitData, const QString&, const QColor&)));
}

/*------------------------------------------------------------------------ */
void SkyMap::addLocation(const QString& locName, const QString& time_rec,
                         const QColor& col, double ra, double dec)
{
SkyPos *spos = new SkyPos;

// fill the SkyPos structure
spos->location      = locName;
spos->time_recorded = time_rec;
spos->color         = col;
spos->ra            = ra;
spos->dec           = dec;
spos->pathButton    = 0;

// and add it to the list
SeLi.append(spos);
pic->update();
}

/*------------------------------------------------------------------------ */
void SkyMap::clearMap()
{
for(SkyPos* pos=SeLi.first(); pos != 0; pos=SeLi.next() )
  {
  if(pos->pathButton)
    delete pos->pathButton;
  }
SeLi.clear();
pic->update();
}

/*------------------------------------------------------------------------ */
void SkyMap::updateLocation(WorkUnitData wud, const QString& locname,
                            const QColor& col)
{
for(SkyPos* pos=SeLi.first(); pos != 0; pos=SeLi.next() )
  {
  if(pos->location == locname)
    {
    pos->time_recorded = wud.time_recorded;
    pos->color = col;
    pos->ra = wud.start_ra;
    pos->dec = wud.start_dec;
    pic->update();
    break;
    }
  }
}

/*------------------------------------------------------------------------ */
void SkyMap::setLocationButton(SkyPos* pos)
{
int    ms = globalopts->MarkerSize;
QPoint lp = locationPosition(pos->ra, pos->dec);

pos->pathButton = new LocButton(pic);
pos->pathButton->setGeometry(lp.x()-ms*3, lp.y()-ms*3, ms*6, ms*6);
pos->pathButton->setColor(pos->color);
pos->pathButton->setBackgroundPixmap(starmap);
pos->pathButton->setBackgroundOrigin(ParentOrigin);
#if QT_VERSION < 0x030000
    pos->pathButton->setCursor(Qt::pointingHandCursor);
#else
    pos->pathButton->setCursor(Qt::PointingHandCursor);
#endif
pos->pathButton->show();
QToolTip::remove(pos->pathButton);
QToolTip::add(pos->pathButton, pos->location + "\n" + pos->time_recorded);
}

/*------------------------------------------------------------------------ */
void SkyMap::checkPopupStatus()
{
pic_options->setItemChecked(ShowText, globalopts->ShowText);
pic_options->setItemChecked(FillMarker, globalopts->FillMarker);
for(int i=SmallMarker;i<=LargeMarker;i++)
  {
  pic_options->setItemChecked(i, false);
  }
pic_options->setItemChecked(globalopts->MarkerSize+SmallMarker-1, true);
}

/*------------------------------------------------------------------------ */
void SkyMap::handlePopupCommand(int id)
{
switch(id)
  {
  case Clear: // clear the skymap
    clearMap();
    break;
  case ShowText: // handle the text display
    globalopts->ShowText = !globalopts->ShowText;
    break;
  case FillMarker: // fill marker?
    globalopts->FillMarker = !globalopts->FillMarker;
    break;
  default: // marker size
    if(id >= SmallMarker && id <= LargeMarker)
      globalopts->MarkerSize = id - SmallMarker + 1;
    break;
  }

int ms = globalopts->MarkerSize;
for(SkyPos* pos=SeLi.first(); pos != 0; pos=SeLi.next() )
  {
  if(pos->pathButton)
    {
    QPoint lp = locationPosition(pos->ra, pos->dec);
    pos->pathButton->setGeometry(lp.x()-ms*3, lp.y()-ms*3, ms*6, ms*6);
    pos->pathButton->update();
    }
  }
  
pic->update();
}

/*------------------------------------------------------------------------ */
QPoint SkyMap::locationPosition(double ra, double dec)
{
int x = (int)(484.0 - (fmod(ra, 24.0)*965.0)/24.0);
if(x < 0)
  x = 965 + x;
int y = (int)(241.0 - (dec*482.0)/180.0);

return(QPoint(x, y));
}

/*------------------------------------------------------------------------ */
void SkyMap::showTelescopePath(const QString& name)
{
SetiLoc* loc = Ksetiwatch::getLocation(name);
if(loc)
  {
  if(pathDlg == 0)
    {
    pathDlg = new KDialog(this);

    QBoxLayout *layout = new QVBoxLayout(pathDlg);

    if(path == 0)
      path = new TelescopePath((SetiLoc*)0, pathDlg);
    layout->addWidget(path);
    }

  path->setLocation(loc);

  pathDlg->setCaption(i18n("Location %1: Telescope Path").arg(name));
  pathDlg->show();
  pathDlg->move(QCursor::pos());
  double aspect = 2.0*(path->xRange()/24.0)*(180.0/path->yRange());
  //qDebug("x: %f y: %f a: %f", path->xRange(), path->yRange(), aspect);
  if(aspect > 2.0)
    aspect = 2.0;
  if(aspect < 0.5)
    aspect = 0.5;
  if(aspect > 1.0)
    pathDlg->resize(static_cast<int>(150.0*aspect), 150);
  else
    pathDlg->resize(150, static_cast<int>(150.0/aspect));
  }
}

#include "skymap.moc"

