/***************************************************************************/
/** @file     skymap.h
  * @brief    The skymap window  (declaration).
  *
  * @b begin      : Fri Nov 12 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * The skymap window is actually composed of two parts: a widget displaying
  * the skmap picture and a scroll view object containing the aforementioned
  * widget as a child. This guarantees efficient and flicker-free scrolling.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef SKYMAP_H
#define SKYMAP_H

#include <qwidget.h>
#include <qscrollview.h>
#include <qptrlist.h>
#include <qbutton.h>
#include <qpixmap.h>
#include <qpoint.h>
#include <qcolor.h>

#include "setidefs.h"

class KPopupMenu;
class KDialog;
class QLabel;
class QSignalMapper;
class LocButton;
class SetiLoc;
class TelescopePath;

/** A structure containing details about the work unit shown on the skymap. */
struct SkyPos {
    QString location;      /**< The name of the location the WU belongs to.*/
    QString time_recorded; /**< The time and date the WU was recorded. */
    QColor  color;         /**< The color used for the marker. */
    double  ra;            /**< The position of the WU on the skymap (right
                                ascension). */
    double  dec;           /**< The position of the WU on the skymap
                                (declination). */
    LocButton* pathButton; /**< A pointer to a location button which is shown
                                on the skymap when there is telescope path data
                                associated with the work unit. */
    };


/** A clickable button as location marker.
  *
  * This button is used to mark the position of a WU on the skymap whenever
  * there is telescope path data associated with the work unit, i.e. for all
  * WUs which are actively being crunched.
  */
class LocButton : public QButton
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  LocButton(QWidget* parent, const char* name=0);
  /** The destructor. */
  ~LocButton();
  /** Sets the color of the button.
    *
    * @param col The color.
    */
  void setColor(const QColor& col);
  /** Returns the color of the button. */
  QColor color() {return(m_color);}
  
protected: //methods
  /** Reimplemented for customized painting of the widget.
    *
    * The current implementation draws a circle in different sizes (optionally,
    * filled).
    *
    * @param e Contains event parameters of the paint event.
    */
  void paintEvent(QPaintEvent *e);

protected: // attributes
  /** Stores the color value. */
  QColor m_color;
};

/** The skymap scroll view.
  *
  * The scroll view contains a widget displaying the skymap. By
  * default, the scroll view window tries to open itself maximized. If the
  * desktop is smaller than the size of the skymap, the window size is
  * adjusted appropriately.
  *
  * The SkyMap class is implemented as a singleton, which means that only one
  * instance of this class can be created during the run-time of the
  * application.
  */
class SkyMap : public QScrollView
{
Q_OBJECT

public:
  /** Returns a pointer to the SkyMap object.
    *
    * If the object doesn't exist, it is created.
    */
  static SkyMap* instance();
  /** Opens the skymap window.
    *
    * If the window is minimized or hidden, it is raised and brought to the
    * foreground.
    *
    * @return A pointer to the SkyMap object.
    */
  static SkyMap* showMap();
  /** Adds a location to the skymap.
    *
    * This methods adds the WU currently processed by %S@h location @a loc to
    * the list of displayed WU sky position.
    *
    * @param loc A pointer to the SetiLoc object.
    */
  void addLocation(SetiLoc* loc);
  /** Adds a location to the skymap.
    *
    * @overload
    *
    * @param locName  The name of the location.
    * @param time_rec The time and date the WU was recorded.
    * @param col      The color used for the marker.
    * @param ra       The position of the WU on the skymap (right ascension).
    * @param dec      The position of the WU on the skymap (declination).
    */
  void addLocation(const QString& locName, const QString& time_rec,
                   const QColor& col, double ra, double dec);
  /** Clears the list of added locations and redraws the skymap. */
  void clearMap();

protected: // methods
  /** The destructor. */
  virtual ~SkyMap();
  /** Reimplemented for customized painting of the widget.
    *
    * @param e Contains event parameters of the paint event.
    */
  void paintLocations(QPaintEvent *e);
  /** An event filter for child widget(s).
    *
    * Implemented to catch paint events for the child widget on which the WU
    * position markers are painted.
    *
    * @param o The object to which an event is addressed.
    * @param e The event for the object.
    *
    * @return TRUE if the event should be stopped, or FALSE if the event
    *         should be dispatched normally.
    */
  bool eventFilter(QObject* o, QEvent* e);
  /** An event handler for various types of events in the skymap window.
    *
    * Required to catch the right-mouse-button click for the context menu, and
    * for hide/show events.
    *
    * @param e An object with parameters describing the event.
    */
  bool event(QEvent *e);
  /** Creates and initializes a location button for a given WU.
    *
    * @param pos A pointer to a SkyPos structure containing the sky position
    *            and additional information about the WU.
    */
  void setLocationButton(SkyPos* pos);
  /** Converts the RA/Dec position into proper coordinates for the skymap. */
  QPoint locationPosition(double ra, double dec);

protected slots:
  /** Updates a location with new WU data.
    *
    * @param wud     A structure containing the new WU data.
    * @param locname The descriptive name of the %S@h location.
    * @param col     The color used for the markers.
    */
  void updateLocation(WorkUnitData wud, const QString& locname,
                      const QColor& col);

protected: // attributes
  /** Identifiers for the menu items in the right-click popup menu. */
  enum popupIds {Clear, ShowText, SmallMarker, NormalMarker, LargeMarker,
                 FillMarker};
  KPopupMenu*    pic_options;
  /** This list contains the sky position and additional information of all
    * WUs on the skymap. */
  QPtrList<SkyPos> SeLi;
  /** A picture of the skymap. */
  QPixmap       starmap;
  /** A pointer to the widget displaying the skymap. */
  QLabel*       pic;
  QSignalMapper* buttonSignals;

protected slots:
  /** Prepares the right-click popup menu just before showing it.
    *
    * Checks if the popup menu commands are allowed or make sense in the
    * current situation, and enables or disables the menu items appropriately.
    */
  void checkPopupStatus();
  /** Executes the selected command from the right-click popup menu of the
    * Analysis list.
    *
    * @param id The unique ID number of the selected popup menu item. See
    *           SkyMap::popupIds for details.
    */
  void handlePopupCommand(int id);
  /** Opens a window showing the telescope path.
    *
    * @param name A string containing the name of the location for which the
    *             telescope path is to be shown.
    */
  void showTelescopePath(const QString& name);

private: //methods
  /** The constructor.
    *
    * Implemented as a private method because this class is implemented as
.   * a singleton
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  SkyMap(QWidget* parent=0, const char* name=0);

private: // attributes
  /** A pointer to the one and only instance of this class. */
  static SkyMap* map;
  KDialog*       pathDlg;
  bool           isPathDlgHidden;
  TelescopePath* path;
};

#endif
