/***************************************************************************/
/** @file     telescopepath.cpp
  * @brief    A which shows the telescope path for a particular WU
  *           (implementation).
  *
  * @b begin      : Thu Dec 12 2002<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#include <math.h>

#include <klocale.h>

#include <qpainter.h>
#include <qpixmap.h>
#include <qbitmap.h>

#include "telescopepath.h"
#include "setiloc.h"

/*------------------------------------------------------------------------ */
TelescopePath::TelescopePath(SetiLoc* loc, QWidget *parent, const char *name)
             : SetiGraph(loc, true, false, parent, name)
{
if(loc)
  init(loc->color());
else
  init(Qt::red);
}

/*------------------------------------------------------------------------ */
TelescopePath::TelescopePath(SkyPos* pos, QWidget *parent, const char *name)
             : SetiGraph(0, true, false, parent, name)
{
if(pos)
  init(pos->color);
else
  init(Qt::red);
}

/*------------------------------------------------------------------------ */
TelescopePath::~TelescopePath()
{
}

/*------------------------------------------------------------------------ */
void TelescopePath::init(const QColor& col)
{
setBackgroundColor(Qt::black);
locColor = col;

resize(250, 150);
setMinimumSize(100, 100);
setMargin(6, 6, 50, 6);
}

/*------------------------------------------------------------------------ */
void TelescopePath::paintWidget(QPaintDevice* pd)
{
QPainter p;
p.begin(pd);
p.translate(0, rect().bottom());

if(m_yData.size() > 0 && xdata.size() == m_yData.size())
  {
  drawCurve(xdata, m_yData, locColor, &p);
  if(m_showInfo)
    printInfo(m_infoText, &p);
  }
p.end();
}

/*------------------------------------------------------------------------ */
void TelescopePath::drawCurve(const QMemArray<double>& xd,
                              const QMemArray<double>& yd,
                              const QColor& c, QPainter* p)
{
if(xd.size() == 0)
  return;
  
QBrush bru;
bru.setStyle(SolidPattern);
bru.setColor(c);

QPen pen;
pen.setColor(c);
pen.setWidth(3);

p->setPen(pen);
p->setBrush(bru);

double xfactor = width() - leftMargin() - rightMargin();
double yfactor = height() - topMargin() - bottomMargin();
for(unsigned k=0; k<2; k++)
  {
  int xnext;
  int ynext;
  int x = (int)(width() - leftMargin() - xfactor*(xd[0]-m_xMin)/xRange());
  int y = -(int)(bottomMargin() + yfactor*(yd[0]-m_yMin)/yRange());
  p->moveTo(x, y);
  for(unsigned i=1; i<xd.size(); i++)
    {
    xnext = (int)(width() - leftMargin() - xfactor*(xd[i]-m_xMin)/xRange());
    ynext = -(int)(bottomMargin() + yfactor*(yd[i]-m_yMin)/yRange());
    if(k == 0)
      p->lineTo(xnext, ynext);
    else
      {
      QPixmap m = paintMarker(xnext, ynext, x, y, c);
      p->drawPixmap(x-m.width()/2, y-m.height()/2, m);
      }
    x = xnext;
    y = ynext;
    }
  xnext = (int)(width() - leftMargin() - xfactor*(xd[xd.size()-2]-m_xMin)/xRange());
  ynext = -(int)(bottomMargin() + yfactor*(yd[xd.size()-2]-m_yMin)/yRange());
  QPixmap m = paintMarker(x, y, xnext, ynext, c);
  p->drawPixmap(x-m.width()/2, y-m.height()/2, m);
  }
}

/*------------------------------------------------------------------------ */
QPixmap TelescopePath::paintMarker(int x1, int y1, int x2, int y2,
                                   const QColor& c)
{
QPixmap marker(9, 9);
QPainter p;

// paint the marker
p.begin(&marker);

QBrush bru;
bru.setStyle(SolidPattern);
bru.setColor(c.light());

QPen pen;
pen.setColor(c.light());

p.setPen(pen);
p.setBrush(bru);

p.fillRect(0, 0, 9, 9, Qt::black);

p.translate(4, 4);
double dx = x1 - x2;
double dy = y1 - y2;
double phi = asin(dy/sqrt(dx*dx + dy*dy))*180.0/M_PI;
if(dx < 0.0)
  phi = 180.0 - phi;
p.rotate(phi);

QPointArray a(4);
a.setPoint(0, -4, -4);
a.setPoint(1, -2, 0);
a.setPoint(2, -4, 4);
a.setPoint(3, 4, 0);
p.drawPolygon(a);

pen.setColor(c.dark());
p.setPen(pen);
p.moveTo(-4, -3);
p.lineTo(-2, 0);
p.lineTo(-4, 3);

p.end();

// create the mask
QBitmap mmask(9, 9);
p.begin(&mmask);
p.fillRect(0, 0, 9, 9, Qt::black);
p.translate(4, 4);
p.rotate(phi);
pen.setColor(Qt::white);
bru.setColor(Qt::white);
p.setPen(pen);
p.setBrush(bru);
p.drawPolygon(a);
p.end();

marker.setMask(mmask);

return(marker);
}

/*------------------------------------------------------------------------ */
void TelescopePath::setCurve(const QMemArray<double>& xdat,
                             const QMemArray<double>& ydat)
{
xdata.duplicate(xdat);
int crossed = 0;
for(unsigned i=1; i<xdat.size(); i++)
  {
  if(xdat[i]-xdat[i-1] > 12.0)
    crossed++;
  else if(xdat[i]-xdat[i-1] < -12.0)
    crossed--;
  switch(crossed)
    {
    case 1: // from low to high
      xdata[i] = 24.0 - xdata[i];
      break;
    case -1: // from high to low
      xdata[i] = 24.0 + xdata[i];
      break;
    default:
      break;
    }
  }
m_xMin   = minValue(xdata);
m_xMax   = maxValue(xdata);
m_xRange = m_xMax - m_xMin;
if(m_xRange < 0.001)
  {
  m_xMin   -= 0.001;
  m_xMax   += 0.001;
  m_xRange += 0.002;
  }

m_yData.duplicate(ydat);
m_yMin   = minValue(m_yData);
m_yMax   = maxValue(m_yData);
m_yRange = m_yMax - m_yMin;
if(m_yRange < 0.001)
  {
  m_yMin   -= 0.001;
  m_yMax   += 0.001;
  m_yRange += 0.002;
  }
}

/*------------------------------------------------------------------------ */
void TelescopePath::setLocation(SetiLoc* loc)
{
if(loc)
  {
  QStringList coords = loc->coords();
  QStringList::Iterator end(coords.end());
  QMemArray<double> ra(coords.count());
  QMemArray<double> dec(coords.count());
  int i=0;
  for(QStringList::Iterator it=coords.begin(); it!=end; ++it, ++i)
    {
    QStringList cline = QStringList::split(' ', *it);
    ra[i] = cline[1].toDouble();
    dec[i] = cline[2].toDouble();
    }
  setCurve(ra, dec);

  setName(loc->wuName());
  locColor = loc->color();
  QString text;
  text  = "RA : " + loc->convertRAToString(xdata[0]) +
          " - " + loc->convertRAToString(xdata[xdata.size()-1]) +
          " (";
  if(xdata[xdata.size()-1]-xdata[0] < 0.0)
    text += "-";
  text += loc->convertRAToString(fabs(xdata[xdata.size()-1]-xdata[0])) + ")\n";
  text += "Dec: " + loc->convertDecToString(m_yData[0]) +
          " - " + loc->convertDecToString(m_yData[m_yData.size()-1]) +
          " (";
  if(m_yData[m_yData.size()-1]-m_yData[0] < 0.0)
    text += "-";
  text += loc->convertDecToString(fabs(m_yData[m_yData.size()-1]-m_yData[0])) + ")\n" +
          "Angle Range: " + QString().setNum(loc->angleRange()) + "";
  setInfoText(text);
  update();
  }
}

#include "telescopepath.moc"
