/***************************************************************************/
/** @file     telescopepath.h
  * @brief    A which shows the telescope path for a particular WU
  *           (declaration).
  *
  * @b begin      : Thu Dec 12 2002<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef TELESCOPEPATH_H
#define TELESCOPEPATH_H

#include "setigraph.h"
#include "skymap.h"

/** Displays a window with the telescope path.
  *
  * This widget shows the telescope path of a specified location. The
  * telescope path is reconstructed from the coordinates given in the
  * work unit file @c work_unit.sah.
  */
class TelescopePath : public SetiGraph
{
Q_OBJECT

public: 
  /** The constructor.
    *
    * @param loc    The %S@h location for which the telescope path shall be
    *               shown.
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  TelescopePath(SetiLoc* loc, QWidget *parent=0, const char *name=0);
  /** The constructor.
    *
    * @overload
    *
    * @param pos    The SkyPos structure containig the telescope path data.
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  TelescopePath(SkyPos* pos, QWidget *parent=0, const char *name=0);
  /** The destructor. */
  ~TelescopePath();
  /** Shows the telescope path for location @a loc.
    *
    * @param loc    The %S@h location for which the telescope path shall be
    *               shown.
    */
  void setLocation(SetiLoc* loc);

protected: // methods
  /** Paints the widget.
    *
    * The curve is drawn in the specified mode, and optionally a legend with
    * basic info about the data is printed .
    *
    * @param pd A pointer to the object that shall be painted.
    */
  virtual void paintWidget(QPaintDevice* pd);
  /** Draws the curve.
    *
    * @param xd   The curve x data.
    * @param yd   The curve y data.
    * @param c    The color of the curve.
    * @param p    A pointer to an open painter object.
    */
  void drawCurve(const QMemArray<double>& xd, const QMemArray<double>& yd,
                 const QColor& c, QPainter* p);
  /** Sets the data of the graph.
    *
    * @param data A QArray of doubles representing the data of the signal.
    */
  void setCurve(const QMemArray<double>& xdat,
                const QMemArray<double>& ydat);
  /** Initializes the widget with the given color.
    *
    * @param col The color to draw the telescope path with.
    */
  void init(const QColor& col);
  /** Paints a marker indicating the direction of the telescope's move.
    *
    * @param x1 Horizontal position of the next coordinate.
    * @param y1 Vertical position of the next coordinate.
    * @param x2 Horizontal position of the coordinate for which the marker
    *           is painted.
    * @param y2 Vertical position of the coordinate for which the marker
    *           is painted.
    * @param c  The color of the marker.
    */
  QPixmap paintMarker(int x1, int y1, int x2, int y2,
                      const QColor& c);

protected: // attributes
  QColor locColor;
  QMemArray<double> xdata;
};

#endif
