/***************************************************************************/
/** @file     tripletplot.h
  * @brief    A widget that displays a triplet profile (declaration).
  *
  * @b begin      : Tue Oct 24 2000<br>
  * @b copyright  : &copy; 2000-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef TRIPLETPLOT_H
#define TRIPLETPLOT_H

#include "setigraph.h"
#include "setidefs.h"

/** Displays a triplet profile.
  *
  * This object can be constructed in two ways: either with or without
  * specifying a %S@h location. If a SetiLoc object is passed to the
  * constructor of this class, the triplet profile shown in the widget
  * always represents the best triplet the %S@h client has found in the
  * work unit it processes at that time. The triplet profile is
  * automatically updated whenever the client finds a new and better triplet
  * signal.
  *
  * If this widget is created without a %S@h location, the triplet data
  * have to be explicitly passed with the showNewTripletData() method.
  */
class TripletPlot : public SetiGraph
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  TripletPlot(QWidget *parent=0, const char *name=0);
  /** The constructor.
    *
    * @overload
    *
    * @param loc    The %S@h location for which the best triplet (found in
    *               the currently processed work unit) shall be shown.
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  TripletPlot(SetiLoc* loc, QWidget *parent=0, const char *name=0);
  /** The destructor. */
  ~TripletPlot();

public slots:
  /** Updates the widget with new triplet data.
    *
    * @param score A structure containing the parameters of the new triplet
    *              to be shown.
    */
  void showNewTripletData(TripletScore score);

protected: // methods
  /** Paints the widget.
    *
    * Reimplemented for customized painting of this widget (drawing green
    * markers indicating the triplet).
    *
    * @param pd A pointer to the object that shall be painted.
    */
  virtual void paintWidget(QPaintDevice* pd);
  /** Draws a marker.
    *
    * A green marker indicating the position of a spike of the triplet is
    * drawn at position @e index.
    *
    * @param index The position of the marker in the data array. An integer
    *              between 0 and 256.
    * @param p     A QPainter object open on the relevant paint device.
    */
  void drawPulseMarker(int index, QPainter* p);
  /** Initializes the widget.
    *
    * If a %S@h location is given, the triplet profile is painted, and a
    * connection is made to the SetiLoc::newTriplet signal of the
    * corresponding SetiLoc object.
    */
  void init();

protected: // attributes
  /** This array stores the triplet data. */
  QMemArray<double> tripletData;
  /** A structure containing the triplet parameters. */
  TripletScore triplet;
  /** Indicates if the data of a triplet is loaded. */
  bool dataLoaded;

};

#endif
