/***************************************************************************/
/** @file     userinfo.h
  * @brief    The User Info tab page and helper classes (declaration).
  *
  * @b begin      : Tue Oct 26 1999<br>
  * @b copyright  : &copy; 1999-2001,2002 by Gordon Machel<br>
  * @b email      : gmachel@users.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  **************************************************************************/
#ifndef USERINFO_H
#define USERINFO_H

#include <qwidget.h>
#include <qlistview.h>
#include <qdict.h>
#include "poplistview.h"

/**  A customized QListViewItem for the list view in the User Info tab.
  */
class UserInfoListViewItem : public ColorListViewItem
{
public:
  /** The constructor.
    *
    * @param parent The parent list view.
    */
  UserInfoListViewItem(QListView* parent) : ColorListViewItem(parent) {}
  /** The destructor. */
  ~UserInfoListViewItem() {}
  /** Reimplemented to allow customized sorting of the items in the list.
    *
    * @param column    The column for which a key is returned.
    * @param ascending Indicates the sorting order (usage not recommended).
    *
    * @return A key that can be used for sorting by column @a column.
    */
  virtual QString key(int column, bool ascending) const;
};

/** The widget for the User Info tab page.
  *
  * This widget contains a list view with information about the users
  * running the %S@h clients.
  */
class UserInfo : public QWidget
{
Q_OBJECT

public:
  /** The constructor.
    *
    * @param parent The parent widget. If 0, this widget becomes a top-level
    *               widget.
    * @param name   A text that can be used to identify this widget.
    */
  UserInfo(QWidget *parent=0, const char *name=0);
  /** The destructor. */
  ~UserInfo();

public slots:
  /** Refreshes all items in the list view.
    *
    * If corresponding list view item for a %S@h location does not exist,
    * it is created on-the-fly.
    */
  void refreshList();
  /** Updates the private dictionary of %S@h locations.
    *
    * Updates the private list of %S@h locations with the location @a loc. The
    * @a type parameter specifies what to do with @a loc.
    *
    * @param loc  The %S@h location to be updated.
    * @param type The type of action to be performed with @a loc. See
    *             Ksetiwatch::ListId for details.
    */
  void slotUpdateList(SetiLoc* loc, int type);

protected: // methods
  /** Resizes and rearranges widgets in the tab page when the window size
    * changes.
    *
    * A relict from ancient times when there were no layout widgets.
    *
    * @param e An object containing details about the resize event.
    */
  void resizeEvent(QResizeEvent* e);
  /** Reimplemented to tell the list view whether it shall draw grid lines.
    *
    * @param e An object containing details about the paint event.
    */
  void paintEvent(QPaintEvent* e);
  /** Refreshes all columns of the list view item of location @a loc.
    *
    * @param loc A pointer to the SetiLoc object.
    * @param it  A pointer to the list view item to be updated. If 0, the item
    *            will be looked up in the private dictionary.
    */
  void refreshListItem(SetiLoc* loc, UserInfoListViewItem* it=0);

protected slots:
  /** Determines the sender of the signal and refreshes the corresponding item
    * in the Data Info list.
    */
  void slotUpdateUserInfoData();
  /** Toggles the sorting order of column @a column. */
  void toggleSorting(int column);

protected: // attributes
  /** The list view shown in the Data Info tab page. */
  popListView* QListView_1;
  /** A dictionary that stores all the list view items currently shown in the
    * Data Info tab page. Items are identified by the unique ID of a %S@h
    * location. */
  QDict<UserInfoListViewItem> lvDict;
  bool sortingorder;
  /** Identifiers for the columns in the User Info list view. */
  enum ColumnIds {ColLocation, ColName, ColWuCompl, ColTotalTime,
                  ColAvrgTime, ColLastResult, ColRegTime};
};

#endif





